@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -216,6 +216,13 @@ public:
     int free_addr_by_range(PoolObjectSQL::ObjectType ot, int obid,
         const string& mac, unsigned int rsize);
 
+    /**
+     * Adds the relevant AR definition attributes to the Security Group rule
+     *
+     * @param rule rule to modify
+     */
+    void process_security_rule(VectorAttribute * rule);
+
     // *************************************************************************
     // Address Reservation
     // *************************************************************************
@@ -322,6 +329,15 @@ public:
             bool                keep_restricted,
             string&             error_msg);
 
+    /**
+     *  Get the security groups for this AR.
+     *    @return a reference to the security group set
+     */
+    const set<int>& get_security_groups() const
+    {
+        return security_groups;
+    }
+
     /*
      *  add_ar from AddressRangePool needs to access the internal representation
      *  of the AR to include it in the ARPool template.
@@ -342,6 +358,12 @@ private:
      */
     int mac_to_i(string mac, unsigned int i_mac[]) const;
 
+    /**
+     *  MAC to string
+     *    @param mac in array form
+     */
+    string mac_to_s(const unsigned int mac[]) const;
+
     /**
      *  IP version 4 to binary (32 bits)
      *    @param ip in string form 192.168.0.2
@@ -349,6 +371,14 @@ private:
      */
     int ip_to_i(const string& _ip, unsigned int& i_ip) const;
 
+    /**
+     * IP version 4 to dot notation
+     *
+     * @param i_ip Numeric (32 bits) IP
+     * @return dot notation
+     */
+    string ip_to_s(unsigned int i_ip) const;
+
     /**
      *  IPv6 64bits prefix conversion
      *    @param prefix in string form 2a00:1bc0:b001:A::
@@ -356,6 +386,15 @@ private:
      */
     int prefix6_to_i(const string& prefix, unsigned int ip[]) const;
 
+    /**
+     * IPv6 to string
+     * @param prefix Numeric IPv6 prefix
+     * @param mac Numeric (48 bits) mac address
+     * @param ip6_s Will contain the resulting IPv6 string
+     * @return 0 on success
+     */
+    int ip6_to_s(const unsigned int prefix[], const unsigned int mac[], string& ip6_s) const;
+
     /* ---------------------------------------------------------------------- */
     /* NIC setup functions                                                    */
     /* ---------------------------------------------------------------------- */
@@ -483,6 +522,11 @@ private:
      */
     unsigned int ula6[2];
 
+    /**
+     *  Security Group IDs for this Address Range
+     */
+    set<int> security_groups;
+
     /**
      *  The Address Range attributes as a Template VectorAttribute. This is
      *  used to generate XML or a template representation of the AR.
@@ -515,6 +559,13 @@ private:
      *  The restricted attributes from oned.conf
      */
     static set<string> restricted_attributes;
+
+    /**
+     *  Attributes to be process for Security Group rules
+     */
+    const static char * SG_RULE_ATTRIBUTES[];
+
+    const static int  NUM_SG_RULE_ATTRIBUTES;
 };
 
 #endif
