@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -161,12 +161,12 @@ public:
      */
     string * to_xml() const
     {
-    	string * xml = new string;
+        string * xml = new string;
 
-    	*xml = "<" + name() + "><![CDATA[" + attribute_value
-    		 + "]]></"+ name() + ">";
+        *xml = "<" + name() + "><![CDATA[" + attribute_value
+             + "]]></"+ name() + ">";
 
-    	return xml;
+        return xml;
     }
 
     /**
@@ -246,6 +246,15 @@ public:
      */
     string vector_value(const char *name) const;
 
+    /**
+     *  Returns the string value
+     *    @param name of the attribute
+     *    @param value of the value
+     *
+     *    @return 0 if the attribute was found, -1 otherwise
+     */
+    int vector_value(const char *name, string& value) const;
+
     /**
      *  Returns the boolean value
      *    @param name of the attribute
@@ -465,9 +474,9 @@ public:
 
 private:
 
-	static const char * magic_sep;
+    static const char * magic_sep;
 
-	static const int	magic_sep_size;
+    static const int    magic_sep_size;
 
     map<string,string> attribute_value;
 };
