@@ -1,5 +1,5 @@
 /* ------------------------------------------------------------------------ */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs      */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems              */
 /*                                                                          */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may  */
 /* not use this file except in compliance with the License. You may obtain  */
@@ -108,7 +108,7 @@ public:
      *  Returns a system DS for the cluster when none is set at the API level
      *    @return the ID of the System
      */
-    static int get_default_sysetm_ds(const set<int>& ds_collection);
+    static int get_default_system_ds(const set<int>& ds_collection);
 
     /**
      *  Adds this vnet ID to the set.
