@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -44,12 +44,6 @@ public:
      */
     static const int NONE_CLUSTER_ID;
 
-    /**
-     * Special ID to refer to all OpenNebula resources, from any cluster
-     * or in cluster none (* in ACL rules).
-     */
-    static const int ALL_RESOURCES;
-
     /* ---------------------------------------------------------------------- */
     /* Methods for DB management                                              */
     /* ---------------------------------------------------------------------- */
