@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -57,7 +57,13 @@ public:
         RESCHED_ACTION         = 17,
         UNRESCHED_ACTION       = 18,
         POWEROFF_ACTION        = 19,
-        POWEROFF_HARD_ACTION   = 20
+        POWEROFF_HARD_ACTION   = 20,
+        DISK_ATTACH_ACTION     = 21,
+        DISK_DETACH_ACTION     = 22,
+        NIC_ATTACH_ACTION      = 23,
+        NIC_DETACH_ACTION      = 24,
+        DISK_SNAPSHOT_CREATE_ACTION = 25,
+        DISK_SNAPSHOT_DELETE_ACTION = 26
     };
 
     static string action_to_str(VMAction action)
@@ -99,9 +105,6 @@ public:
             case RESUME_ACTION:
                 st = "resume";
             break;
-            case BOOT_ACTION:
-                st = "boot";
-            break;
             case DELETE_ACTION:
                 st = "delete";
             break;
@@ -126,7 +129,26 @@ public:
             case POWEROFF_HARD_ACTION:
                 st = "poweroff-hard";
             break;
+            case DISK_ATTACH_ACTION:
+                st = "disk-attach";
+            break;
+            case DISK_DETACH_ACTION:
+                st = "disk-detach";
+            break;
+            case NIC_ATTACH_ACTION:
+                st = "nic-attach";
+            break;
+            case NIC_DETACH_ACTION:
+                st = "nic-detach";
+            break;
+            case DISK_SNAPSHOT_CREATE_ACTION:
+                st = "snap-create";
+            break;
+            case DISK_SNAPSHOT_DELETE_ACTION:
+                st = "snap-delete";
+            break;
             case NONE_ACTION:
+            case BOOT_ACTION:
                 st = "none";
             break;
         }
@@ -180,10 +202,6 @@ public:
         {
             action = RESUME_ACTION;
         }
-        else if (st == "boot")
-        {
-            action = BOOT_ACTION;
-        }
         else if (st == "delete")
         {
             action = DELETE_ACTION;
@@ -216,7 +234,31 @@ public:
         {
             action = POWEROFF_HARD_ACTION;
         }
-        else
+        else if (st == "disk-attach")
+        {
+            action = DISK_ATTACH_ACTION;
+        }
+        else if (st == "disk-detach")
+        {
+            action = DISK_DETACH_ACTION;
+        }
+        else if (st == "nic-attach")
+        {
+            action = NIC_ATTACH_ACTION;
+        }
+        else if (st == "nic-detach")
+        {
+            action = NIC_DETACH_ACTION;
+        }
+        else if (st == "snap-create")
+        {
+            action = DISK_SNAPSHOT_CREATE_ACTION;
+        }
+        else if (st == "snap-delete")
+        {
+            action = DISK_SNAPSHOT_DELETE_ACTION;
+        }
+        else //BOOT_ACTION and others
         {
             action = NONE_ACTION;
             return -1;
