@@ -1,5 +1,5 @@
 /* ------------------------------------------------------------------------ */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs      */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems              */
 /*                                                                          */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may  */
 /* not use this file except in compliance with the License. You may obtain  */
@@ -379,13 +379,15 @@ public:
      *    @param cpu needed by the VM (percentage)
      *    @param mem needed by the VM (in KB)
      *    @param disk needed by the VM
+     *    @param pci devices needed by th VM
      *    @return 0 on success
      */
-    void add_capacity(int vm_id, long long cpu, long long mem, long long disk)
+    void add_capacity(int vm_id, long long cpu, long long mem, long long disk,
+            vector<Attribute *> pci)
     {
         if ( vm_collection.add_collection_id(vm_id) == 0 )
         {
-            host_share.add(cpu,mem,disk);
+            host_share.add(vm_id, cpu, mem, disk, pci);
         }
         else
         {
@@ -404,13 +406,15 @@ public:
      *    @param cpu used by the VM (percentage)
      *    @param mem used by the VM (in KB)
      *    @param disk used by the VM
+     *    @param pci devices needed by th VM
      *    @return 0 on success
      */
-    void del_capacity(int vm_id, long long cpu, long long mem, long long disk)
+    void del_capacity(int vm_id, long long cpu, long long mem, long long disk,
+            vector<Attribute *> pci)
     {
         if ( vm_collection.del_collection_id(vm_id) == 0 )
         {
-            host_share.del(cpu,mem,disk);
+            host_share.del(cpu, mem, disk, pci);
         }
         else
         {
@@ -440,11 +444,26 @@ public:
      *    @param cpu needed by the VM (percentage)
      *    @param mem needed by the VM (in Kb)
      *    @param disk needed by the VM
+     *    @param pci devices needed by the VM
+     *    @param error Returns the error reason, if any
      *    @return true if the share can host the VM
      */
-    bool test_capacity(long long cpu, long long mem, long long disk)
+    bool test_capacity(long long cpu, long long mem, long long disk,
+                       vector<Attribute *> &pci, string& error) const
     {
-        return host_share.test(cpu, mem, disk);
+        return host_share.test(cpu, mem, disk, pci, error);
+    }
+
+    /**
+     *  Tests whether a new VM can be hosted by the host or not, checking the
+     *  PCI devices only.
+     *    @param pci devices needed by the VM
+     *    @param error Returns the error reason, if any
+     *    @return true if the share can host the VM
+     */
+    bool test_capacity(vector<Attribute *> &pci, string& error) const
+    {
+        return host_share.test(pci, error);
     }
 
     /**
@@ -470,6 +489,26 @@ public:
      */
     int post_update_template(string& error);
 
+    /**
+     * Returns the rediscovered VMs (from poff to running) in the previous
+     * monitorization cycle
+     * @return the previous rediscovered VMs (from poff to running)
+     */
+    const set<int>& get_prev_rediscovered_vms() const
+    {
+        return prev_rediscovered_vms;
+    }
+
+    /**
+     * Sets the previous rediscovered VMs (from poff to running). This set
+     * is not stored in the DB, the pool update method is not needed
+     * @param rediscovered_vms the previous rediscovered VMs (from poff to running)
+     */
+    void set_prev_rediscovered_vms(const set<int>& rediscovered_vms)
+    {
+        prev_rediscovered_vms = rediscovered_vms;
+    }
+
 private:
 
     // -------------------------------------------------------------------------
@@ -486,22 +525,22 @@ private:
      */
     HostState   state;
 
-	/**
+    /**
      *  Name of the IM driver used to monitor this host
      */
-	string      im_mad_name;
+    string      im_mad_name;
 
-	/**
+    /**
      *  Name of the VM driver used to execute VMs in this host
      */
-	string      vmm_mad_name;
+    string      vmm_mad_name;
 
     /**
      *  Name of the VN driver used to manage networking in this host
      */
     string      vnm_mad_name;
 
-	/**
+    /**
      *  If Host State = MONITORED last time it got fully monitored or 1 Jan 1970
      *     Host State = MONITORING* last time it got a signal to be monitored
      */
@@ -527,6 +566,13 @@ private:
      */
     set<int>        tmp_zombie_vms;
 
+    /**
+     * Set that stores the VMs reported as found from the poweroff state. This
+     * is managed from outside the host to avoid deadlocks, as the current
+     * VM state is needed
+     */
+    set<int>        prev_rediscovered_vms;
+
     // -------------------------------------------------------------------------
     //  VM Collection
     // -------------------------------------------------------------------------
