@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -29,6 +29,7 @@
 #include <vector>
 
 class AuthRequest;
+class Snapshots;
 
 using namespace std;
 
@@ -149,7 +150,8 @@ public:
     }
 
     /**
-     *  Generates a DISK attribute for VM templates using the Image metadata
+     *  Generates a DISK attribute for VM templates using the Image metadata.
+     *  If the disk uses an Image, it tries to acquire it.
      *
      *    @param disk the disk to be generated
      *    @param disk_id the id for this disk
@@ -158,18 +160,33 @@ public:
      *        or the default one
      *    @param uid of VM owner (to look for the image id within its images)
      *    @param image_id on success returns the acquired image id
+     *    @param snaps list of snapshots associated to this image
      *    @param error_str string describing the error
      *
      *    @return 0 on success, -1 otherwise
      */
-    int disk_attribute(int                vm_id,
+    int acquire_disk(  int                vm_id,
                        VectorAttribute *  disk,
                        int                disk_id,
                        Image::ImageType&  img_type,
                        string&            dev_prefix,
                        int                uid,
                        int&               image_id,
+                       Snapshots **       snaps,
                        string&            error_str);
+
+    /**
+     *  Generates a DISK attribute for VM templates using the Image metadata
+     *
+     *    @param disk the disk to be generated
+     *    @param disk_id the id for this disk
+     *    @param uid of VM owner (to look for the image id within its images)
+     *
+     */
+    void disk_attribute(VectorAttribute *   disk,
+                        int                 disk_id,
+                        int                 uid);
+
     /**
      *  Generates an Authorization token for the DISK attribute
      *    @param disk the disk to be authorized
