@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -44,6 +44,10 @@ private:
 
     static const float  CGROUP_BASE_CPU_SHARES;
 
+    static const int CEPH_DEFAULT_PORT;
+
+    static const int GLUSTER_DEFAULT_PORT;
+
     int deployment_description(
         const VirtualMachine *  vm,
         const string&           file_name) const
