@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -20,11 +20,18 @@
 #include "ActionManager.h"
 #include "VirtualMachinePool.h"
 #include "HostPool.h"
+#include "ImagePool.h"
 
 using namespace std;
 
 extern "C" void * lcm_action_loop(void *arg);
 
+//Forward definitions
+class TransferManager;
+class DispatchManager;
+class VirtualMachineManager;
+class ImageManager;
+
 /**
  *  The Virtual Machine Life-cycle Manager module. This class is responsible for
  *  managing the life-cycle of a Virtual Machine.
@@ -33,8 +40,8 @@ class LifeCycleManager : public ActionListener
 {
 public:
 
-    LifeCycleManager(VirtualMachinePool * _vmpool, HostPool * _hpool):
-        vmpool(_vmpool),hpool(_hpool)
+    LifeCycleManager():
+        vmpool(0), hpool(0), ipool(0), tm(0), vmm(0), dm(0), imagem(0)
     {
         am.addListener(this);
     };
@@ -51,7 +58,6 @@ public:
         SHUTDOWN_FAILURE, /**< Sent by the VMM when a shutdown action fails   */
         CANCEL_SUCCESS,   /**< Sent by the VMM when a cancel action succeeds  */
         CANCEL_FAILURE,   /**< Sent by the VMM when a cancel action fails     */
-        MONITOR_FAILURE,  /**< Sent by the VMM when a VM has failed while active */
         MONITOR_SUSPEND,  /**< Sent by the VMM when a VM is paused while active */
         MONITOR_DONE,     /**< Sent by the VMM when a Host cannot be monitored*/
         MONITOR_POWEROFF, /**< Sent by the VMM when a VM is not found */
@@ -70,14 +76,16 @@ public:
         DETACH_NIC_FAILURE,/**< Sent by the VMM when a detach nic action fails     */
         CLEANUP_SUCCESS,  /**< Sent by the VMM when a cleanup action succeeds */
         CLEANUP_FAILURE,  /**< Sent by the VMM when a cleanup action fails    */
-        SAVEAS_HOT_SUCCESS,/**< Sent by the VMM when hot saveas succeeds      */
-        SAVEAS_HOT_FAILURE,/**< Sent by the VMM when hot saveas fails         */
+        SAVEAS_SUCCESS,        /**< Sent by the VMM when saveas succeeds      */
+        SAVEAS_FAILURE,        /**< Sent by the VMM when saveas fails         */
         SNAPSHOT_CREATE_SUCCESS, /**< Sent by the VMM on snap. create success */
         SNAPSHOT_CREATE_FAILURE, /**< Sent by the VMM on snap. create failure */
         SNAPSHOT_REVERT_SUCCESS, /**< Sent by the VMM on snap. revert success */
         SNAPSHOT_REVERT_FAILURE, /**< Sent by the VMM on snap. revert failure */
         SNAPSHOT_DELETE_SUCCESS, /**< Sent by the VMM on snap. revert success */
         SNAPSHOT_DELETE_FAILURE, /**< Sent by the VMM on snap. revert failure */
+        DISK_SNAPSHOT_SUCCESS, /**<Sent by TM when a snap. succeeds */
+        DISK_SNAPSHOT_FAILURE, /**<Sent by TM when a snap. fails */
         DEPLOY,           /**< Sent by the DM to deploy a VM on a host        */
         SUSPEND,          /**< Sent by the DM to suspend an running VM        */
         RESTORE,          /**< Sent by the DM to restore a suspended VM       */
@@ -114,6 +122,12 @@ public:
      */
     int start();
 
+    /**
+     * Initializes internal pointers to other managers. Must be called when
+     * all the other managers exist in Nebula::instance
+     */
+    void init_managers();
+
     /**
      *  Gets the thread identification.
      *    @return pthread_t for the manager thread (that in the action loop).
@@ -130,6 +144,12 @@ public:
      */
     void  recover(VirtualMachine * vm, bool success);
 
+    /**
+     *  Retries the last VM operation that lead to a failure. The underlying
+     *  driver actions may be invoked and should be "re-entrant".
+     */
+    void retry(VirtualMachine * vm);
+
 private:
     /**
      *  Thread id for the Virtual Machine Manager
@@ -146,11 +166,37 @@ private:
      */
     HostPool *              hpool;
 
+    /**
+     *  Pointer to the Image Pool, to access images
+     */
+    ImagePool *             ipool;
+
+    /**
+     * Pointer to TransferManager
+     */
+    TransferManager *       tm;
+
+    /**
+     * Pointer to VirtualMachineManager
+     */
+    VirtualMachineManager * vmm;
+
+    /**
+     * Pointer to DispatchManager
+     */
+    DispatchManager *       dm;
+
     /**
      *  Action engine for the Manager
      */
     ActionManager           am;
 
+    /**
+     * Pointer to ImageManager
+     */
+    ImageManager *          imagem;
+
+
     /**
      *  Function to execute the Manager action loop method within a new pthread
      * (requires C linkage)
@@ -189,12 +235,6 @@ private:
 
     void shutdown_failure_action(int vid);
 
-    void cancel_success_action(int vid);
-
-    void cancel_failure_action(int vid);
-
-    void monitor_failure_action(int vid);
-
     void monitor_suspend_action(int vid);
 
     void monitor_done_action(int vid);
@@ -213,15 +253,15 @@ private:
 
     void attach_success_action(int vid);
 
-    void attach_failure_action(int vid, bool release_save_as);
+    void attach_failure_action(int vid);
 
     void detach_success_action(int vid);
 
     void detach_failure_action(int vid);
 
-    void saveas_hot_success_action(int vid);
+    void saveas_success_action(int vid);
 
-    void saveas_hot_failure_action(int vid);
+    void saveas_failure_action(int vid);
 
     void attach_nic_success_action(int vid);
 
@@ -245,6 +285,10 @@ private:
 
     void snapshot_delete_failure(int vid);
 
+    void disk_snapshot_success(int vid);
+
+    void disk_snapshot_failure(int vid);
+
     void deploy_action(int vid);
 
     void suspend_action(int vid);
@@ -253,15 +297,13 @@ private:
 
     void stop_action(int vid);
 
-    void cancel_action(int vid);
-
     void checkpoint_action(int vid);
 
     void migrate_action(int vid);
 
     void live_migrate_action(int vid);
 
-    void shutdown_action(int vid);
+    void shutdown_action(int vid, bool hard);
 
     void undeploy_action(int vid, bool hard);
 
@@ -271,8 +313,6 @@ private:
 
     void poweroff_action(int vid, bool hard);
 
-    void failure_action(VirtualMachine * vm);
-
     void restart_action(int vid);
 
     void delete_action(int vid);
