@@ -1,6 +1,6 @@
 
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -25,6 +25,7 @@ class HostPool;
 class ClusterPool;
 class DatastorePool;
 class LifeCycleManager;
+class VirtualMachineManager;
 class VirtualMachinePool;
 
 class MonitorThreadPool;
@@ -61,6 +62,8 @@ private:
 
     static LifeCycleManager *lcm;
 
+    static VirtualMachineManager *vmm;
+
     static VirtualMachinePool * vmpool;
 
     static MonitorThreadPool * mthpool;
