@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -60,7 +60,9 @@ public:
         DATASTORE   = 0x0000100000000000LL,
         CLUSTER     = 0x0000200000000000LL,
         DOCUMENT    = 0x0000400000000000LL,
-        ZONE        = 0x0000800000000000LL
+        ZONE        = 0x0000800000000000LL,
+        SECGROUP    = 0x0001000000000000LL,
+        VDC         = 0x0002000000000000LL
     };
 
     static string type_to_str(ObjectType ob)
@@ -79,6 +81,8 @@ public:
             case CLUSTER:   return "CLUSTER" ; break;
             case DOCUMENT:  return "DOCUMENT" ; break;
             case ZONE:      return "ZONE" ; break;
+            case SECGROUP:  return "SECGROUP" ; break;
+            case VDC:       return "VDC" ; break;
             default:        return "";
         }
     };
@@ -113,6 +117,9 @@ public:
              other_m(0),
              other_a(0),
              obj_template(0),
+             locked(false),
+             lock_owner(""),
+             lock_expires(0),
              table(_table)
     {
         pthread_mutex_init(&mutex,0);
@@ -499,7 +506,7 @@ public:
     }
 
     /**
-     *  Removes an Image attribute
+     *  Removes an attribute
      *    @param name of the attribute
      */
     int remove_template_attribute(const string& name)
@@ -587,6 +594,25 @@ public:
      */
     virtual void get_permissions(PoolObjectAuth& auths);
 
+    /**
+     * Tries to get the DB lock. This is a mutex requested by external
+     * applications, not related to the internal mutex lock. The object
+     * must be locked (internal memory mutex) before this method is called
+     *
+     * @param owner String to identify who requested the lock
+     *
+     * @return 0 if the lock was granted, -1 if the object is already locked
+     */
+    int lock_db(const string& owner);
+
+    /**
+     * Unlocks the DB lock for external applications. The object must be locked
+     * (internal memory mutex) before this method is called
+     *
+     * @param owner String to identify who requested the lock
+     */
+    void unlock_db(const string& owner);
+
 protected:
 
     /**
@@ -703,6 +729,21 @@ protected:
         return 0;
     };
 
+    /**
+     * Prints the lock info into a string in XML format
+     *  @param xml the resulting XML string
+     *  @return a reference to the generated string
+     */
+    string& lock_db_to_xml(string& xml) const;
+
+    /**
+     *  Rebuilds the lock info from the xml. ObjectXML::update_from_str
+     *  must be called before this method
+     *
+     *    @return 0 on success, -1 otherwise
+     */
+    int lock_db_from_xml();
+
     /**
      *  The object's unique ID
      */
@@ -769,12 +810,32 @@ protected:
      */
     Template * obj_template;
 
+    /**
+     *  Flag for the DB lock
+     */
+    bool    locked;
+
+    /**
+     *  Owner of the DB lock
+     */
+    string  lock_owner;
+
+    /**
+     *  Expiration time for the DB lock
+     */
+    time_t  lock_expires;
+
 private:
     /**
      *  Characters that can not be in a name
      */
     static const string INVALID_NAME_CHARS;
 
+    /**
+     * Expiration time for the lock stored in the DB
+     */
+    static const int LOCK_DB_EXPIRATION;
+
     /**
      *  The PoolSQL, friend to easily manipulate its Objects
      */
