@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -207,15 +207,6 @@ protected:
      */
     void cleanup_quota(const string& qid);
 
-    /**
-     * Creates a string from the given float, using fixed notation. If the
-     * number has any decimals, they will be truncated to 2.
-     *
-     * @param num
-     * @return
-     */
-    string float_to_str(const float &num);
-
 private:
     /**
      *  Creates an empty quota based on the given attribute. The attribute va
