@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -137,6 +137,45 @@ public:
     };
 };
 
+/* ------------------------------------------------------------------------- */
+/* ------------------------------------------------------------------------- */
+
+class SecurityGroupClone : public RequestManagerClone
+{
+public:
+    SecurityGroupClone():
+        RequestManagerClone("SecurityGroupClone",
+                            "Clone an existing security group")
+    {
+        Nebula& nd  = Nebula::instance();
+        pool        = nd.get_secgrouppool();
+
+        auth_object = PoolObjectSQL::SECGROUP;
+        auth_op     = AuthRequest::USE;
+    };
+
+    ~SecurityGroupClone(){};
+
+    /* -------------------------------------------------------------------- */
+
+    Template * clone_template(PoolObjectSQL* obj)
+    {
+        return static_cast<SecurityGroup*>(obj)->clone_template();
+    };
+
+    int pool_allocate(
+            int                         source_id,
+            Template *                  tmpl,
+            int&                        id,
+            string&                     error_str,
+            RequestAttributes&          att)
+    {
+        SecurityGroupPool * secgrouppool = static_cast<SecurityGroupPool *>(pool);
+
+        return secgrouppool->allocate(att.uid, att.gid, att.uname, att.gname,
+            att.umask, tmpl, &id, error_str);
+    };
+};
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
