@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -54,6 +54,7 @@ protected:
                           RequestAttributes&      att,
                           PoolObjectAuth *        host_perms,
                           PoolObjectAuth *        ds_perm,
+                          PoolObjectAuth *        img_perm,
                           AuthRequest::Operation  op);
 
     bool quota_resize_authorization(
@@ -89,10 +90,9 @@ protected:
         string& tm_mad,
         RequestAttributes& att);
 
-    bool check_host(int     hid,
-                    int     cpu,
-                    int     mem,
-                    int     disk,
+    bool check_host(int hid,
+                    bool enforce,
+                    VirtualMachine* vm,
                     string& error);
 
     int add_history(VirtualMachine * vm,
@@ -156,7 +156,7 @@ public:
     VirtualMachineMigrate():
         RequestManagerVirtualMachine("VirtualMachineMigrate",
                                      "Migrates a virtual machine",
-                                     "A:siibb")
+                                     "A:siibbi")
     {
          auth_op = AuthRequest::ADMIN;
     };
@@ -170,15 +170,15 @@ public:
 /* ------------------------------------------------------------------------- */
 /* ------------------------------------------------------------------------- */
 
-class VirtualMachineSaveDisk : public RequestManagerVirtualMachine
+class VirtualMachineDiskSaveas : public RequestManagerVirtualMachine
 {
 public:
-    VirtualMachineSaveDisk():
-        RequestManagerVirtualMachine("VirtualMachineSaveDisk",
-                           "Saves a disk from virtual machine as a new image",
-                           "A:siissbb"){};
+    VirtualMachineDiskSaveas():
+        RequestManagerVirtualMachine("VirtualMachineDiskSaveas",
+                           "Save a disk from virtual machine as a new image",
+                           "A:siissi"){};
 
-    ~VirtualMachineSaveDisk(){};
+    ~VirtualMachineDiskSaveas(){};
 
     void request_execute(xmlrpc_c::paramList const& _paramList,
             RequestAttributes& att);
@@ -349,7 +349,7 @@ public:
     VirtualMachineRecover():
         RequestManagerVirtualMachine("VirtualMachineRecover",
                                      "Recovers a virtual machine",
-                                     "A:sib")
+                                     "A:sii")
     {
          auth_op = AuthRequest::ADMIN;
     };
@@ -363,4 +363,89 @@ public:
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+class VirtualMachinePoolCalculateShowback : public RequestManagerVirtualMachine
+{
+public:
+
+    VirtualMachinePoolCalculateShowback():
+        RequestManagerVirtualMachine("VirtualMachinePoolCalculateShowback",
+                                     "Processes all the history records, and stores the monthly cost for each VM",
+                                     "A:sii")
+    {
+        Nebula& nd  = Nebula::instance();
+        pool        = nd.get_vmpool();
+        auth_object = PoolObjectSQL::VM;
+    };
+
+    ~VirtualMachinePoolCalculateShowback(){};
+
+    /* -------------------------------------------------------------------- */
+
+    void request_execute(
+            xmlrpc_c::paramList const& paramList, RequestAttributes& att);
+};
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+class VirtualMachineDiskSnapshotCreate: public RequestManagerVirtualMachine
+{
+public:
+    VirtualMachineDiskSnapshotCreate():
+        RequestManagerVirtualMachine("VirtualMachineDiskSnapshotCreate",
+                           "Creates a new virtual machine disk snapshot",
+                           "A:siis"){
+        Nebula& nd  = Nebula::instance();
+        ipool       = nd.get_ipool();
+    };
+
+    ~VirtualMachineDiskSnapshotCreate(){};
+
+    void request_execute(xmlrpc_c::paramList const& _paramList,
+            RequestAttributes& att);
+
+private:
+    ImagePool* ipool;
+};
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+class VirtualMachineDiskSnapshotRevert: public RequestManagerVirtualMachine
+{
+public:
+    VirtualMachineDiskSnapshotRevert():
+        RequestManagerVirtualMachine("VirtualMachineDiskSnapshotRevert",
+                           "Reverts disk state to a snapshot",
+                           "A:siii"){};
+
+    ~VirtualMachineDiskSnapshotRevert(){};
+
+    void request_execute(xmlrpc_c::paramList const& _paramList,
+            RequestAttributes& att);
+};
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+class VirtualMachineDiskSnapshotDelete: public RequestManagerVirtualMachine
+{
+public:
+    VirtualMachineDiskSnapshotDelete():
+        RequestManagerVirtualMachine("VirtualMachineDiskSnapshotDelete",
+                           "Deletes a disk snapshot",
+                           "A:siii"){
+        Nebula& nd  = Nebula::instance();
+        ipool       = nd.get_ipool();
+    };
+
+    ~VirtualMachineDiskSnapshotDelete(){};
+
+    void request_execute(xmlrpc_c::paramList const& _paramList,
+            RequestAttributes& att);
+
+private:
+    ImagePool* ipool;
+};
+
 #endif
