@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -55,12 +55,35 @@ public:
         separator    = t.separator;
         xml_root     = t.xml_root;
 
+        attributes.clear();
+
         for (it = t.attributes.begin() ; it != t.attributes.end() ; it++)
         {
             attributes.insert(make_pair(it->first,(it->second)->clone()));
         }
     }
 
+    Template& operator=(const Template& t)
+    {
+        multimap<string,Attribute *>::const_iterator it;
+
+        if (this != &t)
+        {
+            replace_mode = t.replace_mode;
+            separator    = t.separator;
+            xml_root     = t.xml_root;
+
+            attributes.clear();
+
+            for (it = t.attributes.begin() ; it != t.attributes.end() ; it++)
+            {
+                attributes.insert(make_pair(it->first,(it->second)->clone()));
+            }
+        }
+
+        return *this;
+    }
+
     /**
      *  The class destructor frees all the attributes conforming the template
      */
@@ -117,7 +140,7 @@ public:
      *  The name of the root element is set when the Template object is created
      *    @param xml string that hold the xml template representation
      *    @return a reference to the generated string
-	 */
+     */
     string& to_xml(string& xml) const;
 
     /**
@@ -127,6 +150,11 @@ public:
      */
     string& to_str(string& str) const;
 
+    /**
+     *  Clears all the attributes from the template
+     */
+    void clear();
+
     /**
      *  Sets a new attribute, the attribute MUST BE ALLOCATED IN THE HEAP, and
      *  will be freed when the template destructor is called.
@@ -460,17 +488,17 @@ private:
     /**
      *  Mutex to perform just one flex-bison parsing at a time
      */
-    static pthread_mutex_t			mutex;
+    static pthread_mutex_t          mutex;
 
     /**
      * Character to separate key from value when dump onto a string
      **/
-    char							separator;
+    char                            separator;
 
     /**
      *  Name of the Root element for the XML document
      */
-    string							xml_root;
+    string                          xml_root;
 
     /**
      *  Builds the template attribute from the node
