@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -91,12 +91,12 @@ protected:
      *    @param value of the attribute
      */
     void get_default(
-    	const char *  name,
+        const char *  name,
         string&       value) const
     {
-    	string sn = name;
+        string sn = name;
 
-    	driver_conf.get(sn,value);
+        driver_conf.get(sn,value);
     }
 
     /**
@@ -107,8 +107,8 @@ protected:
      *    @param value of the attribute
      */
     void get_default(
-    	const char *  name,
-    	const char *  vname,
+        const char *  name,
+        const char *  vname,
         string&       value) const;
 
     /**
@@ -126,10 +126,10 @@ protected:
         bool&         value) const;
 
 private:
-	/**
-	 *  Configuration file for the driver
-	 */
-	Template	driver_conf;
+    /**
+     *  Configuration file for the driver
+     */
+    Template    driver_conf;
 
     /**
      *  Pointer to the Virtual Machine Pool, to access VMs
@@ -361,23 +361,30 @@ private:
     }
 
     /**
-     *  Gets VM information from the driver answer
-     *    @param monitor_str from the driver
-     *    @param memory Kilobytes used by the VM (total)
-     *    @param cpu used by the VM (rate)
-     *    @param net_tx transmitted bytes (total)
-     *    @param net_rx received bytes (total)
-     *    @param state of the vm
-     *    @param custom monitor information
+     *  Sends a disk snapshot create request to the MAD:
+     *  "DISKSNAPSHOTCREATE ID XML_DRV_MSG"
+     *    @param oid the virtual machine id.
+     *    @param drv_msg xml data for the mad operation
      */
-    static int parse_vm_info(
-        const string&   monitor_str,
-        int                &cpu,
-        int                &memory,
-        long long          &net_tx,
-        long long          &net_rx,
-        char               &state,
-        map<string,string> &custom);
+    void disk_snapshot_create (
+        const int     oid,
+        const string& drv_msg) const
+    {
+        write_drv("DISKSNAPSHOTCREATE", oid, drv_msg);
+    }
+
+    /**
+     *  Sends a disk snapshot revert request to the MAD:
+     *  "DISKSNAPSHOTREVERT ID XML_DRV_MSG"
+     *    @param oid the virtual machine id.
+     *    @param drv_msg xml data for the mad operation
+     */
+    void disk_snapshot_revert (
+        const int     oid,
+        const string& drv_msg) const
+    {
+        write_drv("DISKSNAPSHOTREVERT", oid, drv_msg);
+    }
 
     /**
      *
