@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -255,6 +255,18 @@ public:
             int                     vid,
             const vector<string>&   inherit_attrs);
 
+    /**
+     * From a Security Group rule that uses this vnet, creates a new rule
+     * copy for each AR.
+     *
+     * @param rule original rule
+     * @param new_rules vector where the new rules will be placed. Rules must
+     * be deleted by the caller
+     */
+    void process_security_rule(
+            VectorAttribute *        rule,
+            vector<VectorAttribute*> &new_rules);
+
     // *************************************************************************
     // Network Reservation functions
     // *************************************************************************
@@ -436,6 +448,11 @@ private:
      */
     int     parent_vid;
 
+    /**
+     *  Security Groups
+     */
+    set<int> security_groups;
+
     /**
      *  The Address Range Pool
      */
