@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -28,7 +28,7 @@ const long long AclRule::CLUSTER_ID     = 0x0000000800000000LL;
 
 const long long AclRule::NONE_ID        = 0x1000000000000000LL;
 
-const int AclRule::num_pool_objects = 11;
+const int AclRule::num_pool_objects = 13;
 const PoolObjectSQL::ObjectType AclRule::pool_objects[] = {
             PoolObjectSQL::VM,
             PoolObjectSQL::HOST,
@@ -40,7 +40,9 @@ const PoolObjectSQL::ObjectType AclRule::pool_objects[] = {
             PoolObjectSQL::DATASTORE,
             PoolObjectSQL::CLUSTER,
             PoolObjectSQL::DOCUMENT,
-            PoolObjectSQL::ZONE
+            PoolObjectSQL::ZONE,
+            PoolObjectSQL::SECGROUP,
+            PoolObjectSQL::VDC
 };
 
 const int AclRule::num_auth_operations = 4;
@@ -54,15 +56,16 @@ const AuthRequest::Operation AclRule::auth_operations[] = {
 const long long AclRule::INVALID_CLUSTER_OBJECTS =
         PoolObjectSQL::VM | PoolObjectSQL::IMAGE | PoolObjectSQL::USER |
         PoolObjectSQL::TEMPLATE | PoolObjectSQL::GROUP | PoolObjectSQL::ACL |
-        PoolObjectSQL::CLUSTER | PoolObjectSQL::DOCUMENT | PoolObjectSQL::ZONE;
+        PoolObjectSQL::CLUSTER | PoolObjectSQL::DOCUMENT | PoolObjectSQL::ZONE |
+        PoolObjectSQL::SECGROUP | PoolObjectSQL::VDC;
 
 const long long AclRule::INVALID_GROUP_OBJECTS =
         PoolObjectSQL::HOST | PoolObjectSQL::GROUP | PoolObjectSQL::CLUSTER |
-        PoolObjectSQL::ZONE;
+        PoolObjectSQL::ZONE | PoolObjectSQL::VDC;
 
 const long long AclRule::FEDERATED_OBJECTS =
         PoolObjectSQL::USER | PoolObjectSQL::GROUP | PoolObjectSQL::ZONE |
-        PoolObjectSQL::ACL;
+        PoolObjectSQL::ACL | PoolObjectSQL::VDC;
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
@@ -181,8 +184,9 @@ bool AclRule::malformed(string& error_str) const
         oss << "[resource] GROUP(@) selector cannot be applied to "
             << PoolObjectSQL::type_to_str(PoolObjectSQL::HOST) << ", "
             << PoolObjectSQL::type_to_str(PoolObjectSQL::GROUP) << ", "
-            << PoolObjectSQL::type_to_str(PoolObjectSQL::CLUSTER) << " or "
-            << PoolObjectSQL::type_to_str(PoolObjectSQL::ZONE) << " types";
+            << PoolObjectSQL::type_to_str(PoolObjectSQL::CLUSTER) << ", "
+            << PoolObjectSQL::type_to_str(PoolObjectSQL::ZONE) << " or "
+            << PoolObjectSQL::type_to_str(PoolObjectSQL::VDC) << " types";
     }
 
     if ( (resource & 0xF00000000LL) == 0 )
@@ -218,7 +222,7 @@ bool AclRule::malformed(string& error_str) const
         oss << "when using the ALL bit, [resource] ID must be 0";
     }
 
-    if ( (resource & 0xFFF000000000LL) == 0 )
+    if ( (resource & 0xFFFF000000000LL) == 0 )
     {
         if ( error )
         {
@@ -229,7 +233,7 @@ bool AclRule::malformed(string& error_str) const
         oss << "[resource] type is missing";
     }
 
-    if ( (resource & 0xFFFF000000000000LL) != 0 )
+    if ( (resource & 0xFFFC000000000000LL) != 0 )
     {
         if ( error )
         {
