@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -18,19 +18,35 @@ require 'one_helper'
 require 'optparse/time'
 
 class AcctHelper < OpenNebulaHelper::OneHelper
-    START_TIME = {
+    START_TIME_ACCT = {
         :name   => "start_time",
         :short  => "-s TIME",
         :large  => "--start TIME" ,
-        :description => "Start date and time to take into account",
+        :description => "First day of the data to retrieve",
         :format => Time
     }
 
-    END_TIME = {
+    END_TIME_ACCT = {
         :name   => "end_time",
         :short  => "-e TIME",
         :large  => "--end TIME" ,
-        :description => "End date and time",
+        :description => "Last day of the data to retrieve",
+        :format => Time
+    }
+
+    START_TIME_SHOWBACK = {
+        :name   => "start_time",
+        :short  => "-s TIME",
+        :large  => "--start TIME" ,
+        :description => "First month of the data",
+        :format => Time
+    }
+
+    END_TIME_SHOWBACK = {
+        :name   => "end_time",
+        :short  => "-e TIME",
+        :large  => "--end TIME" ,
+        :description => "Last month of the data",
         :format => Time
     }
 
@@ -95,10 +111,10 @@ class AcctHelper < OpenNebulaHelper::OneHelper
         :description => "Split the output in a table for each VM"
     }
 
-    ACCT_OPTIONS = [START_TIME, END_TIME, USERFILTER, GROUP, HOST, XPATH, XML, JSON, SPLIT]
+    ACCT_OPTIONS     = [START_TIME_ACCT, END_TIME_ACCT, USERFILTER, GROUP, HOST, XPATH, XML, JSON, SPLIT]
+    SHOWBACK_OPTIONS = [START_TIME_SHOWBACK, END_TIME_SHOWBACK, USERFILTER, GROUP, XML, JSON]
 
-
-    ACCT_TABLE = CLIHelper::ShowTable.new("oneacct.yaml", nil) do
+    ACCT_TABLE = CLIHelper::ShowTable.new(self.table_conf("oneacct.yaml"), nil) do
         column :UID, "User ID", :size=>4 do |d|
             d["UID"]
         end
@@ -139,17 +155,83 @@ class AcctHelper < OpenNebulaHelper::OneHelper
             d["VM"]["TEMPLATE"]["CPU"]
         end
 
-        column :NET_RX, "Data received from the network", :size=>6 do |d|
+        column :NETRX, "Data received from the network", :size=>6 do |d|
             # NET is measured in bytes, unit_to_str expects KBytes
-            OpenNebulaHelper.unit_to_str(d["VM"]["NET_RX"].to_i / 1024.0, {})
+            OpenNebulaHelper.unit_to_str(d["VM"]["MONITORING/NETRX"].to_i / 1024.0, {})
         end
 
-        column :NET_TX, "Data sent to the network", :size=>6 do |d|
+        column :NETTX, "Data sent to the network", :size=>6 do |d|
             # NET is measured in bytes, unit_to_str expects KBytes
-            OpenNebulaHelper.unit_to_str(d["VM"]["NET_TX"].to_i / 1024.0, {})
+            OpenNebulaHelper.unit_to_str(d["VM"]["MONITORING/NETTX"].to_i / 1024.0, {})
         end
 
-        default :VID, :HOSTNAME, :ACTION, :REASON, :START_TIME, :END_TIME, :MEMORY, :CPU, :NET_RX, :NET_TX
+        column :DISK, "Total disk size used", :size=>6 do |d|
+            # DISK size is measured in mb, unit_to_str expects KBytes
+            total_disk_size = 0
+
+            vm_id = d["VM"]["ID"].to_i
+
+            disks_all = [d["VM"]["TEMPLATE"]["DISK"]].flatten.compact rescue []
+            disks_all.each do |disk|
+                total_disk_size += disk["SIZE"].to_i
+            end
+
+            snapshots_all = [d["VM"]["SNAPSHOTS"]].flatten.compact rescue []
+            snapshots_all.each do |snapshot|
+                snapshot_disk = [snapshot["SNAPSHOT"]].flatten.compact rescue []
+                snapshot_disk.each do |snapshot|
+                    total_disk_size += snapshot["SIZE"].to_i
+                end
+            end
+
+            OpenNebulaHelper.unit_to_str(total_disk_size * 1024.0, {})
+        end
+
+        default :VID, :HOSTNAME, :ACTION, :REASON, :START_TIME, :END_TIME, :MEMORY, :CPU, :NETRX, :NETTX, :DISK
+    end
+
+    SHOWBACK_TABLE = CLIHelper::ShowTable.new(self.table_conf("oneshowback.yaml"), nil) do
+        column :UID, "User ID", :size=>4 do |d|
+            d["UID"]
+        end
+
+        column :USER_NAME, "User name", :left, :size=>12 do |d|
+            d["UNAME"]
+        end
+
+        column :GID, "Group ID", :size=>4 do |d|
+            d["GID"]
+        end
+
+        column :GROUP_NAME, "Group name", :left, :size=>12 do |d|
+            d["GNAME"]
+        end
+
+        column :VM_ID, "Virtual Machine ID", :size=>6 do |d|
+            d["VMID"]
+        end
+
+        column :VM_NAME, "Virtual Machine name", :left, :size=>12 do |d|
+            d["VMNAME"]
+        end
+
+        column :MONTH, "Month", :size=>5 do |d|
+            d["MONTH"]
+        end
+
+        column :YEAR, "Year", :size=>5 do |d|
+            d["YEAR"]
+        end
+
+        column :HOURS, "Hours", :size=>6 do |d|
+            d["HOURS"]
+        end
+
+        column :COST, "Cost", :size=>15 do |d|
+            d["TOTAL_COST"]
+        end
+
+        default :USER_NAME, :GROUP_NAME, :VM_ID, :VM_NAME, :MONTH, :YEAR, :HOURS, :COST
     end
 
     def self.print_start_end_time_header(start_time, end_time)
@@ -184,4 +266,12 @@ class AcctHelper < OpenNebulaHelper::OneHelper
         CLIHelper.scr_restore
         puts
     end
+
+    def self.print_month_header(year, month)
+        CLIHelper.scr_bold
+        CLIHelper.scr_underline
+        puts "# Showback for #{month}/#{year}".ljust(80)
+        CLIHelper.scr_restore
+        puts
+    end
 end
