@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -108,6 +108,22 @@ class OneGroupHelper < OpenNebulaHelper::OneHelper
             column :VMS , "Number of VMS", :size=>9 do |d|
                 begin
                     q = quotas[d['ID']]
+
+                    if q['VM_QUOTA']['VM'].nil? && d["ID"].to_i != 0
+                        q['VM_QUOTA']['VM'] = {
+                            "VMS"         => OneQuotaHelper::LIMIT_DEFAULT,
+                            "VMS_USED"    => "0",
+                            "CPU"         => OneQuotaHelper::LIMIT_DEFAULT,
+                            "CPU_USED"    => "0",
+                            "MEMORY"      => OneQuotaHelper::LIMIT_DEFAULT,
+                            "MEMORY_USED" => "0",
+                            "SYSTEM_DISK_SIZE"      => OneQuotaHelper::LIMIT_DEFAULT,
+                            "SYSTEM_DISK_SIZE_USED" => "0"
+                        }
+
+                        quotas[d['ID']] = q
+                    end
+
                     limit = q['VM_QUOTA']['VM']["VMS"]
 
                     if limit == OneQuotaHelper::LIMIT_DEFAULT
@@ -129,6 +145,22 @@ class OneGroupHelper < OpenNebulaHelper::OneHelper
             column :MEMORY, "Total memory allocated to user VMs", :size=>17 do |d|
                 begin
                     q = quotas[d['ID']]
+
+                    if q['VM_QUOTA']['VM'].nil? && d["ID"].to_i != 0
+                        q['VM_QUOTA']['VM'] = {
+                            "VMS"         => OneQuotaHelper::LIMIT_DEFAULT,
+                            "VMS_USED"    => "0",
+                            "CPU"         => OneQuotaHelper::LIMIT_DEFAULT,
+                            "CPU_USED"    => "0",
+                            "MEMORY"      => OneQuotaHelper::LIMIT_DEFAULT,
+                            "MEMORY_USED" => "0",
+                            "SYSTEM_DISK_SIZE"      => OneQuotaHelper::LIMIT_DEFAULT,
+                            "SYSTEM_DISK_SIZE_USED" => "0"
+                        }
+
+                        quotas[d['ID']] = q
+                    end
+
                     limit = q['VM_QUOTA']['VM']["MEMORY"]
 
                     if limit == OneQuotaHelper::LIMIT_DEFAULT
@@ -153,6 +185,22 @@ class OneGroupHelper < OpenNebulaHelper::OneHelper
             column :CPU, "Total CPU allocated to user VMs", :size=>11 do |d|
                 begin
                     q = quotas[d['ID']]
+
+                    if q['VM_QUOTA']['VM'].nil? && d["ID"].to_i != 0
+                        q['VM_QUOTA']['VM'] = {
+                            "VMS"         => OneQuotaHelper::LIMIT_DEFAULT,
+                            "VMS_USED"    => "0",
+                            "CPU"         => OneQuotaHelper::LIMIT_DEFAULT,
+                            "CPU_USED"    => "0",
+                            "MEMORY"      => OneQuotaHelper::LIMIT_DEFAULT,
+                            "MEMORY_USED" => "0",
+                            "SYSTEM_DISK_SIZE"      => OneQuotaHelper::LIMIT_DEFAULT,
+                            "SYSTEM_DISK_SIZE_USED" => "0"
+                        }
+
+                        quotas[d['ID']] = q
+                    end
+
                     limit = q['VM_QUOTA']['VM']["CPU"]
 
                     if limit == OneQuotaHelper::LIMIT_DEFAULT
@@ -238,30 +286,24 @@ class OneGroupHelper < OpenNebulaHelper::OneHelper
         puts group.template_str
         puts
 
-        CLIHelper.print_header(str_h1 % "USERS", false)
-        CLIHelper.print_header("%-15s" % ["ID"])
-        group.user_ids.each do |uid|
-            puts "%-15s" % [uid]
-        end
+        admin_ids = group.admin_ids
+
+        CLIHelper::ShowTable.new(nil, self) do
+            column :"USER ID", "", :right, :size=>7 do |d|
+                d
+            end
+
+            column :"ADMIN", "", :left, :size=>5 do |d|
+                if (group.admin_ids.include?(d))
+                    "*"
+                else
+                    ""
+                end
+            end
+        end.show(group.user_ids, {})
 
         group_hash = group.to_hash
 
-        providers = group_hash['GROUP']['RESOURCE_PROVIDER']
-        if(providers != nil)
-            puts
-            CLIHelper.print_header(str_h1 % "RESOURCE PROVIDERS", false)
-
-            CLIHelper::ShowTable.new(nil, self) do
-                column :"ZONE", "", :right, :size=>7 do |d|
-                    d['ZONE_ID']
-                end
-
-                column :"CLUSTER", "", :right, :size=>7 do |d|
-                    d['CLUSTER_ID'] == '10' ? 'ALL' : d['CLUSTER_ID']
-                end
-            end.show([providers].flatten, {})
-        end
-
         default_quotas = nil
 
         group.each('/GROUP/DEFAULT_GROUP_QUOTAS') { |elem|
@@ -269,6 +311,6 @@ class OneGroupHelper < OpenNebulaHelper::OneHelper
         }
 
         helper = OneQuotaHelper.new
-        helper.format_quota(group_hash['GROUP'], default_quotas)
+        helper.format_quota(group_hash['GROUP'], default_quotas, group.id)
     end
 end
