@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -86,10 +86,10 @@ class OneImageHelper < OpenNebulaHelper::OneHelper
                     path=Dir.pwd+"/"+o
                 end
 
-                if File.exist?(path)
+                if File.readable?(path)
                     [0, path]
                 else
-                    [-1, "File '#{path}' does not exist."]
+                    [-1, "File '#{path}' does not exist or is not readable."]
                 end
             end
         },
@@ -292,6 +292,13 @@ class OneImageHelper < OpenNebulaHelper::OneHelper
 
             puts str % [e,  mask]
         }
+
+        if image.has_elements?("/IMAGE/SNAPSHOTS")
+            puts
+            CLIHelper.print_header(str_h1 % "IMAGE SNAPSHOTS",false)
+            format_snapshots(image)
+        end
+
         puts
 
         CLIHelper.print_header(str_h1 % "IMAGE TEMPLATE",false)
@@ -311,6 +318,56 @@ class OneImageHelper < OpenNebulaHelper::OneHelper
         end
     end
 
+    def format_snapshots(image)
+        table=CLIHelper::ShowTable.new(nil, self) do
+            column :AC , "Is active", :left, :size => 2 do |d|
+                if d["ACTIVE"] == "YES"
+                    "=>"
+                else
+                    ""
+                end
+            end
+            column :ID, "Snapshot ID", :size=>3 do |d|
+                d["ID"]
+            end
+
+            column :PARENT, "Snapshot Parent ID", :size=>6 do |d|
+                d["PARENT"]
+            end
+
+            column :CHILDREN, "Snapshot Children IDs", :size=>10 do |d|
+                d["CHILDREN"]
+            end
+
+            column :SIZE, "", :left, :size=>8 do |d|
+                if d["SIZE"]
+                    OpenNebulaHelper.unit_to_str(
+                                d['SIZE'].to_i,
+                                {},
+                                "M"
+                            )
+                else
+                    "-"
+                end
+            end
+
+            column :NAME, "Snapshot Name", :left, :size=>37 do |d|
+                d["NAME"]
+            end
+
+            column :DATE, "Snapshot creation date", :size=>15 do |d|
+                OpenNebulaHelper.time_to_str(d["DATE"])
+            end
+
+            default :AC, :ID, :PARENT, :DATE, :SIZE, :NAME
+        end
+
+        # Convert snapshot data to an array
+        image_hash = image.to_hash
+        image_snapshots = [image_hash['IMAGE']['SNAPSHOTS']].flatten.first
+        table.show(image_snapshots)
+    end
+
     def self.create_image_variables(options, name)
         if Array===name
             names=name
