@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -34,10 +34,10 @@ class OneQuotaHelper
         #  ]
         #
         #  VM = [
-        #    VMS           = <Max. number of VMs>
-        #    MEMORY        = <Max. allocated memory (MB)>
-        #    CPU           = <Max. allocated CPU>
-        #    VOLATILE_SIZE = <Max. allocated volatile disks (MB)>
+        #    VMS              = <Max. number of VMs>
+        #    MEMORY           = <Max. allocated memory (MB)>
+        #    CPU              = <Max. allocated CPU>
+        #    SYSTEM_DISK_SIZE = <Max. allocated system disk (MB)>
         #  ]
         #
         #  NETWORK = [
@@ -74,7 +74,7 @@ class OneQuotaHelper
 
     #  Edits the quota template of a resource
     #  @param [XMLElement] resource to get the current info from
-    #  @param [String] path to the new contents. If nil a editor will be 
+    #  @param [String] path to the new contents. If nil a editor will be
     #         used
     #  @param [True|False] is_default To change the help text
     #  @return [String] contents of the new quotas
@@ -122,7 +122,7 @@ class OneQuotaHelper
 
     #  Retrieves a clean quota template, without any existing resource
     #  information
-    #  @param path [String] path to the new contents. If nil a editor will be 
+    #  @param path [String] path to the new contents. If nil a editor will be
     #         used
     #  @return [String] contents of the new quotas
     def self.get_batch_quota(path)
@@ -159,7 +159,7 @@ class OneQuotaHelper
     #  Edits the quota template of a resource, adding the quotas set in str
     #  @param resource [PoolElement] to get the current info from
     #  @param str [String] quota template, created by get_batch_quota()
-    #  @return [String, OpenNebula::Error] merged contents of the new quotas on 
+    #  @return [String, OpenNebula::Error] merged contents of the new quotas on
     #    success, Error if the user info could not be retrieved
     def self.merge_quota(resource, str)
         rc = resource.info
@@ -189,8 +189,9 @@ class OneQuotaHelper
     #  Outputs formated quota information to stdout
     #  @param qh [Hash] with the quotas for a given resource
     #  @param default_quotas_hash [XMLElement] with the default quota limits
+    #  @param resource_id [Integer] user/group ID
     #
-    def format_quota(qh, default_quotas)
+    def format_quota(qh, default_quotas, resource_id)
         str_h1="%-80s"
 
         puts
@@ -203,6 +204,23 @@ class OneQuotaHelper
 
         vm_quotas = [qh['VM_QUOTA']['VM']].flatten
 
+        # This initializes the VM quotas for users/groups that don't have any
+        # resource usage yet. It not applied to oneamdin
+        if vm_quotas[0].nil? && resource_id.to_i != 0
+            limit = LIMIT_DEFAULT
+
+            vm_quotas = [{
+                "VMS"         => limit,
+                "VMS_USED"    => "0",
+                "CPU"         => limit,
+                "CPU_USED"    => "0",
+                "MEMORY"      => limit,
+                "MEMORY_USED" => "0",
+                "SYSTEM_DISK_SIZE"      => limit,
+                "SYSTEM_DISK_SIZE_USED" => "0"
+            }]
+        end
+
         if !vm_quotas[0].nil?
             CLIHelper::ShowTable.new(nil, self) do
                 column :"NUMBER OF VMS", "", :right, :size=>17 do |d|
@@ -255,20 +273,20 @@ class OneQuotaHelper
                     end
                 end
 
-                column :"VOLATILE_SIZE", "", :right, :size=>20 do |d|
+                column :"SYSTEM_DISK_SIZE", "", :right, :size=>20 do |d|
                     if !d.nil?
-                        elem = 'VOLATILE_SIZE'
+                        elem = 'SYSTEM_DISK_SIZE'
                         limit = d[elem]
                         limit = helper.get_default_limit(
                             limit, "VM_QUOTA/VM/#{elem}")
 
                         if limit == LIMIT_UNLIMITED
                             "%8s /        -" % [
-                                OpenNebulaHelper.unit_to_str(d["VOLATILE_SIZE_USED"].to_i,{},"M")
+                                OpenNebulaHelper.unit_to_str(d["SYSTEM_DISK_SIZE_USED"].to_i,{},"M")
                             ]
                         else
                             "%8s / %8s" % [
-                                OpenNebulaHelper.unit_to_str(d["VOLATILE_SIZE_USED"].to_i,{},"M"),
+                                OpenNebulaHelper.unit_to_str(d["SYSTEM_DISK_SIZE_USED"].to_i,{},"M"),
                                 OpenNebulaHelper.unit_to_str(limit.to_i,{},"M")
                             ]
                         end
