@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -15,6 +15,7 @@
 #--------------------------------------------------------------------------- #
 
 require 'one_helper'
+require 'base64'
 
 class OneTemplateHelper < OpenNebulaHelper::OneHelper
     VM_NAME={
@@ -43,6 +44,13 @@ EOT
         :description => "Integrate userdata into the EC2 section"
     }
 
+    EXTENDED={
+        :name => "extended",
+        :large => "--extended",
+        :description => "Process the template and included extended "+
+                        "information, such as the SIZE for each DISK"
+    }
+
     def self.rname
         "VMTEMPLATE"
     end
@@ -51,6 +59,26 @@ EOT
         "onetemplate.yaml"
     end
 
+
+    def show_resource(id, options)
+        resource = retrieve_resource(id)
+
+        if !options[:extended].nil?
+            rc = resource.info(options[:extended])
+        else
+            rc = resource.info
+        end
+
+        return -1, rc.message if OpenNebula.is_error?(rc)
+
+        if options[:xml]
+            return 0, resource.to_xml(true)
+        else
+            format_resource(resource, options)
+            return 0
+        end
+    end
+
     def format_pool(options)
         config_file = self.class.table_conf
 
@@ -90,7 +118,7 @@ EOT
 
         answers = ""
 
-        puts "There are some parameters that require user input."
+        puts "There are some parameters that require user input. Use the string <<EDITOR>> to launch an editor (e.g. for multi-line inputs)"
 
         user_inputs.each do |key, val|
             input_cfg = val.split('|')
@@ -109,8 +137,16 @@ EOT
             print "  * (#{key}) #{description}: "
 
             case type
-            when 'text'
+            when 'text', 'text64'
                 answer = STDIN.readline.chop
+
+                if answer == "<<EDITOR>>"
+                    answer = OpenNebulaHelper.editor_input()
+                end
+
+                if type == 'text64'
+                    answer = Base64::encode64(answer).strip.delete("\n")
+                end
             when 'password'
                 answer = OpenNebulaHelper::OneHelper.get_password
             else
