@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -14,9 +14,45 @@
 # limitations under the License.                                             #
 #--------------------------------------------------------------------------- #
 
+if !ONE_LOCATION
+    MAD_LOCATION      = "/usr/lib/one/mads"
+    VAR_LOCATION      = "/var/lib/one"
+else
+    MAD_LOCATION      = ONE_LOCATION + "/lib/mads"
+    VAR_LOCATION      = ONE_LOCATION + "/var"
+end
+
+VMS_LOCATION = VAR_LOCATION + "/vms"
+
+$: << MAD_LOCATION
+
 require 'one_helper'
 require 'optparse/time'
 
+class String
+    def red
+        colorize(31)
+    end
+
+    def green
+        colorize(32)
+    end
+
+private
+
+    def colorize(color_code)
+        "\e[#{color_code}m#{self}\e[0m"
+    end
+end
+
+EXTERNAL_IP_ATTRS = [
+    'GUEST_IP',
+    'AWS_IP_ADDRESS',
+    'AZ_IPADDRESS',
+    'SL_PRIMARYIPADDRESS'
+];
+
+
 class OneVMHelper < OpenNebulaHelper::OneHelper
     MULTIPLE={
         :name  => "multiple",
@@ -123,6 +159,45 @@ class OneVMHelper < OpenNebulaHelper::OneHelper
         return short_state_str
     end
 
+    # Return the IP or several IPs of a VM
+    def self.ip_str(vm)
+        ips = []
+
+        vm_nics = []
+
+        if !vm["TEMPLATE"]["NIC"].nil?
+            vm_nics = [vm["TEMPLATE"]['NIC']].flatten
+        end
+
+        vm_nics.each do |nic|
+            if nic.has_key?("IP")
+                ips.push(nic["IP"])
+            end
+
+            if nic.has_key?("IP6_GLOBAL")
+                ips.push(nic["IP6_GLOBAL"])
+            end
+
+            if nic.has_key?("IP6_ULA")
+                ips.push(nic["IP6_ULA"])
+            end
+        end
+
+        VirtualMachine::EXTERNAL_IP_ATTRS.each do |attr|
+            external_ip = vm["MONITORING"][attr]
+
+            if !external_ip.nil? && !ips.include?(external_ip)
+                ips.push(external_ip)
+            end
+        end
+
+        if ips.empty?
+            return "--"
+        else
+            return ips.join(",")
+        end
+    end
+
     def format_pool(options)
         config_file = self.class.table_conf
 
@@ -168,11 +243,14 @@ class OneVMHelper < OpenNebulaHelper::OneHelper
             end
 
             column :UCPU, "CPU percentage used by the VM", :size=>4 do |d|
-                d["CPU"]
+                cpu = d["MONITORING"]["CPU"]
+                cpu = "0" if cpu.nil?
+
+                cpu
             end
 
             column :UMEM, "Memory used by the VM", :size=>7 do |d|
-                OpenNebulaHelper.unit_to_str(d["MEMORY"].to_i, options)
+                OpenNebulaHelper.unit_to_str(d["MONITORING"]["MEMORY"].to_i, options)
             end
 
             column :HOST, "Host where the VM is running", :left, :size=>10 do |d|
@@ -208,6 +286,10 @@ class OneVMHelper < OpenNebulaHelper::OneHelper
                 OpenNebulaHelper.period_to_str(dtime, false)
             end
 
+            column :IP, "VM IP addresses", :left, :donottruncate, :size=>15 do |d|
+                OneVMHelper.ip_str(d)
+            end
+
             default :ID, :USER, :GROUP, :NAME, :STAT, :UCPU, :UMEM, :HOST,
                 :TIME
         end
@@ -247,6 +329,133 @@ class OneVMHelper < OpenNebulaHelper::OneHelper
         end
     end
 
+    RECOVER_RETRY_STEPS = {
+        :PROLOG_MIGRATE_FAILURE          => :migrate,
+        :PROLOG_MIGRATE_POWEROFF_FAILURE => :migrate,
+        :PROLOG_MIGRATE_SUSPEND_FAILURE  => :migrate,
+        :PROLOG_FAILURE                  => :prolog,
+        :PROLOG_RESUME_FAILURE           => :resume,
+        :PROLOG_UNDEPLOY_FAILURE         => :resume,
+        :EPILOG_FAILURE                  => :epilog,
+        :EPILOG_STOP_FAILURE             => :stop,
+        :EPILOG_UNDEPLOY_FAILURE         => :stop
+    }
+
+    def recover_retry_interactive(vm)
+        begin
+            require 'one_tm'
+        rescue LoadError
+            STDERR.puts <<-EOT
+one_tm library not found. Make sure you execute recover --interactive
+in the frontend machine.
+            EOT
+            exit(-1)
+        end
+
+        # Disable CTRL-C in the menu
+        trap("SIGINT") { }
+
+        if !File.readable?(VAR_LOCATION+"/config")
+            STDERR.puts "Error reading #{VAR_LOCATION+'/config'}. The " <<
+                "TM Debug Interactive Environment must be executed as " <<
+                "oneadmin in the frontend."
+            exit -1
+        end
+
+        rc = vm.info
+        if OpenNebula.is_error?(rc)
+            STDERR.puts rc.message
+            exit -1
+        end
+
+        if !RECOVER_RETRY_STEPS.include?(vm.lcm_state_str.to_sym)
+            STDERR.puts "Current LCM STATE '#{vm.lcm_state_str}' not " <<
+                "compatible with RECOVER RETRY action."
+            exit -1
+        end
+
+        seq = vm['/VM/HISTORY_RECORDS/HISTORY[last()]/SEQ']
+
+        tm_action = RECOVER_RETRY_STEPS[vm.lcm_state_str.to_sym]
+
+        tm_file = "#{VMS_LOCATION}/#{vm.id}/transfer.#{seq}.#{tm_action}"
+
+        if !File.readable?(tm_file)
+            STDERR.puts "Cannot read #{tm_file}"
+            exit -1
+        end
+
+        @tm_action_list = File.read(tm_file)
+
+        puts "TM Debug Interactive Environment.".green
+        puts
+        print_tm_action_list
+
+        @tm = TransferManagerDriver.new(nil)
+        i=0
+        @tm_action_list.lines.each do |tm_command|
+            i+=1
+            success=false
+
+            while !success
+                puts "Current action (#{i}):".green
+                puts tm_command
+                puts
+
+                puts <<-EOF.gsub(/^\s+/,"")
+                Choose action:
+                (r) Run action
+                (n) Skip to next action
+                (a) Show all actions
+                (q) Quit
+                EOF
+
+                ans = ""
+                while !%w(n a r q).include?(ans)
+                    printf "> "
+                    ans = STDIN.gets.strip.downcase
+
+                    puts
+
+                    case ans
+                    when "n"
+                        success = true
+                    when "a"
+                        print_tm_action_list
+                    when "q"
+                        exit -1
+                    when "r"
+                        result, result_message = @tm.do_transfer_action(@id, tm_command.split)
+
+                        if result == "SUCCESS"
+                            success = true
+                            puts "#{result}"
+                            puts
+                        else
+                            puts
+                            puts "#{result}. Repeat command.".red
+                            puts
+                        end
+                    end
+                end
+            end
+        end
+
+        puts "If all the TM actions have been successful and you want to"
+        puts "recover the Virtual Machine to the RUNNING state execute this command:"
+        puts "$ onevm recover #{vm.id} --success"
+    end
+
+    def print_tm_action_list
+        puts "TM Action list:".green
+        i=0
+        @tm_action_list.lines.each do |line|
+            i+=1
+            puts "(#{i}) #{line}"
+        end
+        puts
+    end
+
     private
 
     def factory(id=nil)
@@ -312,22 +521,45 @@ class OneVMHelper < OpenNebulaHelper::OneHelper
         puts
 
         CLIHelper.print_header(str_h1 % "VIRTUAL MACHINE MONITORING",false)
-        poll_attrs = {
-            "USED MEMORY" => "MEMORY",
-            "USED CPU" => "CPU",
-            "NET_TX" => "NET_TX",
-            "NET_RX" => "NET_RX"
-        }
 
-        poll_attrs.each { |k,v|
-            if k == "USED CPU"
-                puts str % [k,vm[v]]
-            elsif k == "USED MEMORY"
-                puts str % [k, OpenNebulaHelper.unit_to_str(vm[v].to_i, {})]
-            else
-                puts str % [k, OpenNebulaHelper.unit_to_str(vm[v].to_i/1024, {})]
+        vm_monitoring = vm.to_hash['VM']['MONITORING']
+
+        # Find out if it is a hybrid VM to avoid showing local IPs
+        isHybrid=false
+        vm_monitoring.each{|key, value|
+            if EXTERNAL_IP_ATTRS.include? key
+                isHybrid=true
             end
         }
+
+        order_attrs  = %w(CPU MEMORY NETTX NETRX)
+
+        vm_monitoring_sort = []
+        order_attrs.each do |key|
+            if (val = vm_monitoring.delete(key))
+                vm_monitoring_sort << [key, val]
+            end
+        end
+
+        vm_monitoring_sort.sort{|a,b| a[0]<=>b[0]}
+
+        filter_attrs = %w(STATE DISK_SIZE SNAPSHOT_SIZE)
+        vm_monitoring.each do |key, val|
+            if !filter_attrs.include?(key)
+                vm_monitoring_sort << [key, val]
+            end
+        end
+
+        vm_monitoring_sort.each do |k,v|
+            if k == "MEMORY"
+                puts str % [k, OpenNebulaHelper.unit_to_str(v.to_i, {})]
+            elsif k  =~ /NET.X/
+                puts str % [k, OpenNebulaHelper.unit_to_str(v.to_i/1024, {})]
+            else
+                puts str % [k, v]
+            end
+        end
+
         puts
 
         CLIHelper.print_header(str_h1 % "PERMISSIONS",false)
@@ -341,7 +573,33 @@ class OneVMHelper < OpenNebulaHelper::OneHelper
             puts str % [e,  mask]
         }
 
+        vm_disks = []
+
         if vm.has_elements?("/VM/TEMPLATE/DISK")
+            vm_disks = [vm.to_hash['VM']['TEMPLATE']['DISK']].flatten
+        end
+
+        if vm.has_elements?("/VM/TEMPLATE/CONTEXT")
+            context_disk = vm.to_hash['VM']['TEMPLATE']['CONTEXT']
+
+            context_disk["IMAGE"]     = "CONTEXT"
+            context_disk["DATASTORE"] = "-"
+            context_disk["TYPE"]      = "-"
+            context_disk["READONLY"]  = "-"
+            context_disk["SAVE"]      = "-"
+            context_disk["CLONE"]     = "-"
+            context_disk["SAVE_AS"]   = "-"
+
+            vm_disks.push(context_disk)
+        end
+
+        # get monitoring data
+        vm_disks.each do |disk|
+            disk_id = disk["DISK_ID"]
+            disk["MONITOR_SIZE"] = vm["MONITORING/DISK_SIZE[ID='#{disk_id}']/SIZE"]
+        end
+
+        if !vm_disks.empty?
             puts
             CLIHelper.print_header(str_h1 % "VM DISKS",false)
             CLIHelper::ShowTable.new(nil, self) do
@@ -374,6 +632,30 @@ class OneVMHelper < OpenNebulaHelper::OneHelper
                     end
                 end
 
+                column :SIZE, "", :left, :size=>9 do |d|
+                    if d["SIZE"]
+                        size = OpenNebulaHelper.unit_to_str(
+                                    d['SIZE'].to_i,
+                                    {},
+                                    "M"
+                                )
+                    else
+                        size = "-"
+                    end
+
+                    if d["MONITOR_SIZE"]
+                        monitor_size = OpenNebulaHelper.unit_to_str(
+                                    d['MONITOR_SIZE'].to_i,
+                                    {},
+                                    "M"
+                                )
+                    else
+                        monitor_size = "-"
+                    end
+
+                    "#{monitor_size}/#{size}"
+                end
+
                 column :TYPE, "", :left, :size=>4 do |d|
                     d["TYPE"].downcase
                 end
@@ -390,69 +672,72 @@ class OneVMHelper < OpenNebulaHelper::OneHelper
                     d["CLONE"]
                 end
 
-                column :"SAVE_AS", "", :size=>7 do |d|
-                    d["SAVE_AS"] || "-"
-                end
-
-
-                default :ID, :TARGET, :IMAGE, :TYPE,
-                    :SAVE, :SAVE_AS
-            end.show([vm.to_hash['VM']['TEMPLATE']['DISK']].flatten, {})
+                default :ID, :DATASTORE, :TARGET, :IMAGE, :SIZE, :TYPE,
+                    :SAVE
+            end.show(vm_disks, {})
 
             while vm.has_elements?("/VM/TEMPLATE/DISK")
                 vm.delete_element("/VM/TEMPLATE/DISK")
             end if !options[:all]
         end
 
-        if vm.has_elements?("/VM/USER_TEMPLATE/HYPERVISOR")
-            vm_information = vm.to_hash['VM']
-            hybridvisor    = vm_information['USER_TEMPLATE']['HYPERVISOR'].to_s
-            isHybrid       = %w{vcenter ec2 azure softlayer}.include? hybridvisor
+        if vm.has_elements?("/VM/SNAPSHOTS")
+            puts
+            CLIHelper.print_header(str_h1 % "VM DISK SNAPSHOTS",false)
+            format_snapshots(vm)
+        end
 
-            if isHybrid
-                vm_tmplt = vm_information['TEMPLATE']
-                nic =   {"NETWORK" => "-",
-                           "IP" => "-",
-                           "MAC"=> "-",
-                           "VLAN"=>"no",
-                           "BRIDGE"=>"-"}
+        sg_nics = []
 
-                case hybridvisor
-                    when "vcenter"
-                        nic["IP"] = vm_tmplt['GUEST_IP'] if vm_tmplt['GUEST_IP']
-                    when "ec2"
-                        nic["IP"] = vm_tmplt['IP_ADDRESS'] if vm_tmplt['IP_ADDRESS']
-                    when "azure"
-                        nic["IP"] = vm_tmplt['IPADDRESS'] if vm_tmplt['IPADDRESS']
-                    when "softlayer"
-                        nic["IP"] = vm_tmplt['PRIMARYIPADDRESS'] if vm_tmplt['PRIMARYIPADDRESS']
-                    else
-                        isHybrid = false
+        if (vm.has_elements?("/VM/TEMPLATE/NIC/SECURITY_GROUPS"))
+            sg_nics = [vm.to_hash['VM']['TEMPLATE']['NIC']].flatten
+
+            sg_nics.each do |nic|
+                sg = nic["SECURITY_GROUPS"]
+
+                if sg.nil?
+                    next
                 end
-
-                vm_nics = [nic]
             end
         end
 
-        if vm.has_elements?("/VM/TEMPLATE/NIC") || vm_nics
+        # This variable holds the extra IP's got from monitoring. Right
+        # now it adds GUEST_IP and GUEST_IP_ADDRESSES from vcenter
+        # monitoring. If other variables hold IPs just add them to this
+        # array. Duplicate IPs are not shown.
+        extra_ips = []
+
+        if val=vm["/VM/MONITORING/GUEST_IP"]
+            extra_ips << val if val && !val.empty?
+        end
+
+        if val=vm["/VM/MONITORING/GUEST_IP_ADDRESSES"]
+            extra_ips += val.split(',') if val && !val.empty?
+        end
+
+        extra_ips.uniq!
+
+        if vm.has_elements?("/VM/TEMPLATE/NIC") || !extra_ips.empty?
             puts
             CLIHelper.print_header(str_h1 % "VM NICS",false)
 
-            vm_nics = [vm.to_hash['VM']['TEMPLATE']['NIC']].flatten if !vm_nics
-
             nic_default = {"NETWORK" => "-",
                            "IP" => "-",
                            "MAC"=> "-",
                            "VLAN"=>"no",
                            "BRIDGE"=>"-"}
 
-            array_id = 0
+            shown_ips = []
 
+            array_id = 0
+            vm_nics = [vm.to_hash['VM']['TEMPLATE']['NIC']].flatten.compact
             vm_nics.each {|nic|
 
                 next if nic.has_key?("CLI_DONE")
 
                 if nic.has_key?("IP6_LINK")
+                    shown_ips << nic["IP6_LINK"]
+
                     ip6_link = {"IP"           => nic.delete("IP6_LINK"),
                                 "CLI_DONE"     => true,
                                 "DOUBLE_ENTRY" => true}
@@ -462,6 +747,8 @@ class OneVMHelper < OpenNebulaHelper::OneHelper
                 end
 
                 if nic.has_key?("IP6_ULA")
+                    shown_ips << nic["IP6_ULA"]
+
                     ip6_link = {"IP"           => nic.delete("IP6_ULA"),
                                 "CLI_DONE"     => true,
                                 "DOUBLE_ENTRY" => true}
@@ -471,6 +758,8 @@ class OneVMHelper < OpenNebulaHelper::OneHelper
                 end
 
                 if nic.has_key?("IP6_GLOBAL")
+                    shown_ips << nic["IP6_GLOBAL"]
+
                     ip6_link = {"IP"           => nic.delete("IP6_GLOBAL"),
                                 "CLI_DONE"     => true,
                                 "DOUBLE_ENTRY" => true}
@@ -479,10 +768,25 @@ class OneVMHelper < OpenNebulaHelper::OneHelper
                     array_id += 1
                 end
 
+                shown_ips << nic["IP"] if nic.has_key?("IP")
+
                 nic.merge!(nic_default) {|k,v1,v2| v1}
                 array_id += 1
             }
 
+            extra_ips -= shown_ips
+
+            # Add extra IPs to the VM NICS table
+            extra_ips.each do |ip|
+                vm_nics << {
+                    "NIC_ID"        => "-",
+                    "IP"            => ip,
+                    "NETWORK"       => "Additional IP",
+                    "BRIDGE"        => "-",
+                    "VLAN"          => "-"
+                }
+            end
+
             CLIHelper::ShowTable.new(nil, self) do
                 column :ID, "", :size=>3 do |d|
                     if d["DOUBLE_ENTRY"]
@@ -535,6 +839,88 @@ class OneVMHelper < OpenNebulaHelper::OneHelper
             end if !options[:all]
         end
 
+        while vm.has_elements?("/VM/TEMPLATE/NIC")
+            vm.delete_element("/VM/TEMPLATE/NIC")
+        end if !options[:all]
+
+        if vm.has_elements?("/VM/TEMPLATE/SECURITY_GROUP_RULE") and !isHybrid
+            puts
+            CLIHelper.print_header(str_h1 % "SECURITY",false)
+            puts
+
+            CLIHelper::ShowTable.new(nil, self) do
+                column :NIC_ID, "", :size=>6 do |d|
+                    d["NIC_ID"]
+                end
+
+                column :NETWORK, "", :left, :size=>25 do |d|
+                    d["NETWORK"]
+                end
+
+                column :SECURITY_GROUPS, "", :left, :size=>47 do |d|
+                    d["SECURITY_GROUPS"]
+                end
+            end.show(sg_nics,{})
+
+            puts
+
+            CLIHelper.print_header(str_h1 % "SECURITY GROUP   TYPE     PROTOCOL NETWORK                       RANGE          ",false)
+
+            CLIHelper::ShowTable.new(nil, self) do
+                column :ID, "", :size=>4 do |d|
+                    d["SECURITY_GROUP_ID"]
+                end
+
+                column :NAME, "", :left, :size=>11 do |d|
+                    d["SECURITY_GROUP_NAME"]
+                end
+
+                column :" ", "", :left, :size=>8 do |d|
+                    d["RULE_TYPE"]
+                end
+
+                column :"  ", "", :left, :size=>8 do |d|
+                    protocol = d["PROTOCOL"]
+
+                    if(protocol.upcase == "ICMP")
+                        icmp = d["ICMP_TYPE"].nil? ? "" : "-#{d["ICMP_TYPE"]}"
+                        protocol += icmp
+                    end
+
+                    protocol
+                end
+
+                column :VNET, "", :size=>4 do |d|
+                    d["NETWORK_ID"]
+                end
+
+                column :START, "", :left, :donottruncate, :size=>17 do |d|
+                    network = ""
+
+                    if(!d["IP"].nil? && d["IP"] != "")
+                        network = d["IP"]
+                    elsif(!d["MAC"].nil? && d["MAC"] != "")
+                        network = d["MAC"]
+                    end
+
+                    network
+                end
+
+                column :SIZE, "", :left, :donottruncate, :size=>6 do |d|
+                    d["SIZE"]
+                end
+
+                column :"   ", "", :left, :donottruncate, :size=>15 do |d|
+                    d["RANGE"]
+                end
+
+            end.show([vm.to_hash['VM']['TEMPLATE']['SECURITY_GROUP_RULE']].flatten, {})
+
+            while vm.has_elements?("/VM/TEMPLATE/SECURITY_GROUP_RULE")
+                vm.delete_element("/VM/TEMPLATE/SECURITY_GROUP_RULE")
+            end if !options[:all]
+        end
+
         if vm.has_elements?("/VM/TEMPLATE/SNAPSHOT")
             puts
             CLIHelper.print_header(str_h1 % "SNAPSHOTS",false)
@@ -666,4 +1052,91 @@ class OneVMHelper < OpenNebulaHelper::OneHelper
 
         table.show(history)
     end
+
+    def format_snapshots(vm)
+        table=CLIHelper::ShowTable.new(nil, self) do
+            column :AC , "Is active", :left, :size => 2 do |d|
+                if d["ACTIVE"] == "YES"
+                    "=>"
+                else
+                    ""
+                end
+            end
+            column :ID, "Snapshot ID", :size=>3 do |d|
+                d["ID"]
+            end
+
+            column :DISK, "Disk ID", :size=>4 do |d|
+                d["DISK_ID"]
+            end
+
+            column :PARENT, "Snapshot Parent ID", :size=>6 do |d|
+                d["PARENT"]
+            end
+
+            column :CHILDREN, "Snapshot Children IDs", :size=>10 do |d|
+                d["CHILDREN"]
+            end
+
+            column :SIZE, "", :left, :size=>12 do |d|
+                if d["SIZE"]
+                    size = OpenNebulaHelper.unit_to_str(
+                                d['SIZE'].to_i,
+                                {},
+                                "M"
+                            )
+                else
+                    size = "-"
+                end
+
+                if d["MONITOR_SIZE"]
+                    monitor_size = OpenNebulaHelper.unit_to_str(
+                                d['MONITOR_SIZE'].to_i,
+                                {},
+                                "M"
+                            )
+                else
+                    monitor_size = "-"
+                end
+
+                "#{monitor_size}/#{size}"
+            end
+
+            column :NAME, "Snapshot Name", :left, :size=>32 do |d|
+                d["NAME"]
+            end
+
+            column :DATE, "Snapshot creation date", :size=>15 do |d|
+                OpenNebulaHelper.time_to_str(d["DATE"])
+            end
+
+            default :AC, :ID, :DISK, :PARENT, :DATE, :SIZE, :NAME
+        end
+
+        # Convert snapshot data to an array
+        vm_hash = vm.to_hash
+        vm_snapshots = [vm_hash['VM']['SNAPSHOTS']].flatten
+
+        snapshots = []
+
+        vm_snapshots.each do |disk|
+            disk_id = disk['DISK_ID']
+
+            sshots = [disk['SNAPSHOT']].flatten
+            sshots.each do |snapshot|
+                data = snapshot.merge({ 'DISK_ID' => disk_id })
+                snapshots << data
+            end
+        end
+
+        # get monitoring data
+        snapshots.each do |snapshot|
+            disk_id = snapshot["DISK_ID"]
+            snap_id = snapshot["ID"]
+            xpath = "MONITORING/SNAPSHOT_SIZE[ID='#{snap_id}' and DISK_ID='#{disk_id}']/SIZE"
+            snapshot["MONITOR_SIZE"] = vm[xpath]
+        end
+
+        table.show(snapshots)
+    end
 end
