@@ -1,5 +1,5 @@
 # ---------------------------------------------------------------------------- #
-# Copyright 2010-2014, C12G Labs S.L                                           #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                  #
 #                                                                              #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may      #
 # not use this file except in compliance with the License. You may obtain      #
@@ -41,12 +41,12 @@ module Market
         end
 
         def get(path)
-            req = Net::HTTP::Proxy(@host, @port)::Get.new(path)
+            req = Net::HTTP::Proxy(@host, @port)::Get.new(@uri.path + path)
             do_request(req)
         end
 
         def post(path, body)
-            req = Net::HTTP::Proxy(@host, @port)::Post.new(path)
+            req = Net::HTTP::Proxy(@host, @port)::Post.new(@uri.path + path)
             req.body = body
 
             do_request(req)
