@@ -1,5 +1,5 @@
 /* ------------------------------------------------------------------------ */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs      */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems              */
 /*                                                                          */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may  */
 /* not use this file except in compliance with the License. You may obtain  */
@@ -152,7 +152,7 @@ int Cluster::del_datastore(int id, string& error_msg)
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int Cluster::get_default_sysetm_ds(const set<int>& ds_set)
+int Cluster::get_default_system_ds(const set<int>& ds_set)
 {
     Nebula& nd = Nebula::instance();
 
