@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -27,6 +27,9 @@
 #include <sstream>
 #include <iomanip>
 #include <algorithm>
+#include <math.h>
+#include <sys/types.h>
+#include <regex.h>
 
 using namespace std;
 
@@ -238,3 +241,45 @@ vector<string> one_util::split(const string& st, char delim, bool clean_empty)
 
     return parts;
 }
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+string one_util::float_to_str(const float &num)
+{
+    ostringstream oss;
+
+    if ( num == ceil(num) )
+    {
+        oss.precision(0);
+    }
+    else
+    {
+        oss.precision(2);
+    }
+
+    oss << fixed << num;
+
+    return oss.str();
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int one_util::regex_match(const char *pattern, const char *subject)
+{
+    int rc;
+    regex_t re;
+
+    rc = regcomp(&re, pattern, REG_EXTENDED|REG_NOSUB);
+
+    if (rc != 0)
+    {
+        return(rc);
+    }
+
+    rc = regexec(&re, subject, 0, 0, 0);
+    regfree(&re);
+
+    return rc;
+}
