@@ -1,7 +1,7 @@
 #!/usr/bin/env ruby
 
 # -------------------------------------------------------------------------- */
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 # not use this file except in compliance with the License. You may obtain    */
 # a copy of the License at                                                   */
@@ -50,7 +50,10 @@ class DatastoreDriver < OpenNebulaDriver
         :log     => "LOG",
         :stat    => "STAT",
         :clone   => "CLONE",
-        :monitor => "MONITOR"
+        :monitor => "MONITOR",
+        :snap_delete => "SNAP_DELETE",
+        :snap_revert => "SNAP_REVERT",
+        :snap_flatten=> "SNAP_FLATTEN"
     }
 
     # Register default actions for the protocol
@@ -65,7 +68,10 @@ class DatastoreDriver < OpenNebulaDriver
                 ACTION[:rm]      => nil,
                 ACTION[:mkfs]    => nil,
                 ACTION[:clone]   => nil,
-                ACTION[:monitor] => nil
+                ACTION[:monitor] => nil,
+                ACTION[:snap_delete] => nil,
+                ACTION[:snap_revert] => nil,
+                ACTION[:snap_flatten] => nil
             }
         }.merge!(options)
 
@@ -87,6 +93,9 @@ class DatastoreDriver < OpenNebulaDriver
         register_action(ACTION[:stat].to_sym, method("stat"))
         register_action(ACTION[:clone].to_sym, method("clone"))
         register_action(ACTION[:monitor].to_sym, method("monitor"))
+        register_action(ACTION[:snap_delete].to_sym, method("snap_delete"))
+        register_action(ACTION[:snap_revert].to_sym, method("snap_revert"))
+        register_action(ACTION[:snap_flatten].to_sym, method("snap_flatten"))
     end
 
     ############################################################################
@@ -123,6 +132,21 @@ class DatastoreDriver < OpenNebulaDriver
         do_image_action(id, ds, :monitor, "#{drv_message} #{id}", true)
     end
 
+    def snap_delete(id, drv_message)
+        ds = get_ds_type(drv_message)
+        do_image_action(id, ds, :snap_delete, "#{drv_message} #{id}")
+    end
+
+    def snap_revert(id, drv_message)
+        ds = get_ds_type(drv_message)
+        do_image_action(id, ds, :snap_revert, "#{drv_message} #{id}")
+    end
+
+    def snap_flatten(id, drv_message)
+        ds = get_ds_type(drv_message)
+        do_image_action(id, ds, :snap_flatten, "#{drv_message} #{id}")
+    end
+
     private
 
     def is_available?(ds, id, action)
