@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2010-2014, C12G Labs S.L.                                        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -15,27 +15,34 @@
 #--------------------------------------------------------------------------- #
 
 require 'treetop'
+require 'treetop/version'
 require 'grammar'
 require 'parse-cron'
 
+if !(Gem::Version.create(Treetop::VERSION::STRING) >= Gem::Version.create('1.6.3'))
+    raise "treetop gem version must be >= 1.6.3. Current version is #{Treetop::VERSION::STRING}"
+end
+
 module OpenNebula
     class Role
 
         # Actions that can be performed on the VMs of a given Role
         SCHEDULE_ACTIONS = [
             'shutdown',
-            'delete',
+            'shutdown-hard',
+            'undeploy',
+            'undeploy-hard',
             'hold',
             'release',
             'stop',
-            'shutdown-hard',
             'suspend',
             'resume',
-            'boot',
+            'delete',
             'delete-recreate',
             'reboot',
             'reboot-hard',
             'poweroff',
+            'poweroff-hard',
             'snapshot-create'
         ]
 
@@ -205,7 +212,8 @@ module OpenNebula
                     vm_state = node['vm_info']['VM']['STATE']
                     lcm_state = node['vm_info']['VM']['LCM_STATE']
 
-                    running = vm_state == '3' && lcm_state >= '3'
+                    running = (!Role.vm_failure?(vm_state, lcm_state) &&
+                                vm_state == '3' && lcm_state >= '3')
 
                     if running && @service.ready_status_gate
                         running_status = node['vm_info']['VM']['USER_TEMPLATE']['READY'] || ""
@@ -458,6 +466,35 @@ module OpenNebula
             end
         end
 
+        # Returns true if the VM state is failure
+        # @param [Integer] vm_state VM state
+        # @param [Integer] lcm_state VM LCM state
+        # @return [true,false] True if the lcm state is one of *_FAILURE
+        def self.vm_failure?(vm_state, lcm_state)
+            vm_state_str = VirtualMachine::VM_STATE[vm_state.to_i]
+            lcm_state_str = VirtualMachine::LCM_STATE[lcm_state.to_i]
+
+            if vm_state_str == 'ACTIVE' &&
+                (   lcm_state_str == 'BOOT_FAILURE' ||
+                    lcm_state_str == 'BOOT_MIGRATE_FAILURE' ||
+                    lcm_state_str == 'PROLOG_MIGRATE_FAILURE' ||
+                    lcm_state_str == 'PROLOG_FAILURE' ||
+                    lcm_state_str == 'EPILOG_FAILURE' ||
+                    lcm_state_str == 'EPILOG_STOP_FAILURE' ||
+                    lcm_state_str == 'EPILOG_UNDEPLOY_FAILURE' ||
+                    lcm_state_str == 'PROLOG_MIGRATE_POWEROFF_FAILURE' ||
+                    lcm_state_str == 'PROLOG_MIGRATE_SUSPEND_FAILURE' ||
+                    lcm_state_str == 'BOOT_UNDEPLOY_FAILURE' ||
+                    lcm_state_str == 'BOOT_STOPPED_FAILURE' ||
+                    lcm_state_str == 'PROLOG_RESUME_FAILURE' ||
+                    lcm_state_str == 'PROLOG_UNDEPLOY_FAILURE')
+
+                return true
+            end
+
+            return false
+        end
+
         ########################################################################
         # Scalability
         ########################################################################
@@ -823,7 +860,7 @@ module OpenNebula
             set_cardinality( get_nodes.size() - nodes_dispose.size() )
         end
 
-        # Deletes VMs in DONE or FAILED, and sends a boot action to VMs in UNKNOWN
+        # Deletes VMs in DONE or FAILED, and sends a resume action to VMs in UNKNOWN
         def recover()
 
             nodes = @body['nodes']
@@ -837,13 +874,16 @@ module OpenNebula
                 if node['vm_info'] && node['vm_info']['VM'] && node['vm_info']['VM']['STATE']
                     vm_state = node['vm_info']['VM']['STATE']
                     lcm_state = node['vm_info']['VM']['LCM_STATE']
+
+                    vm_state_str = VirtualMachine::VM_STATE[vm_state.to_i]
+                    lcm_state_str = VirtualMachine::LCM_STATE[lcm_state.to_i]
                 end
 
                 if vm_state == '6' # DONE
                     # Store the VM id in the array of disposed nodes
                     disposed_nodes << vm_id
 
-                elsif vm_state == '7' # FAILED
+                elsif ( Role.vm_failure?(vm_state, lcm_state) )
                     vm = OpenNebula::VirtualMachine.new_with_id(vm_id, @service.client)
                     rc = vm.finalize
 
@@ -861,9 +901,11 @@ module OpenNebula
 
                         new_nodes << node
                     end
-                elsif vm_state == '3' && lcm_state == '16' # UNKNOWN
+                elsif (vm_state == '3' && lcm_state == '16') # UNKNOWN
                     vm = OpenNebula::VirtualMachine.new_with_id(vm_id, @service.client)
-                    vm.boot
+                    vm.resume
+
+                    new_nodes << node
                 else
                     new_nodes << node
                 end
