@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2010-2014, C12G Labs S.L.                                        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -276,8 +276,8 @@ protected
                 vm_state = node['vm_info']['VM']['STATE']
                 lcm_state = node['vm_info']['VM']['LCM_STATE']
 
-                # UNKNOWN or FAILED
-                if (vm_state == '3' && lcm_state == '16') || vm_state == '7'
+                # Failure or UNKNOWN
+                if vm_failure?(node) || (vm_state == '3' && lcm_state == '16')
                     return true
                 end
             end
@@ -291,12 +291,8 @@ protected
     # @return [true|false]
     def any_node_failed?(role)
         role.get_nodes.each { |node|
-            if node && node['vm_info']
-                vm_state = node['vm_info']['VM']['STATE']
-
-                if vm_state == '7' # FAILED
-                    return true
-                end
+            if vm_failure?(node)
+                return true
             end
         }
 
@@ -327,9 +323,9 @@ protected
     # @return [true|false]
     def any_node_failed_scaling?(role)
         role.get_nodes.each { |node|
-            if node && node['vm_info'] &&
+            if  node && node['vm_info'] &&
                 (node['disposed'] == '1' || node['scale_up'] == '1') &&
-                node['vm_info']['VM']['STATE'] == '7' # FAILED
+                vm_failure?(node)
 
                 return true
             end
@@ -356,4 +352,14 @@ protected
         # For scale down, it will finish when scaling nodes are deleted
         return role.get_nodes.size() == role.cardinality()
     end
+
+    def vm_failure?(node)
+        if node && node['vm_info']
+            return Role.vm_failure?(
+                vm_state = node['vm_info']['VM']['STATE'],
+                lcm_state = node['vm_info']['VM']['LCM_STATE'])
+        end
+
+        return false
+    end
 end
