@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -19,6 +19,7 @@
 /* ************************************************************************** */
 
 #include "ImagePool.h"
+#include "Snapshots.h"
 #include "AuthManager.h"
 #include "Nebula.h"
 #include "PoolObjectAuth.h"
@@ -306,14 +307,15 @@ int ImagePool::get_disk_id(const string& id_s)
 
 /* -------------------------------------------------------------------------- */
 
-int ImagePool::disk_attribute(int               vm_id,
-                              VectorAttribute * disk,
-                              int               disk_id,
-                              Image::ImageType& img_type,
-                              string&           dev_prefix,
-                              int               uid,
-                              int&              image_id,
-                              string&           error_str)
+int ImagePool::acquire_disk(int               vm_id,
+                            VectorAttribute * disk,
+                            int               disk_id,
+                            Image::ImageType& img_type,
+                            string&           dev_prefix,
+                            int               uid,
+                            int&              image_id,
+                            Snapshots **      snap,
+                            string&           error_str)
 {
     string  source;
     Image * img = 0;
@@ -326,6 +328,8 @@ int ImagePool::disk_attribute(int               vm_id,
     Nebula&         nd      = Nebula::instance();
     ImageManager *  imagem  = nd.get_imagem();
 
+    *snap = 0;
+
     if (!(source = disk->vector_value("IMAGE")).empty())
     {
         int uiid = get_disk_uid(disk,uid);
@@ -406,22 +410,62 @@ int ImagePool::disk_attribute(int               vm_id,
         DatastorePool * ds_pool = nd.get_dspool();
         Datastore *     ds;
 
+        long long size = 0;
+        bool has_size = (disk->vector_value("SIZE", size) == 0);
+
+        if (has_size && img->is_persistent() && size != img->get_size())
+        {
+            img->unlock();
+
+            imagem->release_image(vm_id, iid, false);
+
+            oss << "SIZE attribute is not supported for persistent image ["
+                << img->get_oid() << "].";
+            error_str = oss.str();
+
+            return -1;
+        }
+
+        if (has_size && img->get_type() == Image::CDROM && size != img->get_size())
+        {
+            img->unlock();
+
+            imagem->release_image(vm_id, iid, false);
+
+            oss << "SIZE attribute is not supported for CDROM image ["
+                << img->get_oid() << "].";
+            error_str = oss.str();
+
+            return -1;
+        }
+
+        if (has_size && size < img->get_size())
+        {
+            img->unlock();
+
+            imagem->release_image(vm_id, iid, false);
+
+            oss << "SIZE of " << size << "MB is less than the image ["
+                << img->get_oid() << "] size of " << img->get_size() << "MB.";
+            error_str = oss.str();
+
+            return -1;
+        }
+
         iid = img->get_oid();
-        rc  = img->disk_attribute(disk, img_type, dev_prefix, inherit_image_attrs);
+        img->disk_attribute(disk, img_type, dev_prefix, inherit_image_attrs);
 
         image_id     = img->get_oid();
         datastore_id = img->get_ds_id();
 
-        img->unlock();
-
-        if (rc == -1)
+        if (img->snapshots.size() > 0)
         {
-            imagem->release_image(vm_id, iid, false);
-            error_str = "Unknown internal error";
-
-            return -1;
+            *snap = new Snapshots(img->snapshots);
+            (*snap)->set_disk_id(disk_id);
         }
 
+        img->unlock();
+
         ds = ds_pool->get(datastore_id, true);
 
         if ( ds == 0 )
@@ -429,6 +473,9 @@ int ImagePool::disk_attribute(int               vm_id,
             imagem->release_image(vm_id, iid, false);
             error_str = "Associated datastore for the image does not exist";
 
+            delete *snap;
+            *snap = 0;
+
             return -1;
         }
 
@@ -437,8 +484,7 @@ int ImagePool::disk_attribute(int               vm_id,
         ds->unlock();
     }
 
-    oss << disk_id;
-    disk->replace("DISK_ID",oss.str());
+    disk->replace("DISK_ID", disk_id);
 
     return rc;
 }
@@ -446,11 +492,71 @@ int ImagePool::disk_attribute(int               vm_id,
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+void ImagePool::disk_attribute(
+        VectorAttribute *   disk,
+        int                 disk_id,
+        int                 uid)
+{
+    string  source;
+    Image * img = 0;
+    int     datastore_id;
+
+    string           dev_prefix;
+    Image::ImageType img_type;
+
+    ostringstream oss;
+
+    Nebula&         nd      = Nebula::instance();
+    DatastorePool * ds_pool = nd.get_dspool();
+    Datastore *     ds;
+
+    if (!(source = disk->vector_value("IMAGE")).empty())
+    {
+        int uiid = get_disk_uid(disk,uid);
+
+        if ( uiid != -1)
+        {
+            img = get(source, uiid, true);
+        }
+    }
+    else if (!(source = disk->vector_value("IMAGE_ID")).empty())
+    {
+        int iid = get_disk_id(source);
+
+        if ( iid != -1)
+        {
+            img = get(iid, true);
+        }
+    }
+
+    if ( img != 0 )
+    {
+        img->disk_attribute(disk, img_type, dev_prefix, inherit_image_attrs);
+
+        datastore_id = img->get_ds_id();
+
+        img->unlock();
+
+        ds = ds_pool->get(datastore_id, true);
+
+        if ( ds != 0 )
+        {
+            ds->disk_attribute(disk, inherit_datastore_attrs);
+
+            ds->unlock();
+        }
+    }
+
+    disk->replace("DISK_ID", disk_id);
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 void ImagePool::authorize_disk(VectorAttribute * disk,int uid, AuthRequest * ar)
 {
     string          source;
     Image *         img = 0;
-
     PoolObjectAuth  perm;
 
     if (!(source = disk->vector_value("IMAGE")).empty())
