@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -21,8 +21,5 @@
 
 vector<string> ImageTemplate::restricted_attributes;
 
-string ImageTemplate::saving_attribute     = "SAVE_AS";
-string ImageTemplate::saving_hot_attribute = "SAVE_AS_HOT";
-
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
