@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -18,42 +18,43 @@ export LANG=C
 
 # Paths for utilities
 export PATH=/bin:/sbin:/usr/bin:$PATH
-AWK=awk
-BASH=bash
-CUT=cut
-DATE=date
-DD=dd
-DF=df
-DU=du
-GREP=grep
-ISCSIADM=iscsiadm
-LVCREATE=lvcreate
-LVREMOVE=lvremove
-LVRENAME=lvrename
-LVS=lvs
-LN=ln
-MD5SUM=md5sum
-MKFS=mkfs
-MKISOFS=genisoimage
-MKSWAP=mkswap
-QEMU_IMG=qemu-img
-RADOS=rados
-RBD=rbd
-READLINK=readlink
-RM=rm
-SCP=scp
-SED=sed
-SSH=ssh
-SUDO=sudo
-SYNC=sync
-TAR=tar
-TGTADM=tgtadm
-TGTADMIN=tgt-admin
-TGTSETUPLUN=tgt-setup-lun-one
-TR=tr
-VGDISPLAY=vgdisplay
-VMKFSTOOLS=vmkfstools
-WGET=wget
+AWK=${AWK:-awk}
+BASH=${BASH:-bash}
+CUT=${CUT:-cut}
+CEPH=${CEPH:-ceph}
+DATE=${DATE:-date}
+DD=${DD:-dd}
+DF=${DF:-df}
+DU=${DU:-du}
+GREP=${GREP:-grep}
+ISCSIADM=${ISCSIADM:-iscsiadm}
+LVCREATE=${LVCREATE:-lvcreate}
+LVREMOVE=${LVREMOVE:-lvremove}
+LVRENAME=${LVRENAME:-lvrename}
+LVS=${LVS:-lvs}
+LN=${LN:-ln}
+MD5SUM=${MD5SUM:-md5sum}
+MKFS=${MKFS:-mkfs}
+MKISOFS=${MKISOFS:-genisoimage}
+MKSWAP=${MKSWAP:-mkswap}
+QEMU_IMG=${QMEMU_IMG:-qemu-img}
+RADOS=${RADOS:-rados}
+RBD=${RBD:-rbd}
+READLINK=${READLINK:-readlink}
+RM=${RM:-rm}
+SCP=${SCP:-scp}
+SED=${SED:-sed}
+SSH=${SSH:-ssh}
+SUDO=${SUDO:-sudo}
+SYNC=${SYNC:-sync}
+TAR=${TAR:-tar}
+TGTADM=${TGTADM:-tgtadm}
+TGTADMIN=${TGTADMIN:-tgt-admin}
+TGTSETUPLUN=${TGTSETUPLUN:-tgt-setup-lun-one}
+TR=${TR:-tr}
+VGDISPLAY=${VGDISPLAY:-vgdisplay}
+VMKFSTOOLS=${VMKFSTOOLS:-vmkfstools}
+WGET=${WGET:-wget}
 
 if [ "x$(uname -s)" = "xLinux" ]; then
     SED="$SED -r"
@@ -148,6 +149,33 @@ function exec_and_log
     fi
 }
 
+# This function executes $1 and returns stdout
+# If a second parameter is present it is used as the error message when
+# the command fails
+function monitor_and_log
+{
+    EXEC_OUT=`bash -s 2>/dev/null <<EOF
+export LANG=C
+export LC_ALL=C
+set -xv
+$1
+EOF`
+    EXEC_RC=$?
+
+    if [ $EXEC_RC -ne 0 ]; then
+
+        if [ -n "$2" ]; then
+            log_error "Command \"$2\" failed: $EXEC_OUT"
+        else
+            log_error "Command \"$1\" failed: $EXEC_OUT"
+        fi
+
+        exit $EXEC_RC
+    fi
+
+    echo $EXEC_OUT
+}
+
 # Executes a command, if it fails returns error message and exits. Similar to
 # exec_and_log, except that it allows multiline commands.
 # If a second parameter is present it is used as the error message when
@@ -292,7 +320,7 @@ function mkfs_command {
             OPTS="-F"
             ;;
 
-        "reiserfs")
+        "reiserfs"|"xfs")
             OPTS="-f -q"
             ;;
 
@@ -327,7 +355,9 @@ function mkfs_command {
             return 0
             ;;
         *)
-            OPTS=""
+            echo ""
+            echo "Filesystem '$FSTYPE' not valid." 1>&2
+            return 1
             ;;
     esac
 
@@ -357,7 +387,39 @@ EOF`
     fi
 }
 
-#This function executes $2 at $1 host and returns stdout
+# Remote command execution over SSH preloading a local file
+# $1: HOSTNAME
+# $2: COMMAND
+# $3: file to be loaded into the script
+# $4: ERROR_REPORT
+function ssh_exec_and_log_stdin
+{
+    SSH_EXEC_ERR=`$SSH $1 sh -s 2>&1 1>/dev/null <<EOF
+export LANG=C
+export LC_ALL=C
+$(cat $3)
+
+$2
+EOF`
+
+    SSH_EXEC_RC=$?
+
+    if [ $SSH_EXEC_RC -ne 0 ]; then
+        log_error "Command \"$2\" failed: $SSH_EXEC_ERR"
+
+        if [ -n "$4" ]; then
+            error_message "$4"
+        else
+            error_message "Error executing $2: $SSH_EXEC_ERR"
+        fi
+
+        exit $SSH_EXEC_RC
+    fi
+}
+
+# This function executes $2 at $1 host and returns stdout
+# If $3 is present, it is used as the error message when
+# the command fails
 function ssh_monitor_and_log
 {
     SSH_EXEC_OUT=`$SSH $1 sh -s 2>/dev/null <<EOF
@@ -368,8 +430,12 @@ EOF`
     SSH_EXEC_RC=$?
 
     if [ $SSH_EXEC_RC -ne 0 ]; then
-        log_error "Command \"$2\" failed: $SSH_EXEC_OUT"
-        error_message "Cannot monitor $1"
+
+        if [ -n "$3" ]; then
+            log_error "Command \"$3\" failed: $SSH_EXEC_OUT"
+        else
+            log_error "Command \"$2\" failed: $SSH_EXEC_OUT"
+        fi
 
         exit $SSH_EXEC_RC
     fi
