@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -105,6 +105,7 @@ void OpenNebulaTemplate::set_conf_default()
 #  VM_INDIVIDUAL_MONITORING
 #  VM_PER_INTERVAL
 #  VM_MONITORING_EXPIRATION_TIME
+#  LISTEN_ADDRESS
 #  PORT
 #  DB
 #  VNC_BASE_PORT
@@ -160,6 +161,12 @@ void OpenNebulaTemplate::set_conf_default()
     attribute = new SingleAttribute("PORT",value);
     conf_default.insert(make_pair(attribute->name(),attribute));
 
+    //XML-RPC Server listen address
+    value = "0.0.0.0";
+
+    attribute = new SingleAttribute("LISTEN_ADDRESS",value);
+    conf_default.insert(make_pair(attribute->name(),attribute));
+
     //DB CONFIGURATION
     vvalue.insert(make_pair("BACKEND","sqlite"));
 
@@ -172,12 +179,6 @@ void OpenNebulaTemplate::set_conf_default()
     attribute = new SingleAttribute("VNC_BASE_PORT",value);
     conf_default.insert(make_pair(attribute->name(),attribute));
 
-    //XML-RPC Server PORT
-    value = "2633";
-
-    attribute = new SingleAttribute("PORT",value);
-    conf_default.insert(make_pair(attribute->name(),attribute));
-
     //SCRIPTS_REMOTE_DIR
     value = "/var/tmp/one";
 
@@ -215,6 +216,20 @@ void OpenNebulaTemplate::set_conf_default()
 
     vattribute = new VectorAttribute("FEDERATION",vvalue);
     conf_default.insert(make_pair(vattribute->name(),vattribute));
+
+/*
+#*******************************************************************************
+# Default showback cost
+#*******************************************************************************
+*/
+    vvalue.clear();
+    vvalue.insert(make_pair("CPU_COST","0"));
+    vvalue.insert(make_pair("MEMORY_COST","0"));
+    vvalue.insert(make_pair("DISK_COST","0"));
+
+    vattribute = new VectorAttribute("DEFAULT_COST",vvalue);
+    conf_default.insert(make_pair(vattribute->name(),vattribute));
+
 /*
 #*******************************************************************************
 # XML-RPC server configuration
@@ -343,11 +358,18 @@ void OpenNebulaTemplate::set_conf_default()
 #*******************************************************************************
 # Auth Manager Configuration
 #*******************************************************************************
+# DEFAULT_AUTH
 # SESSION_EXPIRATION_TIME
 # ENABLE_OTHER_PERMISSIONS
 # DEFAULT_UMASK
 #*******************************************************************************
 */
+    // DEFAULT_AUTH
+    value = "default";
+
+    attribute = new SingleAttribute("DEFAULT_AUTH",value);
+    conf_default.insert(make_pair(attribute->name(),attribute));
+
     // SESSION_EXPIRATION_TIME
     value = "0";
 
