@@ -1,7 +1,7 @@
 #!/bin/bash
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -78,7 +78,7 @@ do_documentation()
 {
     echo "Generating javadocs..."
 
-    rm -rf $DOC_DIR > /dev/null 2>&1 
+    rm -rf $DOC_DIR > /dev/null 2>&1
     mkdir -p $DOC_DIR
     javadoc -quiet -classpath $LIB_DIR"/*" -d $DOC_DIR \
       -sourcepath ./src/ \
@@ -87,8 +87,8 @@ do_documentation()
       -doctitle 'OpenNebula Cloud API Specification' \
       -header '<b>OpenNebula</b><br><font size="-1">Cloud API</font>' \
       -bottom 'Visit <a
-href="http://opennebula.org/">OpenNebula.org</a><br>Copyright 2002-2014 &copy;
-OpenNebula Project (OpenNebula.org), C12G Labs.'
+href="http://opennebula.org/">OpenNebula.org</a><br>Copyright 2002-2015 &copy;
+OpenNebula Project, OpenNebula Systems.'
 }
 
 do_jar()
