@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs
+ * Copyright 2002-2015, OpenNebula Project, OpenNebula Systems
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -69,6 +69,8 @@ public class Acl extends PoolElement{
         tmpResources.put("CLUSTER"  , 0x0000200000000000L);
         tmpResources.put("DOCUMENT" , 0x0000400000000000L);
         tmpResources.put("ZONE"     , 0x0000800000000000L);
+        tmpResources.put("SECGROUP" , 0x0001000000000000L);
+        tmpResources.put("VDC"      , 0x0002000000000000L);
 
         RESOURCES = Collections.unmodifiableMap(tmpResources);
 
