@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs
+ * Copyright 2002-2015, OpenNebula Project, OpenNebula Systems
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -39,6 +39,9 @@ public class Image extends PoolElement
     private static final String CHTYPE      = METHOD_PREFIX + "chtype";
     private static final String CLONE       = METHOD_PREFIX + "clone";
     private static final String RENAME      = METHOD_PREFIX + "rename";
+    private static final String SNAPSHOTDELETE  = METHOD_PREFIX + "snapshotdelete";
+    private static final String SNAPSHOTREVERT  = METHOD_PREFIX + "snapshotrevert";
+    private static final String SNAPSHOTFLATTEN = METHOD_PREFIX + "snapshotflatten";
 
     private static final String[] IMAGE_STATES =
         {"INIT", "READY", "USED", "DISABLED", "LOCKED",
@@ -282,6 +285,43 @@ public class Image extends PoolElement
         return client.call(RENAME, id, name);
     }
 
+    /**
+     * Deletes Image from snapshot
+     *
+     * @param client XML-RPC Client.
+     * @param id The Image id of the target Image.
+     * @param snapId ID of the snapshot to delete
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse snapshotDelete(Client client, int id, int snapId)
+    {
+        return client.call(SNAPSHOTDELETE, id, snapId);
+    }
+
+    /**
+     * Reverts Image state to a previous snapshot
+     * @param client XML-RPC Client.
+     * @param id The Image id of the target Image.
+     * @param snapId ID of the snapshot to revert to
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse snapshotRevert(Client client, int id, int snapId)
+    {
+        return client.call(SNAPSHOTREVERT, id, snapId);
+    }
+
+    /**
+     * Flattens an image snapshot
+     * @param client XML-RPC Client.
+     * @param id The Image id of the target Image.
+     * @param snapId ID of the snapshot to flatten
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse snapshotFlatten(Client client, int id, int snapId)
+    {
+        return client.call(SNAPSHOTFLATTEN, id, snapId);
+    }
+
     // =================================
     // Instanced object XML-RPC methods
     // =================================
@@ -550,6 +590,37 @@ public class Image extends PoolElement
         return rename(client, id, name);
     }
 
+    /**
+     * Deletes Image from snapshot
+     *
+     * @param snapId ID of the snapshot to delete
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse snapshotDelete(int snapId)
+    {
+        return snapshotDelete(client, id, snapId);
+    }
+
+    /**
+     * Reverts Image state to a previous snapshot
+     * @param snapId ID of the snapshot to revert to
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse snapshotRevert(int snapId)
+    {
+        return snapshotRevert(client, id, snapId);
+    }
+
+    /**
+     * Flattens an image snapshot
+     * @param snapId ID of the snapshot to flatten
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse snapshotFlatten(int snapId)
+    {
+        return snapshotFlatten(client, id, snapId);
+    }
+
     // =================================
     // Helpers
     // =================================
