@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs
+ * Copyright 2002-2015, OpenNebula Project, OpenNebula Systems
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -84,7 +84,22 @@ public class Template extends PoolElement
      */
     public static OneResponse info(Client client, int id)
     {
-        return client.call(INFO, id);
+        return info(client, id, false);
+    }
+
+    /**
+     * Retrieves the information of the given Template.
+     *
+     * @param client XML-RPC Client.
+     * @param id The template id for the template to retrieve the information from
+     * @param extended optional flag to process the template and include
+     * extended information, such as the SIZE for each DISK
+     * @return If successful the message contains the string
+     * with the information returned by OpenNebula.
+     */
+    public static OneResponse info(Client client, int id, boolean extended)
+    {
+        return client.call(INFO, id, extended);
     }
 
     /**
@@ -131,7 +146,7 @@ public class Template extends PoolElement
 
     /**
      * Changes the owner/group
-     * 
+     *
      * @param client XML-RPC Client.
      * @param id The template id of the target template we want to modify.
      * @param uid The new owner user ID. Set it to -1 to leave the current one.
@@ -145,7 +160,7 @@ public class Template extends PoolElement
 
     /**
      * Changes the template permissions
-     * 
+     *
      * @param client XML-RPC Client.
      * @param id The template id of the target template.
      * @param owner_u 1 to allow, 0 deny, -1 do not change
@@ -172,7 +187,7 @@ public class Template extends PoolElement
 
     /**
      * Changes the permissions
-     * 
+     *
      * @param client XML-RPC Client.
      * @param id The id of the target object.
      * @param octet Permissions octed , e.g. 640
@@ -185,7 +200,7 @@ public class Template extends PoolElement
 
     /**
      * Changes the permissions
-     * 
+     *
      * @param client XML-RPC Client.
      * @param id The id of the target object.
      * @param octet Permissions octed , e.g. 640
@@ -198,7 +213,7 @@ public class Template extends PoolElement
 
     /**
      * Creates a VM instance from a Template
-     * 
+     *
      * @param client XML-RPC Client.
      * @param id The template id of the target template.
      * @param name A string containing the name of the VM instance, can be empty.
@@ -251,7 +266,20 @@ public class Template extends PoolElement
      */
     public OneResponse info()
     {
-        OneResponse response = info(client, id);
+        return info(false);
+    }
+
+    /**
+     * Retrieves the information of the given Template.
+     *
+     * @param extended optional flag to process the template and include
+     * extended information, such as the SIZE for each DISK
+     * @return If successful the message contains the string
+     * with the information returned by OpenNebula.
+     */
+    public OneResponse info(boolean extended)
+    {
+        OneResponse response = info(client, id, extended);
         super.processInfo(response);
         return response;
     }
@@ -322,7 +350,7 @@ public class Template extends PoolElement
 
     /**
      * Changes the owner/group
-     * 
+     *
      * @param uid The new owner user ID. Set it to -1 to leave the current one.
      * @param gid The new group ID. Set it to -1 to leave the current one.
      * @return If an error occurs the error message contains the reason.
@@ -334,7 +362,7 @@ public class Template extends PoolElement
 
     /**
      * Changes the owner
-     * 
+     *
      * @param uid The new owner user ID.
      * @return If an error occurs the error message contains the reason.
      */
@@ -345,7 +373,7 @@ public class Template extends PoolElement
 
     /**
      * Changes the group
-     * 
+     *
      * @param gid The new group ID.
      * @return If an error occurs the error message contains the reason.
      */
@@ -356,7 +384,7 @@ public class Template extends PoolElement
 
     /**
      * Changes the template permissions
-     * 
+     *
      * @param owner_u 1 to allow, 0 deny, -1 do not change
      * @param owner_m 1 to allow, 0 deny, -1 do not change
      * @param owner_a 1 to allow, 0 deny, -1 do not change
@@ -402,7 +430,7 @@ public class Template extends PoolElement
 
     /**
      * Creates a VM instance from a Template
-     * 
+     *
      * @param name A string containing the name of the VM instance, can be empty.
      * @param onHold False to create this VM in pending state, true on hold
      * @param template User provided Template to merge with the one
@@ -416,7 +444,7 @@ public class Template extends PoolElement
 
     /**
      * Creates a VM instance from a Template
-     * 
+     *
      * @param name A string containing the name of the VM instance, can be empty.
      * @return If successful the message contains the VM Instance ID.
      */
@@ -427,7 +455,7 @@ public class Template extends PoolElement
 
     /**
      * Creates a VM instance from a Template
-     * 
+     *
      * @return If successful the message contains the VM Instance ID.
      */
     public OneResponse instantiate()
