@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs
+ * Copyright 2002-2015, OpenNebula Project, OpenNebula Systems
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -33,7 +33,6 @@ public class VirtualMachine extends PoolElement{
     private static final String DEPLOY   = METHOD_PREFIX + "deploy";
     private static final String ACTION   = METHOD_PREFIX + "action";
     private static final String MIGRATE  = METHOD_PREFIX + "migrate";
-    private static final String SAVEDISK = METHOD_PREFIX + "savedisk";
     private static final String CHOWN    = METHOD_PREFIX + "chown";
     private static final String CHMOD    = METHOD_PREFIX + "chmod";
     private static final String MONITORING = METHOD_PREFIX + "monitoring";
@@ -48,6 +47,10 @@ public class VirtualMachine extends PoolElement{
     private static final String SNAPSHOTREVERT = METHOD_PREFIX + "snapshotrevert";
     private static final String SNAPSHOTDELETE = METHOD_PREFIX + "snapshotdelete";
     private static final String RECOVER = METHOD_PREFIX + "recover";
+    private static final String DISKSAVEAS          = METHOD_PREFIX + "disksaveas";
+    private static final String DISKSNAPSHOTCREATE  = METHOD_PREFIX + "disksnapshotcreate";
+    private static final String DISKSNAPSHOTREVERT  = METHOD_PREFIX + "disksnapshotrevert";
+    private static final String DISKSNAPSHOTDELETE  = METHOD_PREFIX + "disksnapshotdelete";
 
     private static final String[] VM_STATES =
     {
@@ -109,43 +112,99 @@ public class VirtualMachine extends PoolElement{
         "SHUTDOWN_UNDEPLOY",
         "EPILOG_UNDEPLOY",
         "PROLOG_UNDEPLOY",
-        "BOOT_UNDEPLOY" };
+        "BOOT_UNDEPLOY",
+        "HOTPLUG_PROLOG_POWEROFF",
+        "HOTPLUG_EPILOG_POWEROFF",
+        "BOOT_MIGRATE",
+        "BOOT_FAILURE",
+        "BOOT_MIGRATE_FAILURE",
+        "PROLOG_MIGRATE_FAILURE",
+        "PROLOG_FAILURE",
+        "EPILOG_FAILURE",
+        "EPILOG_STOP_FAILURE",
+        "EPILOG_UNDEPLOY_FAILURE",
+        "PROLOG_MIGRATE_POWEROFF",
+        "PROLOG_MIGRATE_POWEROFF_FAILURE",
+        "PROLOG_MIGRATE_SUSPEND",
+        "PROLOG_MIGRATE_SUSPEND_FAILURE",
+        "BOOT_UNDEPLOY_FAILURE",
+        "BOOT_STOPPED_FAILURE",
+        "PROLOG_RESUME_FAILURE",
+        "PROLOG_UNDEPLOY_FAILURE",
+        "DISK_SNAPSHOT_POWEROFF",
+        "DISK_SNAPSHOT_REVERT_POWEROFF",
+        "DISK_SNAPSHOT_DELETE_POWEROFF",
+        "DISK_SNAPSHOT_SUSPENDED",
+        "DISK_SNAPSHOT_REVERT_SUSPENDED",
+        "DISK_SNAPSHOT_DELETE_SUSPENDED",
+        "DISK_SNAPSHOT",
+        "DISK_SNAPSHOT_REVERT",
+        "DISK_SNAPSHOT_DELETE"
+    };
 
     private static final String[] SHORT_LCM_STATES =
     {
-        null,
-        "prol",
-        "boot",
-        "runn",
-        "migr",
-        "save",
-        "save",
-        "save",
-        "migr",
-        "prol",
-        "epil",
-        "epil",
-        "shut",
-        "shut",
-        "fail",
-        "clea",
-        "unkn",
-        "hotp",
-        "shut",
-        "boot",
-        "boot",
-        "boot",
-        "boot",
-        "clea",
-        "snap",
-        "hotp",
-        "hotp",
-        "hotp",
-        "hotp",
-        "shut",
-        "epil",
-        "prol",
-        "boot" };
+        "",         // LCM_INIT
+        "prol",     // PROLOG
+        "boot",     // BOOT
+        "runn",     // RUNNING
+        "migr",     // MIGRATE
+        "save",     // SAVE_STOP
+        "save",     // SAVE_SUSPEND
+        "save",     // SAVE_MIGRATE
+        "migr",     // PROLOG_MIGRATE
+        "prol",     // PROLOG_RESUME
+        "epil",     // EPILOG_STOP
+        "epil",     // EPILOG
+        "shut",     // SHUTDOWN
+        "shut",     // CANCEL
+        "fail",     // FAILURE
+        "clea",     // CLEANUP_RESUBMIT
+        "unkn",     // UNKNOWN
+        "hotp",     // HOTPLUG
+        "shut",     // SHUTDOWN_POWEROFF
+        "boot",     // BOOT_UNKNOWN
+        "boot",     // BOOT_POWEROFF
+        "boot",     // BOOT_SUSPENDED
+        "boot",     // BOOT_STOPPED
+        "clea",     // CLEANUP_DELETE
+        "snap",     // HOTPLUG_SNAPSHOT
+        "hotp",     // HOTPLUG_NIC
+        "hotp",     // HOTPLUG_SAVEAS
+        "hotp",     // HOTPLUG_SAVEAS_POWEROFF
+        "hotp",     // HOTPLUG_SAVEAS_SUSPENDED
+        "shut",     // SHUTDOWN_UNDEPLOY
+        "epil",     // EPILOG_UNDEPLOY
+        "prol",     // PROLOG_UNDEPLOY
+        "boot",     // BOOT_UNDEPLOY
+        "hotp",     // HOTPLUG_PROLOG_POWEROFF
+        "hotp",     // HOTPLUG_EPILOG_POWEROFF
+        "boot",     // BOOT_MIGRATE
+        "fail",     // BOOT_FAILURE
+        "fail",     // BOOT_MIGRATE_FAILURE
+        "fail",     // PROLOG_MIGRATE_FAILURE
+        "fail",     // PROLOG_FAILURE
+        "fail",     // EPILOG_FAILURE
+        "fail",     // EPILOG_STOP_FAILURE
+        "fail",     // EPILOG_UNDEPLOY_FAILURE
+        "migr",     // PROLOG_MIGRATE_POWEROFF
+        "fail",     // PROLOG_MIGRATE_POWEROFF_FAILURE
+        "migr",     // PROLOG_MIGRATE_SUSPEND
+        "fail",     // PROLOG_MIGRATE_SUSPEND_FAILURE
+        "fail",     // BOOT_UNDEPLOY_FAILURE
+        "fail",     // BOOT_STOPPED_FAILURE
+        "fail",     // PROLOG_RESUME_FAILURE
+        "fail",     // PROLOG_UNDEPLOY_FAILURE
+        "snap",     // DISK_SNAPSHOT_POWEROFF
+        "snap",     // DISK_SNAPSHOT_REVERT_POWEROFF
+        "snap",     // DISK_SNAPSHOT_DELETE_POWEROFF
+        "snap",     // DISK_SNAPSHOT_SUSPENDED
+        "snap",     // DISK_SNAPSHOT_REVERT_SUSPENDED
+        "snap",     // DISK_SNAPSHOT_DELETE_SUSPENDED
+        "snap",     // DISK_SNAPSHOT
+        "snap",     // DISK_SNAPSHOT_REVERT
+        "snap"      // DISK_SNAPSHOT_DELETE
+    };
 
     /**
      * Creates a new VM representation.
@@ -355,7 +414,7 @@ public class VirtualMachine extends PoolElement{
 
 
     /**
-     * Sets the specified vm's disk to be saved in a new image.
+     * Sets the specified vm's disk to be saved as a new image.
      *
      * @param client XML-RPC Client.
      * @param id The virtual machine id (vid) of the target instance.
@@ -363,18 +422,14 @@ public class VirtualMachine extends PoolElement{
      * @param imageName Name of the new Image that will be created.
      * @param imageType Type of the new image. Set to empty string to use
      * the default type
-     * @param hot True to save the disk immediately, false will perform
-     * the operation when the VM shuts down
-     * @param doTemplate True to clone also the VM originating template
-     * and replace the disk with the saved image
+     * @param snapId ID of the snapshot to save, -1 to use the
+     * current disk image state
      * @return If an error occurs the error message contains the reason.
      */
-    public static OneResponse diskSnapshot(Client client, int id,
-        int diskId, String imageName, String imageType,
-        boolean hot, boolean doTemplate)
+    public static OneResponse diskSaveas(Client client, int id,
+        int diskId, String imageName, String imageType, int snapId)
     {
-        return client.call(SAVEDISK, id ,diskId, imageName, imageType,
-                            hot, doTemplate);
+        return client.call(DISKSAVEAS, id ,diskId, imageName, imageType, snapId);
     }
 
     /**
@@ -457,17 +512,63 @@ public class VirtualMachine extends PoolElement{
         return client.call(SNAPSHOTDELETE, id, snapId);
     }
 
+    /**
+     * Takes a new snapshot of a disk
+     *
+     * @param client XML-RPC Client.
+     * @param id The VM id of the target VM.
+     * @param diskId Id of the disk
+     * @param name description for the snapshot
+     * @return New snapshot Id, or the error message
+     */
+    public static OneResponse diskSnapshotCreate(Client client, int id,
+        int diskId, String name)
+    {
+        return client.call(DISKSNAPSHOTCREATE, id, diskId, name);
+    }
+
+    /**
+     * Reverts disk state to a previously taken snapshot
+     *
+     * @param client XML-RPC Client.
+     * @param id The VM id of the target VM.
+     * @param diskId Id of the disk
+     * @param snapId Id of the snapshot
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse diskSnapshotRevert(Client client, int id,
+        int diskId, int snapId)
+    {
+        return client.call(DISKSNAPSHOTREVERT, id, diskId, snapId);
+    }
+
+    /**
+     * Deletes a disk snapshot
+     *
+     * @param client XML-RPC Client.
+     * @param id The VM id of the target VM.
+     * @param diskId Id of the disk
+     * @param snapId Id of the snapshot
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse diskSnapshotDelete(Client client, int id,
+        int diskId, int snapId)
+    {
+        return client.call(DISKSNAPSHOTDELETE, id, diskId, snapId);
+    }
+
     /**
      * Recovers a stuck VM.
      *
      * @param client XML-RPC Client.
      * @param id The virtual machine id (vid) of the target instance.
-     * @param success recover by succeeding the missing transaction if true.
+     * @param operation to recover the VM: (0) failure, (1) success or (2)
+     * retry
      * @return If an error occurs the error message contains the reason.
      */
-    public static OneResponse recover(Client client, int id, boolean success)
+    public static OneResponse recover(Client client, int id, int operation)
     {
-        return client.call(RECOVER, id, success);
+        return client.call(RECOVER, id, operation);
     }
 
     // =================================
@@ -529,7 +630,6 @@ public class VirtualMachine extends PoolElement{
      * <li>{@link VirtualMachine#suspend()}</li>
      * <li>{@link VirtualMachine#resume()}</li>
      * <li>{@link VirtualMachine#delete(boolean)}</li>
-     * <li>{@link VirtualMachine#boot()}</li>
      * <li>{@link VirtualMachine#poweroff()}</li>
      * <li>{@link VirtualMachine#resched()}</li>
      * <li>{@link VirtualMachine#unresched()}</li>
@@ -538,7 +638,7 @@ public class VirtualMachine extends PoolElement{
      *
      * @param action The action name to be performed, can be:<br/>
      * "shutdown", "hold", "release", "stop", "shutdown-hard", "suspend",
-     * "resume", "boot", "delete", "delete-recreate", "reboot", "resched",
+     * "resume", "delete", "delete-recreate", "reboot", "resched",
      * "unresched", "reboot-hard", "poweroff", "undeploy", "undeploy-hard"
      * @return If an error occurs the error message contains the reason.
      */
@@ -557,11 +657,13 @@ public class VirtualMachine extends PoolElement{
      * @param enforce If it is set to true, the host capacity
      * will be checked, and the deployment will fail if the host is
      * overcommited. Defaults to false
+     * @param ds_id The System Datastore where to migrate the VM. To use the
+     * current one, set it to -1
      * @return If an error occurs the error message contains the reason.
      */
-    public OneResponse migrate(int hostId, boolean live, boolean enforce)
+    public OneResponse migrate(int hostId, boolean live, boolean enforce, int ds_id)
     {
-        return client.call(MIGRATE, id, hostId, live, enforce);
+        return client.call(MIGRATE, id, hostId, live, enforce, ds_id);
     }
 
     /**
@@ -574,7 +676,7 @@ public class VirtualMachine extends PoolElement{
      */
     public OneResponse migrate(int hostId, boolean live)
     {
-        return migrate(hostId, live, false);
+        return migrate(hostId, live, false, -1);
     }
 
     /**
@@ -586,7 +688,7 @@ public class VirtualMachine extends PoolElement{
      */
     public OneResponse migrate(int hostId)
     {
-        return migrate(hostId, false, false);
+        return migrate(hostId, false, false, -1);
     }
 
     /**
@@ -703,36 +805,32 @@ public class VirtualMachine extends PoolElement{
     }
 
     /**
-     * Sets the specified vm's disk to be saved in a new image.
+     * Sets the specified vm's disk to be saved as a new image.
      *
      * @param diskId ID of the disk to be saved.
      * @param imageName Name of the new Image that will be created.
      * @param imageType Type of the new image. Set to empty string to use
      * the default type
-     * @param hot True to save the disk immediately, false will perform
-     * the operation when the VM shuts down
-     * @param doTemplate True to clone also the VM originating template
-     * and replace the disk with the saved image
+     * @param snapId ID of the snapshot to save, -1 to use the
+     * current disk image state
      * @return If an error occurs the error message contains the reason.
      */
-    public OneResponse diskSnapshot(int diskId, String imageName,
-        String imageType, boolean hot, boolean doTemplate)
+    public OneResponse diskSaveas(int diskId, String imageName,
+        String imageType, int snapId)
     {
-        return diskSnapshot(client, id, diskId, imageName, imageType,
-                            hot, doTemplate);
+        return diskSaveas(client, id, diskId, imageName, imageType, snapId);
     }
 
     /**
-     * Sets the specified vm's disk to be saved in a new image when the
-     * VirtualMachine shuts down.
+     * Sets the specified vm's disk to be saved as a new image.
      *
      * @param diskId ID of the disk to be saved.
      * @param imageName Name of the new Image that will be created.
      * @return If an error occurs the error message contains the reason.
      */
-    public OneResponse diskSnapshot(int diskId, String imageName)
+    public OneResponse diskSaveas(int diskId, String imageName)
     {
-        return diskSnapshot(diskId, imageName, "", false, false);
+        return diskSaveas(diskId, imageName, "", -1);
     }
 
     /**
@@ -740,13 +838,13 @@ public class VirtualMachine extends PoolElement{
      *
      * @param diskId ID of the disk to be saved.
      * @param imageName Name of the new Image that will be created.
-     * @param hot True to save the disk immediately, false will perform
-     * the operation when the VM shuts down
+     * @param snapId ID of the snapshot to save, -1 to use the
+     * current disk image state
      * @return If an error occurs the error message contains the reason.
      */
-    public OneResponse diskSnapshot(int diskId, String imageName, boolean hot)
+    public OneResponse diskSaveas(int diskId, String imageName, int snapId)
     {
-        return diskSnapshot(diskId, imageName, "", hot, false);
+        return diskSaveas(diskId, imageName, "", snapId);
     }
 
     /**
@@ -855,14 +953,51 @@ public class VirtualMachine extends PoolElement{
     }
 
     /**
-     * Recovers a stuck VM.
+     * Takes a new snapshot of a disk
      *
-     * @param success recover by succeeding the missing transaction if true.
+     * @param diskId Id of the disk
+     * @param name description for the snapshot
+     * @return New snapshot Id, or the error message
+     */
+    public OneResponse diskSnapshotCreate(int diskId, String name)
+    {
+        return diskSnapshotCreate(client, id, diskId, name);
+    }
+
+    /**
+     * Reverts disk state to a previously taken snapshot
+     *
+     * @param diskId Id of the disk
+     * @param snapId Id of the snapshot
      * @return If an error occurs the error message contains the reason.
      */
-    public OneResponse recover(boolean success)
+    public OneResponse diskSnapshotRevert(int diskId, int snapId)
     {
-        return recover(client, id, success);
+        return diskSnapshotRevert(client, id, diskId, snapId);
+    }
+
+    /**
+     * Deletes a disk snapshot
+     *
+     * @param diskId Id of the disk
+     * @param snapId Id of the snapshot
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse diskSnapshotDelete(int diskId, int snapId)
+    {
+        return diskSnapshotDelete(client, id, diskId, snapId);
+    }
+
+    /**
+     * Recovers a stuck VM.
+     *
+     * @param operation to recover the VM: (0) failure, (1) success or (2)
+     * retry
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse recover(int operation)
+    {
+        return recover(client, id, operation);
     }
 
     // =================================
@@ -1017,15 +1152,6 @@ public class VirtualMachine extends PoolElement{
         return action(actionSt);
     }
 
-    /**
-     * Forces a re-deployment of a VM in UNKNOWN or BOOT states.
-     * @return If an error occurs the error message contains the reason.
-     */
-    public OneResponse boot()
-    {
-        return action("boot");
-    }
-
     /**
      * Sets the re-scheduling flag for the VM
      * @return If an error occurs the error message contains the reason.
@@ -1134,22 +1260,6 @@ public class VirtualMachine extends PoolElement{
         return diskDetach(client, id, diskId);
     }
 
-    /**
-     * @deprecated  Replaced by {@link #diskSnapshot(int,String)}
-     */
-    @Deprecated public OneResponse savedisk(int diskId, String imageName)
-    {
-        return diskSnapshot(diskId, imageName);
-    }
-
-    /**
-     * @deprecated  Replaced by {@link #diskSnapshot(int,String,String,boolean,boolean)}
-     */
-    public OneResponse savedisk(int diskId, String imageName, String imageType)
-    {
-        return diskSnapshot(diskId, imageName, imageType, false, false);
-    }
-
     /**
      * @deprecated  Replaced by {@link #diskAttach(String)}
      */
@@ -1207,17 +1317,6 @@ public class VirtualMachine extends PoolElement{
         return action("resubmit");
     }
 
-    /**
-     * Forces a re-deployment of a VM in UNKNOWN or BOOT states.
-     * @return If an error occurs the error message contains the reason.
-     *
-     * @deprecated  Replaced by {@link #boot}
-     */
-    @Deprecated public OneResponse restart()
-    {
-        return action("restart");
-    }
-
     /**
      * Performs a live migration of the virtual machine to the
      * target host (hid).
