@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -30,7 +30,8 @@ module OpenNebula
             :update     => "datastore.update",
             :chown      => "datastore.chown",
             :chmod      => "datastore.chmod",
-            :rename     => "datastore.rename"
+            :rename     => "datastore.rename",
+            :enable     => "datastore.enable"
         }
 
         DATASTORE_TYPES=%w{IMAGE SYSTEM FILE}
@@ -41,6 +42,13 @@ module OpenNebula
             "FILE"  => "fil"
         }
 
+        DATASTORE_STATES=%w{READY DISABLED}
+
+        SHORT_DATASTORE_STATES={
+            "READY"     => "on",
+            "DISABLED"  => "off"
+        }
+
         # Creates a Datastore description with just its identifier
         # this method should be used to create plain Datastore objects.
         # +id+ the id of the user
@@ -66,20 +74,6 @@ module OpenNebula
         #######################################################################
         # XML-RPC Methods for the Datastore Object
         #######################################################################
-        # Returns the datastore type
-        def type
-            self['TYPE'].to_i
-        end
-
-        # Returns the datastore type (string value)
-        def type_str
-            DATASTORE_TYPES[type]
-        end
-
-        # Returns the datastore type (string value)
-        def short_type_str
-            SHORT_DATASTORE_TYPES[type_str]
-        end
 
         # Retrieves the information of the given Datastore.
         def info()
@@ -157,10 +151,50 @@ module OpenNebula
             return call(DATASTORE_METHODS[:rename], @pe_id, name)
         end
 
+        # Enables a Datastore
+        def enable
+            return call(DATASTORE_METHODS[:enable], @pe_id, true)
+        end
+
+        # Disables a Datastore
+        def disable
+            return call(DATASTORE_METHODS[:enable], @pe_id, false)
+        end
+
         # ---------------------------------------------------------------------
         # Helpers to get information
         # ---------------------------------------------------------------------
 
+        # Returns the datastore type
+        def type
+            self['TYPE'].to_i
+        end
+
+        # Returns the datastore type (string value)
+        def type_str
+            DATASTORE_TYPES[type]
+        end
+
+        # Returns the datastore type (string value)
+        def short_type_str
+            SHORT_DATASTORE_TYPES[type_str]
+        end
+
+        # Returns the state of the datastore (numeric value)
+        def state
+            self['STATE'].to_i
+        end
+
+        # Returns the state of the datastore (string value)
+        def state_str
+            DATASTORE_STATES[state]
+        end
+
+        # Returns the state of the datastore (string value)
+        def short_state_str
+            SHORT_DATASTORE_STATES[state_str]
+        end
+
         # Returns whether or not the image with id 'id' is part of this datastore
         def contains(id)
             #This doesn't work in ruby 1.8.5
