@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -25,7 +25,7 @@ module OpenNebula
         alias_method :each_with_xpath, :each
 
         PAGINATED_POOLS=%w{VM_POOL IMAGE_POOL TEMPLATE_POOL VN_POOL
-                           DOCUMENT_POOL}
+                           DOCUMENT_POOL SECGROUP_POOL}
 
     protected
         #pool:: _String_ XML name of the root element
@@ -202,8 +202,9 @@ module OpenNebula
 
                 a_array=parser.parse(a)
 
-                array += a_array
+                array   += a_array
                 current += size
+
                 break if !a || a_array.length<size
             end
 
