@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -47,14 +47,18 @@ module OpenNebula
         attr_accessor :pool
 
         def initialize (pool_name, elem_name)
-            @current = 0
-            @levels  = [{}]
-            @pool    = Array.new
+            clear
 
             @pool_name = pool_name
             @elem_name = elem_name
         end
 
+        def clear
+            @current = 0
+            @levels  = [{}]
+            @pool    = Array.new
+        end
+
         def start_element(name, attrs = [])
             return if name == @pool_name
             @value = nil
@@ -98,7 +102,7 @@ module OpenNebula
         end
 
         def parse(str)
-            @pool_sax.pool.clear
+            @pool_sax.clear
             sax_parse(str)
             @pool_sax.pool
         end
