@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -31,7 +31,7 @@ require 'nokogiri'
 
 module OneDBFsck
     VERSION = "4.11.80"
-    LOCAL_VERSION = "4.11.80"
+    LOCAL_VERSION = "4.13.85"
 
     def check_db_version()
         db_version = read_db_version()
@@ -136,6 +136,9 @@ EOT
         init_log_time()
 
         @errors = 0
+        @repaired_errors = 0
+        @unrepaired_errors = 0
+
         puts
 
         db_version = read_db_version()
@@ -175,12 +178,14 @@ EOT
             end
 
             if ( max_oid > control_oid )
-                log_error("pool_control for table #{table} has last_oid #{control_oid}, but it is #{max_oid}")
+                msg = "pool_control for table #{table} has last_oid #{control_oid}, but it is #{max_oid}"
 
                 if control_oid != -1
                     if db_version[:is_slave] && federated_tables.include?(table)
-                        log_error("^ Needs to be fixed in the master OpenNebula")
+                        log_error(msg, false)
+                        log_msg("^ Needs to be fixed in the master OpenNebula")
                     else
+                        log_error(msg)
                         @db.run("UPDATE pool_control SET last_oid=#{max_oid} WHERE tablename='#{table}'")
                     end
                 else
@@ -214,14 +219,14 @@ EOT
         users_fix = {}
 
         @db.fetch("SELECT oid,body,gid FROM user_pool") do |row|
-            doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+            doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
             gid = doc.root.at_xpath('GID').text.to_i
             user_gid = gid
             user_gids = Set.new
 
             if group[gid].nil?
-                log_error("User #{row[:oid]} has primary group #{gid}, but it does not exist")
+                log_error("User #{row[:oid]} has primary group #{gid}, but it does not exist", !db_version[:is_slave])
 
                 user_gid = 1
 
@@ -247,7 +252,7 @@ EOT
             }
 
             if !user_gids.include?(user_gid)
-                log_error("User #{row[:oid]} does not have his primary group #{user_gid} in the list of secondary groups")
+                log_error("User #{row[:oid]} does not have his primary group #{user_gid} in the list of secondary groups", !db_version[:is_slave])
 
                 doc.root.xpath("GROUPS").each { |e|
                     e.add_child(doc.create_element("ID")).content = user_gid.to_s
@@ -260,7 +265,7 @@ EOT
 
             user_gids.each do |secondary_gid|
                 if group[secondary_gid].nil?
-                    log_error("User #{row[:oid]} has secondary group #{secondary_gid}, but it does not exist")
+                    log_error("User #{row[:oid]} has secondary group #{secondary_gid}, but it does not exist", !db_version[:is_slave])
 
                     doc.root.xpath("GROUPS").each { |e|
                         e.xpath("ID[.=#{secondary_gid}]").each{|x| x.remove}
@@ -275,7 +280,7 @@ EOT
             if gid != row[:gid]
                 log_error(
                     "User #{row[:oid]} is in group #{gid}, but the DB "<<
-                    "table has GID column #{row[:gid]}")
+                    "table has GID column #{row[:gid]}", !db_version[:is_slave])
 
                 users_fix[row[:oid]] = {:body => doc.root.to_s, :gid => user_gid}
             end
@@ -290,61 +295,58 @@ EOT
                 end
             end
         elsif !users_fix.empty?
-            log_error("^ User errors need to be fixed in the master OpenNebula")
+            log_msg("^ User errors need to be fixed in the master OpenNebula")
         end
 
         log_time()
 
-        if !db_version[:is_slave]
-            @db.run "CREATE TABLE group_pool_new (oid INTEGER PRIMARY KEY, name VARCHAR(128), body MEDIUMTEXT, uid INTEGER, gid INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER, UNIQUE(name));"
-        end
+        groups_fix = {}
 
-        @db.transaction do
-            @db.fetch("SELECT * from group_pool") do |row|
-                gid = row[:oid]
-                doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+        @db.fetch("SELECT oid,body from group_pool") do |row|
+            gid = row[:oid]
+            doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
-                users_elem = doc.root.at_xpath("USERS")
-                users_elem.remove if !users_elem.nil?
+            users_elem = doc.root.at_xpath("USERS")
+            users_elem.remove if !users_elem.nil?
 
-                users_new_elem = doc.create_element("USERS")
-                doc.root.add_child(users_new_elem)
+            users_new_elem = doc.create_element("USERS")
+            doc.root.add_child(users_new_elem)
 
-                error_found = false
+            error_found = false
 
-                group[gid].each do |id|
-                    id_elem = users_elem.at_xpath("ID[.=#{id}]")
+            group[gid].each do |id|
+                id_elem = users_elem.at_xpath("ID[.=#{id}]")
 
-                    if id_elem.nil?
-                        log_error("User #{id} is missing from Group #{gid} users id list")
-                        error_found = true
-                    else
-                        id_elem.remove
-                    end
-
-                    users_new_elem.add_child(doc.create_element("ID")).content = id.to_s
-                end
-
-                users_elem.xpath("ID").each do |id_elem|
-                    log_error("User #{id_elem.text} is in Group #{gid} users id list, but it should not")
+                if id_elem.nil?
+                    log_error("User #{id} is missing from Group #{gid} users id list", !db_version[:is_slave])
                     error_found = true
+                else
+                    id_elem.remove
                 end
 
-                row[:body] = doc.root.to_s
+                users_new_elem.add_child(doc.create_element("ID")).content = id.to_s
+            end
 
-                if !db_version[:is_slave]
-                    # commit
-                    @db[:group_pool_new].insert(row)
-                elsif error_found
-                    log_error("^ Group errors need to be fixed in the master OpenNebula")
-                end
+            users_elem.xpath("ID").each do |id_elem|
+                log_error("User #{id_elem.text} is in Group #{gid} users id list, but it should not", !db_version[:is_slave])
+                error_found = true
+            end
+
+
+            if error_found
+                groups_fix[row[:oid]] = doc.root.to_s
             end
         end
 
         if !db_version[:is_slave]
-            # Rename table
-            @db.run("DROP TABLE group_pool")
-            @db.run("ALTER TABLE group_pool_new RENAME TO group_pool")
+            @db.transaction do
+                groups_fix.each do |id, body|
+                    @db[:group_pool].where(:oid => id).update(
+                        :body => body)
+                end
+            end
+        elsif !groups_fix.empty?
+            log_msg("^ Group errors need to be fixed in the master OpenNebula")
         end
 
         log_time()
@@ -650,7 +652,7 @@ EOT
                         "    #{doc.root.get_text('SOURCE')}\n"<<
                         "  * The DB entry can be then deleted with the command:\n"<<
                         "    DELETE FROM image_pool WHERE oid=#{row[:oid]};\n"<<
-                        "  * Run fsck again.\n")
+                        "  * Run fsck again.\n", false)
                 else
                     if ds_name != ds_entry[:name]
                         log_error("Image #{row[:oid]} has a wrong name for datastore #{ds_id}, #{ds_name}. It will be changed to #{ds_entry[:name]}")
@@ -675,7 +677,7 @@ EOT
 
         log_time()
 
-        @db.run "CREATE TABLE datastore_pool_new (oid INTEGER PRIMARY KEY, name VARCHAR(128), body MEDIUMTEXT, uid INTEGER, gid INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER, cid INTEGER, UNIQUE(name));"
+        @db.run "CREATE TABLE datastore_pool_new (oid INTEGER PRIMARY KEY, name VARCHAR(128), body MEDIUMTEXT, uid INTEGER, gid INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER, cid INTEGER);"
 
         @db.transaction do
             @db.fetch("SELECT * from datastore_pool") do |row|
@@ -768,7 +770,7 @@ EOT
 
         # Init vnet counters
         @db.fetch("SELECT oid,body FROM network_pool") do |row|
-            doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+            doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
             ar_leases = {}
 
@@ -788,10 +790,10 @@ EOT
 
         # Aggregate information of the RUNNING vms
         @db.fetch("SELECT oid,body FROM vm_pool WHERE state<>6") do |row|
-            vm_doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+            vm_doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
             state     = vm_doc.root.at_xpath('STATE').text.to_i
-            lcm_state = vm_doc.root.at_xpath('LCM_STATE').text.to_i            
+            lcm_state = vm_doc.root.at_xpath('LCM_STATE').text.to_i
 
             # Images used by this VM
             vm_doc.root.xpath("TEMPLATE/DISK/IMAGE_ID").each do |e|
@@ -799,7 +801,7 @@ EOT
 
                 if counters[:image][img_id].nil?
                     log_error("VM #{row[:oid]} is using Image #{img_id}, but "<<
-                        "it does not exist")
+                        "it does not exist", false)
                 else
                     counters[:image][img_id][:vms].add(row[:oid])
                 end
@@ -815,7 +817,7 @@ EOT
                 if !net_id.nil?
                     if counters[:vnet][net_id].nil?
                         log_error("VM #{row[:oid]} is using VNet #{net_id}, "<<
-                            "but it does not exist")
+                            "but it does not exist", false)
                     else
                         mac = nic.at_xpath("MAC").nil? ? nil : nic.at_xpath("MAC").text
 
@@ -824,7 +826,7 @@ EOT
                         if ar_id_e.nil?
                             if !counters[:vnet][net_id][:no_ar_leases][mac_s_to_i(mac)].nil?
                                 log_error("VNet has more than one VM with the same MAC address (#{mac}). "<<
-                                    "FSCK can't handle this, and consistency is not guaranteed")
+                                    "FSCK can't handle this, and consistency is not guaranteed", false)
                             end
 
                             counters[:vnet][net_id][:no_ar_leases][mac_s_to_i(mac)] = {
@@ -841,7 +843,7 @@ EOT
 
                             if counters[:vnet][net_id][:ar_leases][ar_id].nil?
                                 log_error("VM #{row[:oid]} is using VNet #{net_id}, AR #{ar_id}, "<<
-                                    "but the AR does not exist")
+                                    "but the AR does not exist", false)
                             else
                                 counters[:vnet][net_id][:ar_leases][ar_id][mac_s_to_i(mac)] = {
                                     :ip         => nic.at_xpath("IP").nil? ? nil : nic.at_xpath("IP").text,
@@ -885,7 +887,7 @@ EOT
 
             if counters_host.nil?
                 log_error("VM #{row[:oid]} is using Host #{hid}, "<<
-                    "but it does not exist")
+                    "but it does not exist", false)
             else
                 if counters_host[:name] != hostname
                     log_error("VM #{row[:oid]} has a wrong hostname for "<<
@@ -914,6 +916,59 @@ EOT
 
         log_time()
 
+        # Bug #4000 may cause history records with etime=0 when they should
+        # be closed. The last history can be fixed with the VM etime, but
+        # previous history entries need to be fixed manually
+
+        # Query to select history elements that:
+        #   - have etime = 0
+        #   - are not the last seq
+        @db.fetch("SELECT vid,seq FROM history WHERE (etime = 0 AND seq <> (SELECT MAX(seq) FROM history AS subhistory WHERE history.vid = subhistory.vid) )") do |row|
+            log_error("History record for VM #{row[:vid]} seq # #{row[:seq]} is not closed (etime = 0)", false)
+        end
+
+        log_time()
+
+        history_fix = []
+
+        # Query to select history elements that have:
+        #   - etime = 0
+        #   - is last seq
+        #   - VM is DONE
+        @db.fetch("SELECT * FROM history WHERE (etime = 0 AND vid IN (SELECT oid FROM vm_pool WHERE state=6) AND seq = (SELECT MAX(seq) FROM history AS subhistory WHERE history.vid=subhistory.vid))") do |row|
+            log_error("History record for VM #{row[:vid]} seq # #{row[:seq]} is not closed (etime = 0), but the VM is in state DONE")
+
+            etime = 0
+
+            @db.fetch("SELECT body FROM vm_pool WHERE oid=#{row[:vid]}") do |vm_row|
+                vm_doc = Nokogiri::XML(vm_row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
+
+                etime = vm_doc.root.at_xpath("ETIME").text.to_i
+            end
+
+            history_doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
+
+            ["RETIME", "ESTIME", "EETIME", "ETIME"].each do |att|
+                elem = history_doc.root.at_xpath(att)
+                if (elem.text == "0")
+                    elem.content = etime
+                end
+            end
+
+            row[:body]  = history_doc.root.to_s
+            row[:etime] = etime
+
+            history_fix.push(row)
+        end
+
+        @db.transaction do
+            history_fix.each do |row|
+                @db[:history].where({:vid => row[:vid], :seq => row[:seq]}).update(row)
+            end
+        end
+
+        log_time()
+
         ########################################################################
         # Hosts
         #
@@ -928,12 +983,12 @@ EOT
                 "name VARCHAR(128), body MEDIUMTEXT, state INTEGER, " <<
                 "last_mon_time INTEGER, uid INTEGER, gid INTEGER, " <<
                 "owner_u INTEGER, group_u INTEGER, other_u INTEGER, " <<
-                "cid INTEGER, UNIQUE(name));"
+                "cid INTEGER);"
 
         # Calculate the host's xml and write them to host_pool_new
         @db.transaction do
             @db[:host_pool].each do |row|
-                host_doc = Document.new(row[:body])
+                host_doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
                 hid = row[:oid]
 
@@ -944,54 +999,67 @@ EOT
                 mem_usage   = counters_host[:memory]*1024
 
                 # rewrite running_vms
-                host_doc.root.each_element("HOST_SHARE/RUNNING_VMS") {|e|
+                host_doc.root.xpath("HOST_SHARE/RUNNING_VMS").each {|e|
                     if e.text != rvms.to_s
                         log_error(
                             "Host #{hid} RUNNING_VMS has #{e.text} \tis\t#{rvms}")
-                        e.text = rvms
+                        e.content = rvms
                     end
                 }
 
 
-                # re-do list of VM IDs 
-                vms_elem = host_doc.root.elements.delete("VMS")
+                # re-do list of VM IDs
+                vms_elem = host_doc.root.at_xpath("VMS").remove
 
-                vms_new_elem = host_doc.root.add_element("VMS")
+                vms_new_elem = host_doc.create_element("VMS")
+                host_doc.root.add_child(vms_new_elem)
 
                 counters_host[:rvms].each do |id|
-                    id_elem = vms_elem.elements.delete("ID[.=#{id}]")
+                    id_elem = vms_elem.at_xpath("ID[.=#{id}]")
 
                     if id_elem.nil?
                         log_error(
                             "VM #{id} is missing from Host #{hid} VM id list")
+                    else
+                        id_elem.remove
                     end
 
-                    vms_new_elem.add_element("ID").text = id.to_s
+                    vms_new_elem.add_child(host_doc.create_element("ID")).content = id.to_s
                 end
 
-                vms_elem.each_element("ID") do |id_elem|
+                vms_elem.xpath("ID").each do |id_elem|
                     log_error(
                         "VM #{id_elem.text} is in Host #{hid} VM id list, "<<
                         "but it should not")
                 end
 
+                host_doc.root.xpath("HOST_SHARE/PCI_DEVICES/PCI").each do |pci|
+                    if !pci.at_xpath("VMID").nil?
+                        vmid = pci.at_xpath("VMID").text.to_i
+
+                        if vmid != -1 && !counters_host[:rvms].include?(vmid)
+                            log_error("VM #{vmid} has a PCI device assigned in host #{hid}, but it should not. Device: #{pci.at_xpath('DEVICE_NAME').text}")
+                            pci.at_xpath("VMID").content = "-1"
+                        end
+                    end
+                end
 
                 # rewrite cpu
-                host_doc.root.each_element("HOST_SHARE/CPU_USAGE") {|e|
+                host_doc.root.xpath("HOST_SHARE/CPU_USAGE").each {|e|
                     if e.text != cpu_usage.to_s
                         log_error(
                             "Host #{hid} CPU_USAGE has #{e.text} "<<
                             "\tis\t#{cpu_usage}")
-                        e.text = cpu_usage
+                        e.content = cpu_usage
                     end
                 }
 
                 # rewrite memory
-                host_doc.root.each_element("HOST_SHARE/MEM_USAGE") {|e|
+                host_doc.root.xpath("HOST_SHARE/MEM_USAGE").each {|e|
                     if e.text != mem_usage.to_s
                         log_error("Host #{hid} MEM_USAGE has #{e.text} "<<
                             "\tis\t#{mem_usage}")
-                        e.text = mem_usage
+                        e.content = mem_usage
                     end
                 }
 
@@ -1045,7 +1113,7 @@ EOT
                     end
                 }
 
-                # re-do list of VM IDs 
+                # re-do list of VM IDs
                 vms_elem = doc.root.elements.delete("VMS")
 
                 vms_new_elem = doc.root.add_element("VMS")
@@ -1151,7 +1219,7 @@ EOT
         ########################################################################
 
         @db.fetch("SELECT oid,body,pid FROM network_pool WHERE pid<>-1") do |row|
-            doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+            doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
             parent_vnet = doc.root.at_xpath("PARENT_NETWORK_ID").text.to_i
 
@@ -1169,7 +1237,7 @@ EOT
                         log_error(
                             "VNet #{row[:oid]} is using parent "<<
                             "VNet #{parent_vnet}, AR #{parent_ar}, "<<
-                            "but the AR does not exist")
+                            "but the AR does not exist", false)
                     end
 
                     # MAC
@@ -1258,7 +1326,7 @@ EOT
 
         @db.transaction do
         @db[:network_pool].each do |row|
-            doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+            doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
             oid = row[:oid]
 
             used_leases = doc.root.at_xpath("USED_LEASES").text.to_i
@@ -1437,7 +1505,7 @@ EOT
                                     "#{lease_obj} #{lease_oid}. #{key.to_s.upcase} "<<
                                     "does not match: "<<
                                     "#{counter_lease[key]} != #{lease[key]}. "<<
-                                    "This can't be fixed")
+                                    "This can't be fixed", false)
                                 end
                             end
                         end
@@ -1493,7 +1561,7 @@ EOT
 
             counter_no_ar.each do |mac, counter_lease|
                 log_error("VM #{counter_lease[:vm]} has a lease from "<<
-                    "VNet #{oid}, but it could not be matched to any AR")
+                    "VNet #{oid}, but it could not be matched to any AR", false)
             end
 
             row[:body] = doc.root.to_s
@@ -1544,7 +1612,7 @@ EOT
             end
 
             @db.fetch("SELECT * FROM old_user_quotas WHERE user_oid>0") do |row|
-                doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+                doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
                 calculate_quotas(doc, "uid=#{row[:user_oid]}", "User")
 
@@ -1592,7 +1660,7 @@ EOT
             end
 
             @db.fetch("SELECT * FROM old_group_quotas WHERE group_oid>0") do |row|
-                doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+                doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
                 calculate_quotas(doc, "gid=#{row[:group_oid]}", "Group")
 
@@ -1611,9 +1679,23 @@ EOT
         return true
     end
 
-    def log_error(message)
+    def log_error(message, repaired=true)
         @errors += 1
 
+        if repaired
+            @repaired_errors += 1
+        else
+            @unrepaired_errors += 1
+        end
+
+        if !repaired
+            message = "[UNREPAIRED] " + message
+        end
+
+        log_msg(message)
+    end
+
+    def log_msg(message)
         @log_file ||= File.open(LOG, "w")
 
         puts message
@@ -1624,7 +1706,10 @@ EOT
 
     def log_total_errors()
         puts
-        puts "Total errors found: #{@errors}"
+        log_msg "Total errors found: #{@errors}"
+        log_msg "Total errors repaired: #{@repaired_errors}"
+        log_msg "Total errors unrepaired: #{@unrepaired_errors}"
+
         puts "A copy of this output was stored in #{LOG}"
     end
 
@@ -1638,7 +1723,7 @@ EOT
         cpu_used = 0
         mem_used = 0
         vms_used = 0
-        vol_used = 0
+        sys_used = 0
 
         # VNet quotas
         vnet_usage = {}
@@ -1647,7 +1732,7 @@ EOT
         img_usage = {}
 
         @db.fetch("SELECT body FROM vm_pool WHERE #{where_filter} AND state<>6") do |vm_row|
-            vmdoc = Nokogiri::XML(vm_row[:body]){|c| c.default_xml.noblanks}
+            vmdoc = Nokogiri::XML(vm_row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
             # VM quotas
             vmdoc.root.xpath("TEMPLATE/CPU").each { |e|
@@ -1667,10 +1752,34 @@ EOT
                     type = t_elem.text.upcase
                 }
 
+                size = 0
+
+                if !e.at_xpath("SIZE").nil?
+                    size = e.at_xpath("SIZE").text.to_i
+                end
+
                 if ( type == "SWAP" || type == "FS")
-                    e.xpath("SIZE").each { |size_elem|
-                        vol_used += size_elem.text.to_i
-                    }
+                    sys_used += size
+                else
+                    if !e.at_xpath("CLONE").nil?
+                        clone = (e.at_xpath("CLONE").text.upcase == "YES")
+
+                        target = nil
+
+                        if clone
+                            target = e.at_xpath("CLONE_TARGET").text if !e.at_xpath("CLONE_TARGET").nil?
+                        else
+                            target = e.at_xpath("LN_TARGET").text if !e.at_xpath("LN_TARGET").nil?
+                        end
+
+                        if !target.nil? && target != "NONE" # self or system
+                            sys_used += size
+
+                            if !e.at_xpath("DISK_SNAPSHOT_TOTAL_SIZE").nil?
+                                sys_used += e.at_xpath("DISK_SNAPSHOT_TOTAL_SIZE").text.to_i
+                            end
+                        end
+                    end
                 end
             }
 
@@ -1710,8 +1819,8 @@ EOT
             vm_elem.add_child(doc.create_element("VMS")).content         = "-1"
             vm_elem.add_child(doc.create_element("VMS_USED")).content    = "0"
 
-            vm_elem.add_child(doc.create_element("VOLATILE_SIZE")).content       = "-1"
-            vm_elem.add_child(doc.create_element("VOLATILE_SIZE_USED")).content  = "0"
+            vm_elem.add_child(doc.create_element("SYSTEM_DISK_SIZE")).content       = "-1"
+            vm_elem.add_child(doc.create_element("SYSTEM_DISK_SIZE_USED")).content  = "0"
         end
 
 
@@ -1750,10 +1859,10 @@ EOT
             end
         }
 
-        vm_elem.xpath("VOLATILE_SIZE_USED").each { |e|
-            if e.text != vol_used.to_s
-                log_error("#{resource} #{oid} quotas: VOLATILE_SIZE_USED has #{e.text} \tis\t#{vol_used}")
-                e.content = vol_used.to_s
+        vm_elem.xpath("SYSTEM_DISK_SIZE_USED").each { |e|
+            if e.text != sys_used.to_s
+                log_error("#{resource} #{oid} quotas: SYSTEM_DISK_SIZE_USED has #{e.text} \tis\t#{sys_used}")
+                e.content = sys_used.to_s
             end
         }
 
@@ -1826,13 +1935,11 @@ EOT
             new_elem.add_child(doc.create_element("RVMS_USED")).content = rvms.to_s
         }
 
-
         # Datastore quotas
-
         ds_usage = {}
 
         @db.fetch("SELECT body FROM image_pool WHERE #{where_filter}") do |img_row|
-            img_doc = Nokogiri::XML(img_row[:body]){|c| c.default_xml.noblanks}
+            img_doc = Nokogiri::XML(img_row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
             img_doc.root.xpath("DATASTORE_ID").each { |e|
                 ds_usage[e.text] = [0,0] if ds_usage[e.text].nil?
@@ -1841,6 +1948,10 @@ EOT
                 img_doc.root.xpath("SIZE").each { |size|
                     ds_usage[e.text][1] += size.text.to_i
                 }
+
+                img_doc.root.xpath("SNAPSHOTS/SNAPSHOT/SIZE").each { |size|
+                    ds_usage[e.text][1] += size.text.to_i
+                }
             }
         end
 
