@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -29,7 +29,7 @@ include OpenNebula
 
 module OneDBImportSlave
     VERSION = "4.11.80"
-    LOCAL_VERSION = "4.11.80"
+    LOCAL_VERSION = "4.13.85"
 
     def check_db_version(master_db_version, slave_db_version)
         if ( master_db_version[:version] != VERSION ||
@@ -307,16 +307,16 @@ EOT
 
         @slave_db.run "ALTER TABLE document_pool RENAME TO old_document_pool;"
         @slave_db.run "CREATE TABLE document_pool (oid INTEGER PRIMARY KEY, name VARCHAR(128), body MEDIUMTEXT, type INTEGER, uid INTEGER, gid INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER);"
-        
+
         @slave_db.run "ALTER TABLE image_pool RENAME TO old_image_pool;"
         @slave_db.run "CREATE TABLE image_pool (oid INTEGER PRIMARY KEY, name VARCHAR(128), body MEDIUMTEXT, uid INTEGER, gid INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER, UNIQUE(name,uid) );"
-        
+
         @slave_db.run "ALTER TABLE network_pool RENAME TO old_network_pool;"
         @slave_db.run "CREATE TABLE network_pool (oid INTEGER PRIMARY KEY, name VARCHAR(128), body MEDIUMTEXT, uid INTEGER, gid INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER, cid INTEGER, pid INTEGER, UNIQUE(name,uid));"
-        
+
         @slave_db.run "ALTER TABLE template_pool RENAME TO old_template_pool;"
         @slave_db.run "CREATE TABLE template_pool (oid INTEGER PRIMARY KEY, name VARCHAR(128), body MEDIUMTEXT, uid INTEGER, gid INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER);"
-        
+
         @slave_db.run "ALTER TABLE vm_pool RENAME TO old_vm_pool;"
         @slave_db.run "CREATE TABLE vm_pool (oid INTEGER PRIMARY KEY, name VARCHAR(128), body MEDIUMTEXT, uid INTEGER, gid INTEGER, last_poll INTEGER, state INTEGER, lcm_state INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER);"
 
@@ -325,7 +325,7 @@ EOT
 
         @slave_db.run "ALTER TABLE group_quotas RENAME TO old_group_quotas;"
         @slave_db.run "CREATE TABLE group_quotas (group_oid INTEGER PRIMARY KEY, body MEDIUMTEXT);"
-        
+
         @slave_db.run "ALTER TABLE user_quotas RENAME TO old_user_quotas;"
         @slave_db.run "CREATE TABLE user_quotas (user_oid INTEGER PRIMARY KEY, body MEDIUMTEXT);"
 
@@ -348,7 +348,7 @@ EOT
                     log("Group ##{row[:gid]} should exist, but it was not found. Run fsck to fix. User ##{new_user[:oid]} (#{new_user[:name]}) will be assigned to group ##{new_group[:oid]}, #{new_group[:name]}")
                 end
 
-                slave_doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+                slave_doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
                 if new_user[:merged]
                     # Merge user objects, giving priority to the contents
@@ -362,7 +362,7 @@ EOT
 
                     @db.fetch("SELECT body from user_pool "<<
                               "WHERE oid=#{new_user[:oid]}") do |master_row|
-                        master_doc = Nokogiri::XML(master_row[:body]){|c| c.default_xml.noblanks}
+                        master_doc = Nokogiri::XML(master_row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
                     end
 
                     # Merge secondary groups
@@ -444,14 +444,14 @@ EOT
             @slave_db.fetch("SELECT * FROM group_pool") do |row|
                 new_group = groups[row[:gid]]
 
-                slave_doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+                slave_doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
                 if new_group[:merged]
                     master_doc = nil
 
                     @db.fetch("SELECT body from group_pool "<<
                               "WHERE oid=#{new_group[:oid]}") do |master_row|
-                        master_doc = Nokogiri::XML(master_row[:body]){|c| c.default_xml.noblanks}
+                        master_doc = Nokogiri::XML(master_row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
                     end
 
                     slave_users_elem  = slave_doc.root.at_xpath("USERS")
@@ -578,7 +578,7 @@ EOT
                 else
                     new_user_id = user[:oid]
 
-                    doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+                    doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
                     doc.root.at_xpath("ID").content = new_user_id
 
@@ -602,7 +602,7 @@ EOT
                 else
                     new_group_id = group[:oid]
 
-                    doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+                    doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
                     doc.root.at_xpath("ID").content = new_group_id
 
@@ -621,14 +621,14 @@ EOT
             @slave_db.fetch("SELECT * FROM vdc_pool") do |row|
                 new_vdc = vdcs[row[:oid]]
 
-                slave_doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+                slave_doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
                 if new_vdc[:merged]
                     master_doc = nil
 
                     @db.fetch("SELECT body from vdc_pool "<<
                               "WHERE oid=#{new_vdc[:oid]}") do |master_row|
-                        master_doc = Nokogiri::XML(master_row[:body]){|c| c.default_xml.noblanks}
+                        master_doc = Nokogiri::XML(master_row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
                     end
 
                     slave_groups_elem  = slave_doc.root.at_xpath("GROUPS")
@@ -918,7 +918,7 @@ EOT
                 log("Group ##{row[:gid]} does not exist anymore. Template ##{row[:oid]} will be assigned to group ##{new_group[:oid]}, #{new_group[:name]}")
             end
 
-            doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+            doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
             doc.root.at_xpath("UID").content    = new_user[:oid]
             doc.root.at_xpath("UNAME").content  = new_user[:name]
@@ -983,7 +983,7 @@ EOT
                 log("Group ##{row[:gid]} does not exist anymore. Image ##{row[:oid]} will be assigned to group ##{new_group[:oid]}, #{new_group[:name]}")
             end
 
-            doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+            doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
             doc.root.at_xpath("UID").content    = new_user[:oid]
             doc.root.at_xpath("UNAME").content  = new_user[:name]
@@ -1016,7 +1016,7 @@ EOT
                 log("Group ##{row[:gid]} does not exist anymore. Document ##{row[:oid]} will be assigned to group ##{new_group[:oid]}, #{new_group[:name]}")
             end
 
-            doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+            doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
             doc.root.at_xpath("UID").content    = new_user[:oid]
             doc.root.at_xpath("UNAME").content  = new_user[:name]
@@ -1050,7 +1050,7 @@ EOT
                 log("Group ##{row[:gid]} does not exist anymore. Network ##{row[:oid]} will be assigned to group ##{new_group[:oid]}, #{new_group[:name]}")
             end
 
-            doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+            doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
             doc.root.at_xpath("UID").content    = new_user[:oid]
             doc.root.at_xpath("UNAME").content  = new_user[:name]
@@ -1085,7 +1085,7 @@ EOT
                 log("Group ##{row[:gid]} does not exist anymore. VM ##{row[:oid]} will be assigned to group ##{new_group[:oid]}, #{new_group[:name]}")
             end
 
-            doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+            doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
             doc.root.at_xpath("UID").content    = new_user[:oid]
             doc.root.at_xpath("UNAME").content  = new_user[:name]
@@ -1121,7 +1121,7 @@ EOT
                 log("Group ##{row[:gid]} does not exist anymore. Security Group ##{row[:oid]} will be assigned to group ##{new_group[:oid]}, #{new_group[:name]}")
             end
 
-            doc = Nokogiri::XML(row[:body]){|c| c.default_xml.noblanks}
+            doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
             doc.root.at_xpath("UID").content    = new_user[:oid]
             doc.root.at_xpath("UNAME").content  = new_user[:name]
