@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -14,7 +14,9 @@
 # limitations under the License.                                             #
 #--------------------------------------------------------------------------- #
 
+require 'time'
 require 'rubygems'
+require 'cgi'
 
 begin
     require 'sequel'
@@ -55,7 +57,7 @@ class OneDBBacKEnd
                     ret[:local_timestamp] = row[:timestamp]
                     ret[:local_comment]   = row[:comment]
                     ret[:is_slave]        = row[:is_slave]
-               end 
+               end
             rescue Exception => e
                 if e.class == Sequel::DatabaseConnectionError
                     raise e
@@ -229,12 +231,14 @@ class BackEndMySQL < OneDBBacKEnd
     end
 
     def bck_file
-        "#{VAR_LOCATION}/mysql_#{@server}_#{@db_name}.sql"
+        t = Time.now
+        "#{VAR_LOCATION}/mysql_#{@server}_#{@db_name}_"<<
+        "#{t.year}-#{t.month}-#{t.day}_#{t.hour}:#{t.min}:#{t.sec}.sql"
     end
 
     def backup(bck_file)
         cmd = "mysqldump -u #{@user} -p'#{@passwd}' -h #{@server} " +
-              "-P #{@port} #{@db_name} > #{bck_file}"
+              "-P #{@port} --add-drop-table #{@db_name} > #{bck_file}"
 
         rc = system(cmd)
         if !rc
@@ -255,22 +259,7 @@ class BackEndMySQL < OneDBBacKEnd
                   " use -f to overwrite."
         end
 
-        mysql_cmd = "mysql -u #{@user} -p'#{@passwd}' -h #{@server} -P #{@port} "
-
-        drop_cmd = mysql_cmd + "-e 'DROP DATABASE IF EXISTS #{@db_name};'"
-        rc = system(drop_cmd)
-        if !rc
-            raise "Error dropping MySQL DB #{@db_name} at #{@server}."
-        end
-
-        create_cmd = mysql_cmd+"-e 'CREATE DATABASE IF NOT EXISTS #{@db_name};'"
-        rc = system(create_cmd)
-        if !rc
-            raise "Error creating MySQL DB #{@db_name} at #{@server}."
-        end
-
-        restore_cmd = mysql_cmd + "#{@db_name} < #{bck_file}"
-        rc = system(restore_cmd)
+        rc = system("mysql -u #{@user} -p'#{@passwd}' -h #{@server} -P #{@port} #{@db_name} < #{bck_file}")
         if !rc
             raise "Error while restoring MySQL DB #{@db_name} at #{@server}."
         end
@@ -281,7 +270,9 @@ class BackEndMySQL < OneDBBacKEnd
     private
 
     def connect_db
-        endpoint = "mysql://#{@user}:#{@passwd}@#{@server}:#{@port}/#{@db_name}"
+        passwd = CGI.escape(@passwd)
+
+        endpoint = "mysql://#{@user}:#{passwd}@#{@server}:#{@port}/#{@db_name}"
 
         begin
             @db = Sequel.connect(endpoint)
@@ -299,7 +290,9 @@ class BackEndSQLite < OneDBBacKEnd
     end
 
     def bck_file
-        "#{VAR_LOCATION}/one.db.bck"
+        t = Time.now
+        "#{VAR_LOCATION}/one.db_"<<
+        "#{t.year}-#{t.month}-#{t.day}_#{t.hour}:#{t.min}:#{t.sec}.bck"
     end
 
     def backup(bck_file)
@@ -332,4 +325,4 @@ class BackEndSQLite < OneDBBacKEnd
             raise "Error connecting to DB: " + e.message
         end
     end
-end
\ No newline at end of file
+end
