@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -27,6 +27,7 @@
 #include "RequestManagerChmod.h"
 #include "RequestManagerClone.h"
 #include "RequestManagerRename.h"
+#include "RequestManagerLock.h"
 
 #include "RequestManagerVirtualNetwork.h"
 #include "RequestManagerVirtualMachine.h"
@@ -37,6 +38,8 @@
 #include "RequestManagerAcl.h"
 #include "RequestManagerCluster.h"
 #include "RequestManagerGroup.h"
+#include "RequestManagerVdc.h"
+#include "RequestManagerDatastore.h"
 
 #include "RequestManagerSystem.h"
 #include "RequestManagerProxy.h"
@@ -47,6 +50,7 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
+#include <arpa/inet.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <string.h>
@@ -61,7 +65,9 @@ RequestManager::RequestManager(
         int _keepalive_max_conn,
         int _timeout,
         const string _xml_log_file,
-        const string call_log_format):
+        const string call_log_format,
+        const string _listen_address,
+        int message_size):
             port(_port),
             socket_fd(-1),
             max_conn(_max_conn),
@@ -69,10 +75,13 @@ RequestManager::RequestManager(
             keepalive_timeout(_keepalive_timeout),
             keepalive_max_conn(_keepalive_max_conn),
             timeout(_timeout),
-            xml_log_file(_xml_log_file)
+            xml_log_file(_xml_log_file),
+            listen_address(_listen_address)
 {
     Request::set_call_log_format(call_log_format);
 
+    xmlrpc_limit_set(XMLRPC_XML_SIZE_LIMIT_ID, message_size);
+
     am.addListener(this);
 };
 
@@ -186,7 +195,20 @@ int RequestManager::setup_socket()
 
     rm_addr.sin_family      = AF_INET;
     rm_addr.sin_port        = htons(port);
-    rm_addr.sin_addr.s_addr = INADDR_ANY;
+
+    rc = inet_aton(listen_address.c_str(), &rm_addr.sin_addr);
+
+    if ( rc == 0 )
+    {
+        ostringstream oss;
+
+        oss << "Invalid listen address: " << listen_address;
+        NebulaLog::log("ReM",Log::ERROR,oss);
+
+        close(socket_fd);
+
+        return -1;
+    }
 
     rc = bind(socket_fd,(struct sockaddr *) &(rm_addr),sizeof(struct sockaddr));
 
@@ -194,7 +216,7 @@ int RequestManager::setup_socket()
     {
         ostringstream oss;
 
-        oss << "Cannot bind to port " << port << " : " << strerror(errno);
+        oss << "Cannot bind to " << listen_address << ":" << port << " : " << strerror(errno);
         NebulaLog::log("ReM",Log::ERROR,oss);
 
         close(socket_fd);
@@ -287,7 +309,6 @@ void RequestManager::register_xml_methods()
     xmlrpc_c::methodPtr vm_deploy(new VirtualMachineDeploy());
     xmlrpc_c::methodPtr vm_migrate(new VirtualMachineMigrate());
     xmlrpc_c::methodPtr vm_action(new VirtualMachineAction());
-    xmlrpc_c::methodPtr vm_savedisk(new VirtualMachineSaveDisk());
     xmlrpc_c::methodPtr vm_monitoring(new VirtualMachineMonitoring());
     xmlrpc_c::methodPtr vm_attach(new VirtualMachineAttach());
     xmlrpc_c::methodPtr vm_detach(new VirtualMachineDetach());
@@ -297,11 +318,18 @@ void RequestManager::register_xml_methods()
     xmlrpc_c::methodPtr vm_snap_create(new VirtualMachineSnapshotCreate());
     xmlrpc_c::methodPtr vm_snap_revert(new VirtualMachineSnapshotRevert());
     xmlrpc_c::methodPtr vm_snap_delete(new VirtualMachineSnapshotDelete());
+    xmlrpc_c::methodPtr vm_dsaveas(new VirtualMachineDiskSaveas());
+    xmlrpc_c::methodPtr vm_dsnap_create(new VirtualMachineDiskSnapshotCreate());
+    xmlrpc_c::methodPtr vm_dsnap_revert(new VirtualMachineDiskSnapshotRevert());
+    xmlrpc_c::methodPtr vm_dsnap_delete(new VirtualMachineDiskSnapshotDelete());
     xmlrpc_c::methodPtr vm_recover(new VirtualMachineRecover());
 
     xmlrpc_c::methodPtr vm_pool_acct(new VirtualMachinePoolAccounting());
     xmlrpc_c::methodPtr vm_pool_monitoring(new VirtualMachinePoolMonitoring());
 
+    xmlrpc_c::methodPtr vm_pool_showback(new VirtualMachinePoolShowback());
+    xmlrpc_c::methodPtr vm_pool_calculate_showback(new VirtualMachinePoolCalculateShowback());
+
     // VirtualNetwork Methods
     xmlrpc_c::methodPtr vn_add_ar(new VirtualNetworkAddAddressRange());
     xmlrpc_c::methodPtr vn_rm_ar(new VirtualNetworkRmAddressRange());
@@ -320,6 +348,7 @@ void RequestManager::register_xml_methods()
     xmlrpc_c::methodPtr datastore_update(new DatastoreUpdateTemplate());
     xmlrpc_c::methodPtr doc_update(new DocumentUpdateTemplate());
     xmlrpc_c::methodPtr cluster_update(new ClusterUpdateTemplate());
+    xmlrpc_c::methodPtr secg_update(new SecurityGroupUpdateTemplate());
 
     // Allocate Methods
     xmlrpc_c::methodPtr vm_allocate(new VirtualMachineAllocate());
@@ -330,10 +359,12 @@ void RequestManager::register_xml_methods()
     xmlrpc_c::methodPtr datastore_allocate(new DatastoreAllocate());
     xmlrpc_c::methodPtr cluster_allocate(new ClusterAllocate());
     xmlrpc_c::methodPtr doc_allocate(new DocumentAllocate());
+    xmlrpc_c::methodPtr secg_allocate(new SecurityGroupAllocate());
 
     // Clone Methods
     xmlrpc_c::methodPtr template_clone(new VMTemplateClone());
     xmlrpc_c::methodPtr doc_clone(new DocumentClone());
+    xmlrpc_c::methodPtr secg_clone(new SecurityGroupClone());
 
     // Delete Methods
     xmlrpc_c::methodPtr host_delete(new HostDelete());
@@ -343,6 +374,7 @@ void RequestManager::register_xml_methods()
     xmlrpc_c::methodPtr datastore_delete(new DatastoreDelete());
     xmlrpc_c::methodPtr cluster_delete(new ClusterDelete());
     xmlrpc_c::methodPtr doc_delete(new DocumentDelete());
+    xmlrpc_c::methodPtr secg_delete(new SecurityGroupDelete());
 
     // Info Methods
     xmlrpc_c::methodPtr vm_info(new VirtualMachineInfo());
@@ -353,6 +385,11 @@ void RequestManager::register_xml_methods()
     xmlrpc_c::methodPtr datastore_info(new DatastoreInfo());
     xmlrpc_c::methodPtr cluster_info(new ClusterInfo());
     xmlrpc_c::methodPtr doc_info(new DocumentInfo());
+    xmlrpc_c::methodPtr secg_info(new SecurityGroupInfo());
+
+    // Lock Methods
+    xmlrpc_c::methodPtr doc_lock(new DocumentLock());
+    xmlrpc_c::methodPtr doc_unlock(new DocumentUnlock());
 
     // PoolInfo Methods
     xmlrpc_c::methodPtr hostpool_info(new HostPoolInfo());
@@ -363,6 +400,7 @@ void RequestManager::register_xml_methods()
     xmlrpc_c::methodPtr imagepool_info(new ImagePoolInfo());
     xmlrpc_c::methodPtr clusterpool_info(new ClusterPoolInfo());
     xmlrpc_c::methodPtr docpool_info(new DocumentPoolInfo());
+    xmlrpc_c::methodPtr secgpool_info(new SecurityGroupPoolInfo());
 
     // Host Methods
     xmlrpc_c::methodPtr host_enable(new HostEnable());
@@ -374,6 +412,12 @@ void RequestManager::register_xml_methods()
     xmlrpc_c::methodPtr image_enable(new ImageEnable());
     xmlrpc_c::methodPtr image_chtype(new ImageChangeType());
     xmlrpc_c::methodPtr image_clone(new ImageClone());
+    xmlrpc_c::methodPtr image_snap_delete(new ImageSnapshotDelete());
+    xmlrpc_c::methodPtr image_snap_revert(new ImageSnapshotRevert());
+    xmlrpc_c::methodPtr image_snap_flatten(new ImageSnapshotFlatten());
+
+    // Datastore Methods
+    xmlrpc_c::methodPtr datastore_enable(new DatastoreEnable());
 
     // Chown Methods
     xmlrpc_c::methodPtr vm_chown(new VirtualMachineChown());
@@ -382,6 +426,7 @@ void RequestManager::register_xml_methods()
     xmlrpc_c::methodPtr image_chown(new ImageChown());
     xmlrpc_c::methodPtr datastore_chown(new DatastoreChown());
     xmlrpc_c::methodPtr doc_chown(new DocumentChown());
+    xmlrpc_c::methodPtr secg_chown(new SecurityGroupChown());
 
     // Chmod Methods
     xmlrpc_c::methodPtr vm_chmod(new VirtualMachineChmod());
@@ -390,6 +435,7 @@ void RequestManager::register_xml_methods()
     xmlrpc_c::methodPtr image_chmod(new ImageChmod());
     xmlrpc_c::methodPtr datastore_chmod(new DatastoreChmod());
     xmlrpc_c::methodPtr doc_chmod(new DocumentChmod());
+    xmlrpc_c::methodPtr secg_chmod(new SecurityGroupChmod());
 
     // Cluster Methods
     xmlrpc_c::methodPtr cluster_addhost(new ClusterAddHost());
@@ -412,12 +458,12 @@ void RequestManager::register_xml_methods()
     xmlrpc_c::methodPtr cluster_rename(new ClusterRename());
     xmlrpc_c::methodPtr datastore_rename(new DatastoreRename());
     xmlrpc_c::methodPtr host_rename(new HostRename());
+    xmlrpc_c::methodPtr secg_rename(new SecurityGroupRename());
 
     /* VM related methods  */
     RequestManagerRegistry.addMethod("one.vm.deploy", vm_deploy);
     RequestManagerRegistry.addMethod("one.vm.action", vm_action);
     RequestManagerRegistry.addMethod("one.vm.migrate", vm_migrate);
-    RequestManagerRegistry.addMethod("one.vm.savedisk", vm_savedisk);
     RequestManagerRegistry.addMethod("one.vm.allocate", vm_allocate);
     RequestManagerRegistry.addMethod("one.vm.info", vm_info);
     RequestManagerRegistry.addMethod("one.vm.chown", vm_chown);
@@ -433,11 +479,17 @@ void RequestManager::register_xml_methods()
     RequestManagerRegistry.addMethod("one.vm.snapshotcreate", vm_snap_create);
     RequestManagerRegistry.addMethod("one.vm.snapshotrevert", vm_snap_revert);
     RequestManagerRegistry.addMethod("one.vm.snapshotdelete", vm_snap_delete);
+    RequestManagerRegistry.addMethod("one.vm.disksaveas", vm_dsaveas);
+    RequestManagerRegistry.addMethod("one.vm.disksnapshotcreate", vm_dsnap_create);
+    RequestManagerRegistry.addMethod("one.vm.disksnapshotrevert", vm_dsnap_revert);
+    RequestManagerRegistry.addMethod("one.vm.disksnapshotdelete", vm_dsnap_delete);
     RequestManagerRegistry.addMethod("one.vm.recover", vm_recover);
 
     RequestManagerRegistry.addMethod("one.vmpool.info", vm_pool_info);
     RequestManagerRegistry.addMethod("one.vmpool.accounting", vm_pool_acct);
     RequestManagerRegistry.addMethod("one.vmpool.monitoring", vm_pool_monitoring);
+    RequestManagerRegistry.addMethod("one.vmpool.showback", vm_pool_showback);
+    RequestManagerRegistry.addMethod("one.vmpool.calculateshowback", vm_pool_calculate_showback);
 
     /* VM Template related methods*/
     RequestManagerRegistry.addMethod("one.template.update", template_update);
@@ -469,31 +521,31 @@ void RequestManager::register_xml_methods()
     xmlrpc_c::method * group_allocate_pt;
     xmlrpc_c::method * group_update_pt;
     xmlrpc_c::method * group_delete_pt;
-    xmlrpc_c::method * group_add_provider_pt;
-    xmlrpc_c::method * group_del_provider_pt;
+    xmlrpc_c::method * group_add_admin_pt;
+    xmlrpc_c::method * group_del_admin_pt;
 
     if (nebula.is_federation_slave())
     {
         group_allocate_pt       = new RequestManagerProxy("one.group.allocate");
         group_delete_pt         = new RequestManagerProxy("one.group.delete");
-        group_add_provider_pt   = new RequestManagerProxy("one.group.addprovider");
-        group_del_provider_pt   = new RequestManagerProxy("one.group.delprovider");
         group_update_pt         = new RequestManagerProxy("one.group.update");
+        group_add_admin_pt      = new RequestManagerProxy("one.group.addadmin");
+        group_del_admin_pt      = new RequestManagerProxy("one.group.deladmin");
     }
     else
     {
         group_allocate_pt       = new GroupAllocate();
         group_delete_pt         = new GroupDelete();
-        group_add_provider_pt   = new GroupAddProvider();
-        group_del_provider_pt   = new GroupDelProvider();
         group_update_pt         = new GroupUpdateTemplate();
+        group_add_admin_pt      = new GroupAddAdmin();
+        group_del_admin_pt      = new GroupDelAdmin();
     }
 
     xmlrpc_c::methodPtr group_allocate(group_allocate_pt);
     xmlrpc_c::methodPtr group_delete(group_delete_pt);
-    xmlrpc_c::methodPtr group_add_provider(group_add_provider_pt);
-    xmlrpc_c::methodPtr group_del_provider(group_del_provider_pt);
     xmlrpc_c::methodPtr group_update(group_update_pt);
+    xmlrpc_c::methodPtr group_add_admin(group_add_admin_pt);
+    xmlrpc_c::methodPtr group_del_admin(group_del_admin_pt);
 
     xmlrpc_c::methodPtr group_info(new GroupInfo());
     xmlrpc_c::methodPtr group_set_quota(new GroupSetQuota());
@@ -505,9 +557,9 @@ void RequestManager::register_xml_methods()
     RequestManagerRegistry.addMethod("one.group.delete",     group_delete);
     RequestManagerRegistry.addMethod("one.group.info",       group_info);
     RequestManagerRegistry.addMethod("one.group.quota",      group_set_quota);
-    RequestManagerRegistry.addMethod("one.group.addprovider",group_add_provider);
-    RequestManagerRegistry.addMethod("one.group.delprovider",group_del_provider);
     RequestManagerRegistry.addMethod("one.group.update",     group_update);
+    RequestManagerRegistry.addMethod("one.group.addadmin",   group_add_admin);
+    RequestManagerRegistry.addMethod("one.group.deladmin",   group_del_admin);
 
     RequestManagerRegistry.addMethod("one.grouppool.info",  grouppool_info);
 
@@ -618,6 +670,9 @@ void RequestManager::register_xml_methods()
     RequestManagerRegistry.addMethod("one.image.chtype", image_chtype);
     RequestManagerRegistry.addMethod("one.image.clone", image_clone);
     RequestManagerRegistry.addMethod("one.image.rename", image_rename);
+    RequestManagerRegistry.addMethod("one.image.snapshotdelete", image_snap_delete);
+    RequestManagerRegistry.addMethod("one.image.snapshotrevert", image_snap_revert);
+    RequestManagerRegistry.addMethod("one.image.snapshotflatten", image_snap_flatten);
 
     RequestManagerRegistry.addMethod("one.imagepool.info", imagepool_info);
 
@@ -654,6 +709,7 @@ void RequestManager::register_xml_methods()
     RequestManagerRegistry.addMethod("one.datastore.chown",   datastore_chown);
     RequestManagerRegistry.addMethod("one.datastore.chmod",   datastore_chmod);
     RequestManagerRegistry.addMethod("one.datastore.rename",  datastore_rename);
+    RequestManagerRegistry.addMethod("one.datastore.enable",  datastore_enable);
 
     RequestManagerRegistry.addMethod("one.datastorepool.info",datastorepool_info);
 
@@ -681,7 +737,9 @@ void RequestManager::register_xml_methods()
     RequestManagerRegistry.addMethod("one.document.chown",   doc_chown);
     RequestManagerRegistry.addMethod("one.document.chmod",   doc_chmod);
     RequestManagerRegistry.addMethod("one.document.clone",   doc_clone);
-    RequestManagerRegistry.addMethod("one.document.rename",   doc_rename);
+    RequestManagerRegistry.addMethod("one.document.rename",  doc_rename);
+    RequestManagerRegistry.addMethod("one.document.lock",    doc_lock);
+    RequestManagerRegistry.addMethod("one.document.unlock",  doc_unlock);
 
     RequestManagerRegistry.addMethod("one.documentpool.info",docpool_info);
 
@@ -720,6 +778,112 @@ void RequestManager::register_xml_methods()
 
     RequestManagerRegistry.addMethod("one.zonepool.info",zonepool_info);
 
+    /* Security Group objects related methods*/
+
+    RequestManagerRegistry.addMethod("one.secgroup.allocate",secg_allocate);
+    RequestManagerRegistry.addMethod("one.secgroup.delete",  secg_delete);
+    RequestManagerRegistry.addMethod("one.secgroup.info",    secg_info);
+    RequestManagerRegistry.addMethod("one.secgroup.update",  secg_update);
+    RequestManagerRegistry.addMethod("one.secgroup.chown",   secg_chown);
+    RequestManagerRegistry.addMethod("one.secgroup.chmod",   secg_chmod);
+    RequestManagerRegistry.addMethod("one.secgroup.clone",   secg_clone);
+    RequestManagerRegistry.addMethod("one.secgroup.rename",  secg_rename);
+
+    RequestManagerRegistry.addMethod("one.secgrouppool.info",secgpool_info);
+
+    /* Vdc related methods */
+
+    xmlrpc_c::method * vdc_allocate_pt;
+    xmlrpc_c::method * vdc_update_pt;
+    xmlrpc_c::method * vdc_delete_pt;
+
+    xmlrpc_c::method * vdc_add_group_pt;
+    xmlrpc_c::method * vdc_del_group_pt;
+    xmlrpc_c::method * vdc_add_cluster_pt;
+    xmlrpc_c::method * vdc_del_cluster_pt;
+    xmlrpc_c::method * vdc_add_host_pt;
+    xmlrpc_c::method * vdc_del_host_pt;
+    xmlrpc_c::method * vdc_add_datastore_pt;
+    xmlrpc_c::method * vdc_del_datastore_pt;
+    xmlrpc_c::method * vdc_add_vnet_pt;
+    xmlrpc_c::method * vdc_del_vnet_pt;
+
+    if (nebula.is_federation_slave())
+    {
+        vdc_allocate_pt     = new RequestManagerProxy("one.vdc.allocate");
+        vdc_update_pt       = new RequestManagerProxy("one.vdc.update");
+        vdc_delete_pt       = new RequestManagerProxy("one.vdc.delete");
+
+        vdc_add_group_pt    = new RequestManagerProxy("one.vdc.addgroup");
+        vdc_del_group_pt    = new RequestManagerProxy("one.vdc.delgroup");
+        vdc_add_cluster_pt  = new RequestManagerProxy("one.vdc.addcluster");
+        vdc_del_cluster_pt  = new RequestManagerProxy("one.vdc.delcluster");
+        vdc_add_host_pt     = new RequestManagerProxy("one.vdc.addhost");
+        vdc_del_host_pt     = new RequestManagerProxy("one.vdc.delhost");
+        vdc_add_datastore_pt= new RequestManagerProxy("one.vdc.adddatastore");
+        vdc_del_datastore_pt= new RequestManagerProxy("one.vdc.deldatastore");
+        vdc_add_vnet_pt     = new RequestManagerProxy("one.vdc.addvnet");
+        vdc_del_vnet_pt     = new RequestManagerProxy("one.vdc.delvnet");
+    }
+    else
+    {
+        vdc_allocate_pt     = new VdcAllocate();
+        vdc_update_pt       = new VdcUpdateTemplate();
+        vdc_delete_pt       = new VdcDelete();
+
+        vdc_add_group_pt    = new VdcAddGroup();
+        vdc_del_group_pt    = new VdcDelGroup();
+        vdc_add_cluster_pt  = new VdcAddCluster();
+        vdc_del_cluster_pt  = new VdcDelCluster();
+        vdc_add_host_pt     = new VdcAddHost();
+        vdc_del_host_pt     = new VdcDelHost();
+        vdc_add_datastore_pt= new VdcAddDatastore();
+        vdc_del_datastore_pt= new VdcDelDatastore();
+        vdc_add_vnet_pt     = new VdcAddVNet();
+        vdc_del_vnet_pt     = new VdcDelVNet();
+    }
+
+    xmlrpc_c::methodPtr vdc_allocate(vdc_allocate_pt);
+    xmlrpc_c::methodPtr vdc_update(vdc_update_pt);
+    xmlrpc_c::methodPtr vdc_delete(vdc_delete_pt);
+
+    xmlrpc_c::methodPtr vdc_add_group(vdc_add_group_pt);
+    xmlrpc_c::methodPtr vdc_del_group(vdc_del_group_pt);
+    xmlrpc_c::methodPtr vdc_add_cluster(vdc_add_cluster_pt);
+    xmlrpc_c::methodPtr vdc_del_cluster(vdc_del_cluster_pt);
+    xmlrpc_c::methodPtr vdc_add_host(vdc_add_host_pt);
+    xmlrpc_c::methodPtr vdc_del_host(vdc_del_host_pt);
+    xmlrpc_c::methodPtr vdc_add_datastore(vdc_add_datastore_pt);
+    xmlrpc_c::methodPtr vdc_del_datastore(vdc_del_datastore_pt);
+    xmlrpc_c::methodPtr vdc_add_vnet(vdc_add_vnet_pt);
+    xmlrpc_c::methodPtr vdc_del_vnet(vdc_del_vnet_pt);
+
+
+    xmlrpc_c::methodPtr vdc_info(new VdcInfo());
+    xmlrpc_c::methodPtr vdc_rename(new VdcRename());
+    xmlrpc_c::methodPtr vdcpool_info(new VdcPoolInfo());
+
+    RequestManagerRegistry.addMethod("one.vdc.allocate",    vdc_allocate);
+    RequestManagerRegistry.addMethod("one.vdc.update",      vdc_update);
+    RequestManagerRegistry.addMethod("one.vdc.delete",      vdc_delete);
+
+    RequestManagerRegistry.addMethod("one.vdc.addgroup",    vdc_add_group);
+    RequestManagerRegistry.addMethod("one.vdc.delgroup",    vdc_del_group);
+    RequestManagerRegistry.addMethod("one.vdc.addcluster",  vdc_add_cluster);
+    RequestManagerRegistry.addMethod("one.vdc.delcluster",  vdc_del_cluster);
+
+    RequestManagerRegistry.addMethod("one.vdc.addhost",     vdc_add_host);
+    RequestManagerRegistry.addMethod("one.vdc.delhost",     vdc_del_host);
+    RequestManagerRegistry.addMethod("one.vdc.adddatastore",vdc_add_datastore);
+    RequestManagerRegistry.addMethod("one.vdc.deldatastore",vdc_del_datastore);
+    RequestManagerRegistry.addMethod("one.vdc.addvnet",     vdc_add_vnet);
+    RequestManagerRegistry.addMethod("one.vdc.delvnet",     vdc_del_vnet);
+
+    RequestManagerRegistry.addMethod("one.vdc.info",        vdc_info);
+    RequestManagerRegistry.addMethod("one.vdc.rename",      vdc_rename);
+
+    RequestManagerRegistry.addMethod("one.vdcpool.info",    vdcpool_info);
+
     /* System related methods */
     RequestManagerRegistry.addMethod("one.system.version", system_version);
     RequestManagerRegistry.addMethod("one.system.config", system_config);
