@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -60,7 +60,7 @@ PoolObjectSQL * RequestManagerChown::get_and_quota(
         tmpl        = new Template;
 
         tmpl->add("DATASTORE", img->get_ds_id());
-        tmpl->add("SIZE", img->get_size());
+        tmpl->add("SIZE",img->get_size()+img->get_snapshots().get_total_size());
 
         qtype = Quotas::DATASTORE;
     }
