@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -179,50 +179,12 @@ int HostDelete::drop(int oid, PoolObjectSQL * object, string& error_msg)
 
 int ImageDelete::drop(int oid, PoolObjectSQL * object, string& error_msg)
 {
-    Nebula&         nd     = Nebula::instance();
+    Nebula&        nd     = Nebula::instance();
+    ImageManager * imagem = nd.get_imagem();
 
-    ImageManager *  imagem = nd.get_imagem();
-    DatastorePool * dspool = nd.get_dspool();
+    object->unlock();
 
-    Datastore * ds;
-    Image *     img;
-
-    int    ds_id, rc;
-    string ds_data;
-
-    img   = static_cast<Image *>(object);
-    ds_id = img->get_ds_id();
-
-    img->unlock();
-
-    ds = dspool->get(ds_id, true);
-
-    if ( ds == 0 )
-    {
-       error_msg = "Datastore no longer exists cannot remove image";
-       return -1;
-    }
-
-    ds->to_xml(ds_data);
-
-    ds->unlock();
-
-    rc = imagem->delete_image(oid, ds_data, error_msg);
-
-    if ( rc == 0 )
-    {
-        ds = dspool->get(ds_id, true);
-
-        if ( ds != 0 )
-        {
-            ds->del_image(oid);
-            dspool->update(ds);
-
-            ds->unlock();
-        }
-    }
-
-    return rc;
+    return imagem->delete_image(oid, error_msg);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -373,3 +335,31 @@ int VirtualNetworkDelete::drop(int oid, PoolObjectSQL * object, string& error_ms
 
     return rc;
 }
+
+/* ------------------------------------------------------------------------- */
+/* ------------------------------------------------------------------------- */
+
+int SecurityGroupDelete::drop(int oid, PoolObjectSQL * object, string& error_msg)
+{
+    if (object->get_oid() == 0)
+    {
+        error_msg = "The default security group (ID 0) cannot be deleted.";
+
+        object->unlock();
+
+        return -1;
+    }
+
+    SecurityGroup * sgroup = static_cast<SecurityGroup *>(object);
+
+    if ( sgroup->get_vms() > 0 )
+    {
+        error_msg = "The security group has VMs using it";
+
+        sgroup->unlock();
+
+        return -1;
+    }
+
+    return RequestManagerDelete::drop(oid, object, error_msg);
+}
