@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -42,6 +42,7 @@ void VMTemplateInstantiate::request_execute(xmlrpc_c::paramList const& paramList
     VMTemplatePool *    tpool   = static_cast<VMTemplatePool *>(pool);
 
     VirtualMachineTemplate * tmpl;
+    VirtualMachineTemplate * extended_tmpl = 0;
     VirtualMachineTemplate   uattrs;
     VMTemplate *             rtmpl;
 
@@ -79,24 +80,6 @@ void VMTemplateInstantiate::request_execute(xmlrpc_c::paramList const& paramList
 
     rtmpl->unlock();
 
-    // Check template for restricted attributes, only if owner is not oneadmin
-    if (perms.uid!=UserPool::ONEADMIN_ID && perms.gid!=GroupPool::ONEADMIN_ID)
-    {
-        if (tmpl->check(aname))
-        {
-            ostringstream oss;
-
-            oss << "VM Template includes a restricted attribute " << aname;
-
-            failure_response(AUTHORIZATION,
-                    authorization_error(oss.str(), att),
-                    att);
-
-            delete tmpl;
-            return;
-        }
-    }
-
     // Parse & merge user attributes (check if the request user is not oneadmin)
     if (!str_uattrs.empty())
     {
@@ -181,15 +164,18 @@ void VMTemplateInstantiate::request_execute(xmlrpc_c::paramList const& paramList
             return;
         }
 
-        if ( quota_authorization(tmpl, Quotas::VIRTUALMACHINE, att) == false )
+        extended_tmpl = new VirtualMachineTemplate(*tmpl);
+
+        VirtualMachine::disk_extended_info(att.uid, extended_tmpl);
+
+        if ( quota_authorization(extended_tmpl, Quotas::VIRTUALMACHINE, att) == false )
         {
             delete tmpl;
+            delete extended_tmpl;
             return;
         }
     }
 
-    Template tmpl_back(*tmpl);
-
     rc = vmpool->allocate(att.uid, att.gid, att.uname, att.gname, att.umask,
             tmpl, &vid, error_str, on_hold);
 
@@ -199,11 +185,18 @@ void VMTemplateInstantiate::request_execute(xmlrpc_c::paramList const& paramList
                 allocate_error(PoolObjectSQL::VM,error_str),
                 att);
 
-        quota_rollback(&tmpl_back, Quotas::VIRTUALMACHINE, att);
+        if (extended_tmpl != 0)
+        {
+            quota_rollback(extended_tmpl, Quotas::VIRTUALMACHINE, att);
+        }
+
+        delete extended_tmpl;
 
         return;
     }
 
+    delete extended_tmpl;
+
     success_response(vid, att);
 }
 
