@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -20,6 +20,7 @@
 
 #include <map>
 #include "ObjectXML.h"
+#include "HostShare.h"
 
 using namespace std;
 
@@ -50,12 +51,24 @@ public:
      *  Tests whether a new VM can be hosted by the host or not
      *    @param cpu needed by the VM (percentage)
      *    @param mem needed by the VM (in KB)
+     *    @param pci devices needed by the VM
+     *    @param error error message
      *    @return true if the share can host the VM
      */
-    bool test_capacity(long long cpu, long long mem) const
+    bool test_capacity(long long cpu, long long mem, vector<Attribute *> &pci,
+        string & error);
+
+    /**
+     *  Tests whether a new VM can be hosted by the host or not
+     *    @param cpu needed by the VM (percentage)
+     *    @param mem needed by the VM (in KB)
+     *    @param pci devices needed by the VM
+     *    @return true if the share can host the VM
+     */
+    bool test_capacity(long long cpu,long long mem,vector<Attribute *> &p)
     {
-        return (((max_cpu  - cpu_usage ) >= cpu) &&
-                ((max_mem  - mem_usage ) >= mem));
+        string tmp_st;
+        return test_capacity(cpu, mem, p, tmp_st);
     };
 
     /**
@@ -65,11 +78,14 @@ public:
      *    @param mem needed by the VM (in KB)
      *    @return 0 on success
      */
-    void add_capacity(long long cpu, long long mem)
+    void add_capacity(int vmid, long long cpu, long long mem,
+        vector<Attribute *> &p)
     {
         cpu_usage  += cpu;
         mem_usage  += mem;
 
+        pci.add(p, vmid);
+
         running_vms++;
     };
 
@@ -122,6 +138,12 @@ public:
         return public_cloud;
     }
 
+    /**
+     *  Prints the Host information to an output stream. This function is used
+     *  for logging purposes.
+     */
+    friend ostream& operator<<(ostream& o, const HostXML& p);
+
 private:
     int oid;
     int cluster_id;
@@ -141,6 +163,8 @@ private:
 
     bool public_cloud;
 
+    HostSharePCI pci;
+
     // Configuration attributes
     static const char *host_paths[]; /**< paths for search function */
 
