@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -91,6 +91,11 @@ public:
         return (resched == 1);
     }
 
+    bool is_resume() const
+    {
+        return resume;
+    }
+
     const string& get_rank()
     {
         return rank;
@@ -111,7 +116,8 @@ public:
         return ds_requirements;
     }
 
-    void get_requirements (int& cpu, int& memory, long long& disk);
+    void get_requirements (int& cpu, int& memory, long long& disk,
+        vector<Attribute *> &pci);
 
     map<int,long long> get_storage_usage();
 
@@ -215,9 +221,23 @@ public:
     /**
      *  Tests if the Image DS have enough free space to host the VM
      *    @param img_datastores Image Datastores
+     *    @param error_msg error reason
      *    @return true if the Image Datastores can host the VM
      */
-    bool test_image_datastore_capacity(ImageDatastorePoolXML * img_dspool);
+    bool test_image_datastore_capacity(
+            ImageDatastorePoolXML * img_dspool, string & error_msg) const;
+
+    /**
+     *  Tests if the Image DS have enough free space to host the VM
+     *    @param img_datastores Image Datastores
+     *    @return true if the Image Datastores can host the VM
+     */
+    bool test_image_datastore_capacity(
+            ImageDatastorePoolXML * img_dspool) const
+    {
+        string tmp_st;
+        return test_image_datastore_capacity(img_dspool, tmp_st);
+    }
 
     /**
      *  Adds the VM disk requirements to each Image Datastore counter
@@ -289,6 +309,13 @@ public:
      */
     void log(const string &st);
 
+    /**
+     * Clears the VM's USER_TEMPLATE/SCHED_MESSAGE attribute
+     * @return true if the template was modified, false if SCHED_MESSAGE did not
+     * need to be deleted
+     */
+    bool clear_log();
+
 protected:
 
     /**
@@ -314,6 +341,7 @@ protected:
     int   dsid;
 
     int   resched;
+    bool  resume;
 
     int         memory;
     float       cpu;
