@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -20,6 +20,7 @@
 
 int VirtualMachinePoolXML::set_up()
 {
+    ostringstream   oss;
 
     int rc = PoolXML::set_up();
 
@@ -30,14 +31,15 @@ int VirtualMachinePoolXML::set_up()
             return -2;
         }
 
-        if (NebulaLog::log_level() == Log::DEBUG)
+        if (NebulaLog::log_level() >= Log::DDDEBUG)
         {
-            ostringstream   oss;
             oss << "Pending/rescheduling VM and capacity requirements:" << endl;
 
-            oss << right << setw(8)  << "VM"        << " "
+            oss << right << setw(8)  << "ACTION"    << " "
+                << right << setw(8)  << "VM"        << " "
                 << right << setw(4)  << "CPU"       << " "
                 << right << setw(11) << "Memory"    << " "
+                << right << setw(3)  << "PCI"       << " "
                 << right << setw(11) << "System DS" << " "
                 << " Image DS"
                 << endl << setw(60) << setfill('-') << "-" << setfill(' ');
@@ -46,15 +48,29 @@ int VirtualMachinePoolXML::set_up()
             {
                 int cpu, mem;
                 long long disk;
+                vector<Attribute *> pci;
+
+                string action = "DEPLOY";
 
                 VirtualMachineXML * vm = static_cast<VirtualMachineXML *>(it->second);
 
-                vm->get_requirements(cpu, mem, disk);
+                vm->get_requirements(cpu, mem, disk, pci);
+
+                if (vm->is_resched())
+                {
+                    action = "RESCHED";
+                }
+                else if (vm->is_resume())
+                {
+                    action = "RESUME";
+                }
 
                 oss << endl
+                    << right << setw(8)  << action      << " "
                     << right << setw(8)  << it->first   << " "
                     << right << setw(4)  << cpu         << " "
                     << right << setw(11) << mem         << " "
+                    << right << setw(3)  << pci.size()  << " "
                     << right << setw(11) << disk        << " ";
 
                 map<int,long long> ds_usage = vm->get_storage_usage();
@@ -65,9 +81,13 @@ int VirtualMachinePoolXML::set_up()
                     oss << " DS " << ds_it->first << ": " << ds_it->second << " ";
                 }
             }
-
-            NebulaLog::log("VM",Log::DEBUG,oss);
         }
+        else
+        {
+            oss << "Found " << objects.size() << " pending/rescheduling VMs.";
+        }
+
+        NebulaLog::log("VM",Log::DEBUG,oss);
     }
 
     return rc;
@@ -124,20 +144,14 @@ int VirtualMachinePoolXML::load_info(xmlrpc_c::value &result)
 
 int VirtualMachinePoolXML::dispatch(int vid, int hid, int dsid, bool resched) const
 {
-    ostringstream               oss;
-    xmlrpc_c::value             deploy_result;
+    xmlrpc_c::value deploy_result;
 
-    if (resched == true)
-    {
-        oss << "Rescheduling " << "VM " << vid << " to host " << hid;
-    }
-    else
-    {
-        oss << "Dispatching " << "VM " << vid << " to host " << hid
-            << " and datastore " << dsid;
-    }
+    VirtualMachineXML* vm = get(vid);
 
-    NebulaLog::log("VM",Log::INFO,oss);
+    if (vm != 0 && vm->clear_log())
+    {
+        update(vm);
+    }
 
     try
     {
@@ -168,7 +182,8 @@ int VirtualMachinePoolXML::dispatch(int vid, int hid, int dsid, bool resched) co
     }
     catch (exception const& e)
     {
-        oss.str("");
+        ostringstream   oss;
+
         oss << "Exception raised: " << e.what() << '\n';
 
         NebulaLog::log("VM",Log::ERROR,oss);
@@ -176,8 +191,6 @@ int VirtualMachinePoolXML::dispatch(int vid, int hid, int dsid, bool resched) co
         return -1;
     }
 
-    // See how ONE handled the deployment
-
     vector<xmlrpc_c::value> values =
                     xmlrpc_c::value_array(deploy_result).vectorValueValue();
 
@@ -185,9 +198,9 @@ int VirtualMachinePoolXML::dispatch(int vid, int hid, int dsid, bool resched) co
 
     if ( !success )
     {
+        ostringstream oss;
         string message = xmlrpc_c::value_string(values[1]);
 
-        oss.str("");
         oss << "Error deploying virtual machine " << vid
             << " to HID: " << hid << ". Reason: " << message;
 
