@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -20,93 +20,52 @@
 #include "DatastoreXML.h"
 #include "DatastorePoolXML.h"
 #include "NebulaUtil.h"
+#include "History.h"
 
 void VirtualMachineXML::init_attributes()
 {
-    vector<string>     result;
     vector<xmlNodePtr> nodes;
 
+    int rc;
+    int action;
+
     string automatic_requirements;
 
-    oid = atoi(((*this)["/VM/ID"] )[0].c_str());
-    uid = atoi(((*this)["/VM/UID"])[0].c_str());
-    gid = atoi(((*this)["/VM/GID"])[0].c_str());
+    xpath(oid, "/VM/ID", -1);
+    xpath(uid, "/VM/UID", -1);
+    xpath(gid, "/VM/GID", -1);
 
-    result = ((*this)["/VM/TEMPLATE/MEMORY"]);
-
-    if (result.size() > 0)
-    {
-        memory = atoi(result[0].c_str());
-    }
-    else
-    {
-        memory = 0;
-    }
-
-    result = ((*this)["/VM/TEMPLATE/CPU"]);
-
-    if (result.size() > 0)
-    {
-        istringstream   iss;
-        iss.str( result[0] );
-        iss >> cpu;
-    }
-    else
-    {
-        cpu = 0;
-    }
+    xpath(memory, "/VM/TEMPLATE/MEMORY", 0);
+    xpath(cpu,    "/VM/TEMPLATE/CPU", 0);
 
     // ------------------------ RANK & DS_RANK ---------------------------------
 
-    result = ((*this)["/VM/USER_TEMPLATE/SCHED_RANK"]);
+    rc = xpath(rank, "/VM/USER_TEMPLATE/SCHED_RANK", "");
 
-    if (result.size() > 0)
-    {
-        rank = result[0];
-    }
-    else
+    if (rc != 0)
     {
         // Compatibility with previous versions
-        result = ((*this)["/VM/USER_TEMPLATE/RANK"]);
-
-        if (result.size() > 0)
-        {
-            rank = result[0];
-        }
+        xpath(rank, "/VM/USER_TEMPLATE/RANK", "");
     }
 
-    result = ((*this)["/VM/USER_TEMPLATE/SCHED_DS_RANK"]);
-
-    if (result.size() > 0)
-    {
-        ds_rank = result[0];
-    }
+    xpath(ds_rank, "/VM/USER_TEMPLATE/SCHED_DS_RANK", "");
 
     // ------------------- HOST REQUIREMENTS -----------------------------------
 
-    result = ((*this)["/VM/TEMPLATE/AUTOMATIC_REQUIREMENTS"]);
+    xpath(automatic_requirements, "/VM/TEMPLATE/AUTOMATIC_REQUIREMENTS", "");
 
-    if (result.size() > 0)
-    {
-        automatic_requirements = result[0];
-    }
+    rc = xpath(requirements, "/VM/USER_TEMPLATE/SCHED_REQUIREMENTS", "");
 
-    result = ((*this)["/VM/USER_TEMPLATE/SCHED_REQUIREMENTS"]);
-
-    if (result.size() > 0)
+    if (rc == 0)
     {
         if ( !automatic_requirements.empty() )
         {
             ostringstream oss;
 
-            oss << automatic_requirements << " & ( " << result[0] << " )";
+            oss << automatic_requirements << " & ( " << requirements << " )";
 
             requirements = oss.str();
         }
-        else
-        {
-            requirements = result[0];
-        }
     }
     else if ( !automatic_requirements.empty() )
     {
@@ -115,22 +74,18 @@ void VirtualMachineXML::init_attributes()
 
     // ------------------- DS REQUIREMENTS -------------------------------------
 
-    result = ((*this)["/VM/USER_TEMPLATE/SCHED_DS_REQUIREMENTS"]);
+    rc = xpath(ds_requirements, "/VM/USER_TEMPLATE/SCHED_DS_REQUIREMENTS", "");
 
-    if (result.size() > 0)
+    if (rc == 0)
     {
         if ( !automatic_requirements.empty() )
         {
             ostringstream oss;
 
-            oss << automatic_requirements << " & ( " << result[0] << " )";
+            oss << automatic_requirements << " & ( " << ds_requirements << " )";
 
             ds_requirements = oss.str();
         }
-        else
-        {
-            ds_requirements = result[0];
-        }
     }
     else if ( !automatic_requirements.empty() )
     {
@@ -139,38 +94,16 @@ void VirtualMachineXML::init_attributes()
 
     // ---------------- HISTORY HID, DSID, RESCHED & TEMPLATE ------------------
 
-    result = ((*this)["/VM/HISTORY_RECORDS/HISTORY/HID"]);
+    xpath(hid,  "/VM/HISTORY_RECORDS/HISTORY/HID", -1);
+    xpath(dsid, "/VM/HISTORY_RECORDS/HISTORY/DS_ID", -1);
 
-    if (result.size() > 0)
-    {
-        hid = atoi(result[0].c_str());
-    }
-    else
-    {
-        hid = -1;
-    }
+    xpath(resched, "/VM/RESCHED", 0);
 
-    result = ((*this)["/VM/HISTORY_RECORDS/HISTORY/DS_ID"]);
+    xpath(action, "/VM/HISTORY_RECORDS/HISTORY/ACTION", -1);
 
-    if (result.size() > 0)
-    {
-        dsid = atoi(result[0].c_str());
-    }
-    else
-    {
-        dsid = -1;
-    }
-
-    result = ((*this)["/VM/RESCHED"]);
-
-    if (result.size() > 0)
-    {
-        resched = atoi(result[0].c_str());
-    }
-    else
-    {
-        resched = 0;
-    }
+    resume = (action == History::STOP_ACTION ||
+              action == History::UNDEPLOY_ACTION ||
+              action == History::UNDEPLOY_HARD_ACTION );
 
     if (get_nodes("/VM/TEMPLATE", nodes) > 0)
     {
@@ -269,8 +202,16 @@ ostream& operator<<(ostream& os, VirtualMachineXML& vm)
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-void VirtualMachineXML::get_requirements (int& cpu, int& memory, long long& disk)
+void VirtualMachineXML::get_requirements (int& cpu, int& memory,
+    long long& disk, vector<Attribute *> &pci)
 {
+    pci.clear();
+
+    if (vm_template != 0)
+    {
+        vm_template->get("PCI", pci);
+    }
+
     if (this->memory == 0 || this->cpu == 0)
     {
         cpu    = 0;
@@ -309,6 +250,7 @@ void VirtualMachineXML::init_storage_usage()
     vector<Attribute*>::iterator    it;
 
     long long   size;
+    long long   snapshot_size;
     string      st;
     int         ds_id;
     bool        clone;
@@ -331,6 +273,11 @@ void VirtualMachineXML::init_storage_usage()
             continue;
         }
 
+        if (disk->vector_value("DISK_SNAPSHOT_TOTAL_SIZE", snapshot_size) == 0)
+        {
+            size += snapshot_size;
+        }
+
         if (isVolatile(disk))
         {
             system_ds_usage += size;
@@ -399,6 +346,30 @@ void VirtualMachineXML::log(const string &st)
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+bool VirtualMachineXML::clear_log()
+{
+    string st;
+
+    if (user_template == 0)
+    {
+        return false;
+    }
+
+    user_template->get("SCHED_MESSAGE", st);
+
+    if (st.empty())
+    {
+        return false;
+    }
+
+    user_template->erase("SCHED_MESSAGE");
+
+    return true;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 int VirtualMachineXML::parse_action_name(string& action_st)
 {
     one_util::tolower(action_st);
@@ -412,7 +383,6 @@ int VirtualMachineXML::parse_action_name(string& action_st)
         && action_st != "stop"
         && action_st != "suspend"
         && action_st != "resume"
-        && action_st != "boot"
         && action_st != "delete"
         && action_st != "delete-recreate"
         && action_st != "reboot"
@@ -431,7 +401,7 @@ int VirtualMachineXML::parse_action_name(string& action_st)
 /* -------------------------------------------------------------------------- */
 
 bool VirtualMachineXML::test_image_datastore_capacity(
-    ImageDatastorePoolXML * img_dspool)
+    ImageDatastorePoolXML * img_dspool, string & error_msg) const
 {
     map<int,long long>::const_iterator ds_usage_it;
     DatastoreXML* ds;
@@ -440,29 +410,14 @@ bool VirtualMachineXML::test_image_datastore_capacity(
     {
         ds = img_dspool->get(ds_usage_it->first);
 
-        if (ds == 0) //Should never reach here
+        if (ds == 0 || !ds->test_capacity(ds_usage_it->second))
         {
             ostringstream oss;
 
-            oss << "Image Datastore " << ds_usage_it->first << " not found.";
-
-            NebulaLog::log("SCHED",Log::INFO,oss);
-
-            return false;
-        }
-
-        if (!ds->test_capacity(ds_usage_it->second))
-        {
-            ostringstream oss;
-
-            oss << "VM " << oid
-                << ": Image Datastore " << ds_usage_it->first
-                << " does not have enough free storage.";
-
-            NebulaLog::log("SCHED",Log::INFO,oss);
-
-            log(oss.str());
+            oss << "Image Datastore " << ds->get_oid()
+                << " does not have enough capacity";
 
+            error_msg = oss.str();
             return false;
         }
     }
