@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -51,6 +51,8 @@ module OpenNebulaJSON
                  when "chown"         then self.chown(action_hash['params'])
                  when "chmod"         then self.chmod_octet(action_hash['params'])
                  when "rename"        then self.rename(action_hash['params'])
+                 when "enable"        then self.enable
+                 when "disable"       then self.disable
                  else
                      error_msg = "#{action_hash['perform']} action not " <<
                          " available for this resource"
