@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -43,7 +43,6 @@ module OpenNebulaJSON
             end
 
             rc = case action_hash['perform']
-                 when "cancel"       then self.cancel
                  when "deploy"       then self.deploy(action_hash['params'])
                  when "finalize"     then self.finalize
                  when "hold"         then self.hold
@@ -53,13 +52,15 @@ module OpenNebulaJSON
                  when "release"      then self.release
                  when "stop"         then self.stop
                  when "suspend"      then self.suspend
-                 when "restart"      then self.restart
                  when "reset"        then self.reset
-                 when "saveas"       then self.save_as(action_hash['params'])
+                 when "disk_saveas"  then self.disk_saveas(action_hash['params'])
                  when "snapshot_create"       then self.snapshot_create(action_hash['params'])
                  when "snapshot_revert"       then self.snapshot_revert(action_hash['params'])
                  when "snapshot_delete"       then self.snapshot_delete(action_hash['params'])
-                 when "shutdown"     then self.shutdown
+                 when "disk_snapshot_create"  then self.disk_snapshot_create(action_hash['params'])
+                 when "disk_snapshot_revert"  then self.disk_snapshot_revert(action_hash['params'])
+                 when "disk_snapshot_delete"  then self.disk_snapshot_delete(action_hash['params'])
+                 when "shutdown"     then self.shutdown(action_hash['params'])
                  when "reboot"       then self.reboot
                  when "poweroff"     then self.poweroff(action_hash['params'])
                  when "resubmit"     then self.resubmit
@@ -76,6 +77,7 @@ module OpenNebulaJSON
                  when "resched"      then self.resched
                  when "unresched"    then self.unresched
                  when "recover"      then self.recover(action_hash['params'])
+                 when "save_as_template" then self.save_as_template(action_hash['params'])
                  else
                      error_msg = "#{action_hash['perform']} action not " <<
                          " available for this resource"
@@ -95,19 +97,21 @@ module OpenNebulaJSON
             super(params['hard'])
         end
 
+        def shutdown(params=Hash.new)
+            super(params['hard'])
+        end
+
         def poweroff(params=Hash.new)
             super(params['hard'])
         end
 
         def migrate(params=Hash.new, live=false)
-            super(params['host_id'], live, params['enforce'])
+            super(params['host_id'], live, params['enforce'], params['ds_id'])
         end
 
-        def save_as(params=Hash.new)
-            clone = params['clonetemplate']
-            clone = false if clone.nil?
-
-            disk_snapshot(params['disk_id'].to_i, params['image_name'], params['type'], params['hot'], clone)
+        def disk_saveas(params=Hash.new)
+            super(params['disk_id'].to_i, params['image_name'],
+                params['type'], params['snapshot_id'].to_i)
         end
 
         def snapshot_create(params=Hash.new)
@@ -122,6 +126,18 @@ module OpenNebulaJSON
             super(params['snapshot_id'].to_i)
         end
 
+        def disk_snapshot_create(params=Hash.new)
+            super(params['disk_id'].to_i, params['snapshot_name'])
+        end
+
+        def disk_snapshot_revert(params=Hash.new)
+            super(params['disk_id'].to_i, params['snapshot_id'].to_i)
+        end
+
+        def disk_snapshot_delete(params=Hash.new)
+            super(params['disk_id'].to_i, params['snapshot_id'].to_i)
+        end
+
         def chown(params=Hash.new)
             super(params['owner_id'].to_i,params['group_id'].to_i)
         end
@@ -165,8 +181,13 @@ module OpenNebulaJSON
         end
 
         def recover(params=Hash.new)
-            result = params['with'] == "success" ? true : false
-            super(result)
+            super(params['result'].to_i)
+        end
+
+        def save_as_template(params=Hash.new)
+            vm_new = VirtualMachine.new(VirtualMachine.build_xml(@pe_id),
+                                        @client)
+            vm_new.save_as_template(params['name'], params['persistent'])
         end
     end
 end
