@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -63,6 +63,8 @@ class SunstoneServer < CloudServer
             when "acl"        then AclPoolJSON.new(client)
             when "datastore"  then DatastorePoolJSON.new(client)
             when "zone"       then ZonePoolJSON.new(client)
+            when "security_group"   then SecurityGroupPoolJSON.new(client, user_flag)
+            when "vdc"        then VdcPoolJSON.new(client)
             else
                 error = Error.new("Error: #{kind} resource not supported")
                 return [404, error.to_json]
@@ -80,8 +82,8 @@ class SunstoneServer < CloudServer
     ############################################################################
     #
     ############################################################################
-    def get_resource(kind, id)
-        resource = retrieve_resource(kind, id)
+    def get_resource(kind, id, extended=false)
+        resource = retrieve_resource(kind, id, extended)
         if OpenNebula.is_error?(resource)
             return [404, resource.to_json]
         else
@@ -118,6 +120,8 @@ class SunstoneServer < CloudServer
             when "acl"        then AclJSON.new(Acl.build_xml, @client)
             when "datastore"  then DatastoreJSON.new(Acl.build_xml, @client)
             when "zone"       then ZoneJSON.new(Zone.build_xml, @client)
+            when "security_group"   then SecurityGroupJSON.new(SecurityGroup.build_xml, @client)
+            when "vdc"        then VdcJSON.new(Vdc.build_xml, @client)
             else
                 error = Error.new("Error: #{kind} resource not supported")
                 return [404, error.to_json]
@@ -391,13 +395,40 @@ class SunstoneServer < CloudServer
         return [200, rc.to_json]
     end
 
+    def get_vm_showback(options)
+        pool = VirtualMachinePool.new(@client)
+
+        filter_flag = options[:userfilter] ? options[:userfilter].to_i : VirtualMachinePool::INFO_ALL
+        start_month  = options[:start_month] ? options[:start_month].to_i : -1
+        start_year  = options[:start_year] ? options[:start_year].to_i : -1
+        end_month    = options[:end_month]   ? options[:end_month].to_i : -1
+        end_year    = options[:end_year]   ? options[:end_year].to_i : -1
+
+        acct_options = {
+            :start_month => start_month,
+            :start_year => start_year,
+            :end_month   => end_month,
+            :end_year   => end_year,
+            :group      => options[:group]
+        }
+
+        rc = pool.showback(filter_flag, acct_options)
+
+        if OpenNebula.is_error?(rc)
+            error = Error.new(rc.message)
+            return [500, error.to_json]
+        end
+
+        return [200, rc.to_json]
+    end
+
 
     private
 
     ############################################################################
     #
     ############################################################################
-    def retrieve_resource(kind, id)
+    def retrieve_resource(kind, id, extended=false)
         resource = case kind
             when "group"      then GroupJSON.new_with_id(id, @client)
             when "cluster"    then ClusterJSON.new_with_id(id, @client)
@@ -410,12 +441,14 @@ class SunstoneServer < CloudServer
             when "acl"        then AclJSON.new_with_id(id, @client)
             when "datastore"  then DatastoreJSON.new_with_id(id, @client)
             when "zone"       then ZoneJSON.new_with_id(id, @client)
+            when "security_group"   then SecurityGroupJSON.new_with_id(id, @client)
+            when "vdc"        then VdcJSON.new_with_id(id, @client)
             else
                 error = Error.new("Error: #{kind} resource not supported")
                 return error
         end
 
-        rc = resource.info
+        rc = extended ? resource.info(true) : resource.info
         if OpenNebula.is_error?(rc)
             return rc
         else
