@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -32,10 +32,10 @@ class SunstoneViews
     VIEWS_CONFIGURATION_FILE = ETC_LOCATION + "/sunstone-views.yaml"
     VIEWS_CONFIGURATION_DIR  = ETC_LOCATION + "/sunstone-views/"
 
-	def initialize
-		@views_config = YAML.load_file(VIEWS_CONFIGURATION_FILE)
+    def initialize
+        @views_config = YAML.load_file(VIEWS_CONFIGURATION_FILE)
 
-		base_path = SUNSTONE_ROOT_DIR+'/public/js/'
+        base_path = SUNSTONE_ROOT_DIR+'/public/js/'
 
         @views = Hash.new
 
@@ -45,17 +45,17 @@ class SunstoneViews
                 @views[m[1]] = YAML.load_file(p_path)
             end
         end
-	end
+    end
 
-	def view(user_name, group_name, view_name=nil)
+    def view(user_name, group_name, view_name=nil)
         available_views = available_views(user_name, group_name)
 
-		if view_name && available_views.include?(view_name)
-			return @views[view_name]
-		else
-			return @views[available_views.first]
-		end
-	end
+        if view_name && available_views.include?(view_name)
+            return @views[view_name]
+        else
+            return @views[available_views.first]
+        end
+    end
 
     # Return the name of the views avialable to a user. Those defined in the
     # group template and configured in sunstone. If no view is defined in a
@@ -87,10 +87,10 @@ class SunstoneViews
                 available << @views_config['groups'][group.name]
             end
 
-            gadmins       = group["TEMPLATE/GROUP_ADMINS"]
+            gadmins       = group.admin_ids
             gadmins_views = group["TEMPLATE/GROUP_ADMIN_VIEWS"]
 
-            if gadmins && gadmins.split(',').include?(user_name) && gadmins_views
+            if gadmins && gadmins.include?(user.id) && gadmins_views
                 views_array = gadmins_views.split(",")
                 available << views_array.each{|v| v.strip!}
             end
@@ -121,10 +121,6 @@ class SunstoneViews
         @views.keys
     end
 
-    def available_tabs
-        @views_config['available_tabs']
-    end
-
     def logo
         @views_config['logo']
     end
