@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -101,6 +101,17 @@ when 'memcache'
     use Rack::Session::Memcache,
         :memcache_server => memcache_server,
         :namespace => $conf[:memcache_namespace]
+when 'memcache-dalli'
+    require 'rack/session/dalli'
+    memcache_server=$conf[:memcache_host]+':'<<
+        $conf[:memcache_port].to_s
+
+    STDERR.puts memcache_server
+
+    use Rack::Session::Dalli,
+      :memcache_server => memcache_server,
+      :namespace => $conf[:memcache_namespace],
+      :cache => Dalli::Client.new
 else
     STDERR.puts "Wrong value for :sessions in configuration file"
     exit(-1)
@@ -139,6 +150,7 @@ configure do
 end
 
 DEFAULT_TABLE_ORDER = "desc"
+DEFAULT_PAGE_LENGTH = 10
 
 ##############################################################################
 # Helpers
@@ -178,7 +190,7 @@ helpers do
             logger.info { "Unauthorized login attempt" }
             return [401, ""]
         else
-            client  = $cloud_auth.client(result)
+            client  = $cloud_auth.client(result, session[:active_zone_endpoint])
             user_id = OpenNebula::User::SELF
 
             user    = OpenNebula::User.new_with_id(user_id, client)
@@ -218,6 +230,12 @@ helpers do
                 session[:lang] = $conf[:lang]
             end
 
+            if user['TEMPLATE/TABLE_DEFAULT_PAGE_LENGTH']
+                session[:page_length] = user['TEMPLATE/TABLE_DEFAULT_PAGE_LENGTH']
+            else
+                session[:page_length] = DEFAULT_PAGE_LENGTH
+            end
+
             wss = $conf[:vnc_proxy_support_wss]
             #limit to yes,no options
             session[:vnc_wss] = (wss == true || wss == "yes" || wss == "only" ?
@@ -245,7 +263,7 @@ helpers do
                 env['rack.session.options'][:expire_after] = 30*60*60*24-1
             end
 
-            serveradmin_client = $cloud_auth.client()
+            serveradmin_client = $cloud_auth.client(nil, session[:active_zone_endpoint])
             rc = OpenNebula::System.new(serveradmin_client).get_configuration
             return [500, rc.message] if OpenNebula.is_error?(rc)
             return [500, "Couldn't find out zone identifier"] if !rc['FEDERATION/ZONE_ID']
@@ -254,6 +272,8 @@ helpers do
             zone.info
             session[:zone_name] = zone.name
 
+            session[:federation_mode] = rc['FEDERATION/MODE'].upcase
+
             return [204, ""]
         end
     end
@@ -280,23 +300,45 @@ before do
     end
 
     if env['HTTP_ZONE_NAME']
-        client=$cloud_auth.client(session[:user])
+        client = $cloud_auth.client(session[:user], session[:active_zone_endpoint])
         zpool = ZonePoolJSON.new(client)
 
         rc = zpool.info
 
-        return [500, rc.to_json] if OpenNebula.is_error?(rc)
+        halt [500, rc.to_json] if OpenNebula.is_error?(rc)
 
+        found = false
         zpool.each{|z|
             if z.name == env['HTTP_ZONE_NAME']
-              session[:active_zone_endpoint] = z['TEMPLATE/ENDPOINT']
-              session[:zone_name] = env['HTTP_ZONE_NAME']
+                found = true
+                serveradmin_client = $cloud_auth.client(nil, z['TEMPLATE/ENDPOINT'])
+                rc = OpenNebula::System.new(serveradmin_client).get_configuration
+
+                if OpenNebula.is_error?(rc)
+                    msg = "Zone #{env['HTTP_ZONE_NAME']} not available " + rc.message
+                    logger.error { msg }
+                    halt [410, OpenNebula::Error.new(msg).to_json]
+                end
+
+                if !rc['FEDERATION/ZONE_ID']
+                    msg = "Couldn't find out zone identifier"
+                    logger.error { msg }
+                    halt [500, OpenNebula::Error.new(msg).to_json]
+                end
+
+                session[:active_zone_endpoint] = z['TEMPLATE/ENDPOINT']
+                session[:zone_name] = env['HTTP_ZONE_NAME']
             end
          }
+
+         if !found
+            msg = "Zone #{env['HTTP_ZONE_NAME']} does not exist"
+            logger.error { msg }
+            halt [404, OpenNebula::Error.new(msg).to_json]
+        end
     end
 
-    client = $cloud_auth.client(session[:user],
-                              session[:active_zone_endpoint])
+    client = $cloud_auth.client(session[:user], session[:active_zone_endpoint])
 
     @SunstoneServer = SunstoneServer.new(client,$conf,logger)
 end
@@ -398,7 +440,7 @@ post '/config' do
 
     user = OpenNebula::User.new_with_id(
                 OpenNebula::User::SELF,
-                $cloud_auth.client(session[:user]))
+                $cloud_auth.client(session[:user], session[:active_zone_endpoint]))
 
     rc = user.info
     if OpenNebula.is_error?(rc)
@@ -410,9 +452,67 @@ post '/config' do
     session[:vnc_wss]      = user['TEMPLATE/VNC_WSS'] if user['TEMPLATE/VNC_WSS']
     session[:default_view] = user['TEMPLATE/DEFAULT_VIEW'] if user['TEMPLATE/DEFAULT_VIEW']
     session[:table_order]  = user['TEMPLATE/TABLE_ORDER'] if user['TEMPLATE/TABLE_ORDER']
+    session[:page_length]  = user['TEMPLATE/TABLE_DEFAULT_PAGE_LENGTH'] if user['TEMPLATE/TABLE_DEFAULT_PAGE_LENGTH']
     session[:display_name] = user[DISPLAY_NAME_XPATH] || user['NAME']
 
-    [200, ""]
+    [204, ""]
+end
+
+get '/infrastructure' do
+    serveradmin_client = $cloud_auth.client(nil, session[:active_zone_endpoint])
+
+    hpool = OpenNebula::HostPool.new(serveradmin_client)
+
+    rc = hpool.info
+
+    if OpenNebula.is_error?(rc)
+        logger.error { rc.message }
+        error 500, ""
+    end
+
+    infrastructure = {}
+
+    set = Set.new
+
+    xml = XMLElement.new
+    xml.initialize_xml(hpool.to_xml, 'HOST_POOL')
+    xml.each('HOST/HOST_SHARE/PCI_DEVICES/PCI') do |pci|
+        set.add({
+            :device => pci['DEVICE'],
+            :class  => pci['CLASS'],
+            :vendor => pci['VENDOR'],
+            :device_name => pci['DEVICE_NAME']
+        })
+    end
+
+    infrastructure[:pci_devices] = set.to_a
+
+    set = Set.new
+
+    xml.each('HOST/TEMPLATE/CUSTOMIZATION') do |customization|
+        set.add(customization['NAME'])
+    end
+
+    infrastructure[:vcenter_customizations] = set.to_a
+
+    [200, infrastructure.to_json]
+end
+
+get '/onedconf' do
+    serveradmin_client = $cloud_auth.client(nil, session[:active_zone_endpoint])
+
+    rc = OpenNebula::System.new(serveradmin_client).get_configuration
+
+    if OpenNebula.is_error?(rc)
+        logger.error { rc.message }
+        error 500, ""
+    end
+
+    onedconf = {
+        :DEFAULT_COST => rc.to_hash()['TEMPLATE']['DEFAULT_COST']
+    }
+
+    [200, onedconf.to_json]
 end
 
 get '/vm/:id/log' do
@@ -478,9 +578,11 @@ end
 get '/:pool' do
     zone_client = nil
 
-    if params[:zone_id]
+    if params[:zone_id] && session[:federation_mode] != "STANDALONE"
         zone = OpenNebula::Zone.new_with_id(params[:zone_id].to_i,
-                                            $cloud_auth.client(session[:user]))
+                                            $cloud_auth.client(session[:user],
+                                                session[:active_zone_endpoint]))
+
         rc   = zone.info
         return [500, rc.message] if OpenNebula.is_error?(rc)
         zone_client = $cloud_auth.client(session[:user],
@@ -501,7 +603,11 @@ get '/:resource/:id/template' do
 end
 
 get '/:resource/:id' do
-    @SunstoneServer.get_resource(params[:resource], params[:id])
+    if params[:extended]
+        @SunstoneServer.get_resource(params[:resource], params[:id], true)
+    else
+        @SunstoneServer.get_resource(params[:resource], params[:id])
+    end
 end
 
 ##############################################################################
@@ -599,4 +705,3 @@ post '/:resource/:id/action' do
 end
 
 Sinatra::Application.run! if(!defined?(WITH_RACKUP))
-
