@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -16,14 +16,13 @@
 
 #include "Template.h"
 #include "template_syntax.h"
+#include "NebulaUtil.h"
 
 #include <iostream>
 #include <sstream>
 #include <cstring>
 #include <cstdio>
 
-#define TO_UPPER(S) transform(S.begin(),S.end(),S.begin(),(int(*)(int))toupper)
-
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
@@ -524,7 +523,7 @@ bool Template::get(
         return false;
     }
 
-    TO_UPPER(sval);
+    one_util::toupper(sval);
 
     if ( sval == "YES" )
     {
@@ -544,7 +543,7 @@ bool Template::get(
 string& Template::to_xml(string& xml) const
 {
     multimap<string,Attribute *>::const_iterator  it;
-    ostringstream                   		oss;
+    ostringstream                           oss;
     string *                                s;
 
     oss << "<" << xml_root << ">";
@@ -562,14 +561,14 @@ string& Template::to_xml(string& xml) const
 
     xml = oss.str();
 
-	return xml;
+    return xml;
 }
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
 string& Template::to_str(string& str) const
 {
-	ostringstream os;
+    ostringstream os;
     multimap<string,Attribute *>::const_iterator  it;
     string *                                s;
 
@@ -582,7 +581,7 @@ string& Template::to_str(string& str) const
         delete s;
     }
 
-	str = os.str();
+    str = os.str();
     return str;
 }
 
@@ -609,9 +608,9 @@ bool Template::trim(const string& name)
 
 ostream& operator << (ostream& os, const Template& t)
 {
-	string str;
+    string str;
 
-	os << t.to_str(str);
+    os << t.to_str(str);
 
     return os;
 }
@@ -747,18 +746,7 @@ void Template::rebuild_attributes(const xmlNode * root_element)
 
     Attribute * attr;
 
-    //Clear the template if not empty
-    if (!attributes.empty())
-    {
-        multimap<string,Attribute *>::iterator  it;
-
-        for ( it = attributes.begin(); it != attributes.end(); it++)
-        {
-            delete it->second;
-        }
-
-        attributes.clear();
-    }
+    clear();
 
     // Get the root's children and try to build attributes.
     for (cur_node = root_element->children;
@@ -976,17 +964,31 @@ void Template::remove_all_except_restricted(const vector<string> &restricted_att
         remove(*res_it);
     }
 
-    multimap<string,Attribute *>::iterator  att_it;
-
-    for ( att_it = attributes.begin(); att_it != attributes.end(); att_it++)
-    {
-        delete att_it->second;
-    }
-
-    attributes.clear();
+    clear();
 
     for (res_it = restricted.begin(); res_it != restricted.end(); res_it++)
     {
         set(*res_it);
     }
 }
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+void Template::clear()
+{
+    if (attributes.empty())
+    {
+        return;
+    }
+
+    multimap<string,Attribute *>::iterator it;
+
+    for ( it = attributes.begin(); it != attributes.end(); it++)
+    {
+        delete it->second;
+    }
+
+    attributes.clear();
+}
+
