@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -15,7 +15,7 @@
 /* -------------------------------------------------------------------------- */
 
 /* ************************************************************************** */
-/* User Pool                                            	 	      */
+/* User Pool                                                                  */
 /* ************************************************************************** */
 
 #include "UserPool.h"
@@ -36,7 +36,6 @@
 const char * UserPool::CORE_AUTH    = "core";
 const char * UserPool::SERVER_AUTH  = "server*";
 const char * UserPool::PUBLIC_AUTH  = "public";
-const char * UserPool::DEFAULT_AUTH = "default";
 
 const char * UserPool::SERVER_NAME  = "serveradmin";
 
@@ -90,6 +89,7 @@ UserPool::UserPool(SqlDB * db,
             throw("Database has not been bootstrapped with master data.");
         }
 
+        oneadmin_name = oneadmin_user->get_name();
         oneadmin_user->unlock();
 
         return;
@@ -247,7 +247,7 @@ error_no_open:
 
 erro_exists:
     oss << "Password file " << filenames[i] << " already exists "
-        << "but OpenNebula is boostraping the database. Check your " 
+        << "but OpenNebula is boostraping the database. Check your "
         << "database configuration in oned.conf.";
     goto error_common;
 
@@ -712,6 +712,7 @@ bool UserPool::authenticate_external(const string&  username,
     string mad_pass;
     string error_str;
     string tmp_str;
+    string default_auth;
 
     Nebula&     nd      = Nebula::instance();
     AuthManager * authm = nd.get_authm();
@@ -733,7 +734,9 @@ bool UserPool::authenticate_external(const string&  username,
     }
 
     //Initialize authentication request and call the driver
-    ar.add_authenticate(UserPool::DEFAULT_AUTH, username,"-",token);
+    nd.get_configuration_attribute("DEFAULT_AUTH",default_auth);
+
+    ar.add_authenticate(default_auth, username,"-",token);
 
     authm->trigger(AuthManager::AUTHENTICATE, &ar);
     ar.wait();
