@@ -231,7 +231,8 @@ void get_network_attribute(VirtualMachine * vm,
 
     attr_value.clear();
 
-    if ( net_name.empty() || (net_name!="NETWORK" && net_name!="NETWORK_ID") )
+    if ( net_name.empty() ||
+        (net_name!="NETWORK" && net_name!="NETWORK_ID" && net_name!="NIC_ID"))
     {
         return;
     }
@@ -284,7 +285,7 @@ void get_network_attribute(VirtualMachine * vm,
 
     if (attr_name == "TEMPLATE")
     {
-        attr_value = vn->to_xml64(attr_value, true);
+        attr_value = vn->to_xml64(attr_value);
     }
     else
     {
