@@ -1,5 +1,5 @@
 /* ------------------------------------------------------------------------ */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs      */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems              */
 /*                                                                          */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may  */
 /* not use this file except in compliance with the License. You may obtain  */
@@ -172,6 +172,14 @@ error_common:
 /* ************************************************************************ */
 
 string& VMTemplate::to_xml(string& xml) const
+{
+    return to_xml(xml, obj_template);
+}
+
+/* ------------------------------------------------------------------------ */
+/* ------------------------------------------------------------------------ */
+
+string& VMTemplate::to_xml(string& xml, const Template* tmpl) const
 {
     ostringstream   oss;
     string          template_xml;
@@ -186,7 +194,7 @@ string& VMTemplate::to_xml(string& xml) const
             << "<NAME>"     << name       << "</NAME>"
             << perms_to_xml(perm_str)
             << "<REGTIME>"  << regtime    << "</REGTIME>"
-            << obj_template->to_xml(template_xml)
+            << tmpl->to_xml(template_xml)
         << "</VMTEMPLATE>";
 
     xml = oss.str();
