@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -24,12 +24,17 @@
 
 const float LibVirtDriver::CGROUP_BASE_CPU_SHARES = 1024;
 
+const int LibVirtDriver::CEPH_DEFAULT_PORT = 6789;
+
+const int LibVirtDriver::GLUSTER_DEFAULT_PORT = 24007;
+
 /**
  *  This function generates the <host> element for network disks
  */
 static void do_network_hosts(ofstream& file,
                              const string& cg_host,
-                             const string& transport)
+                             const string& transport,
+                             int   default_port)
 {
     if (cg_host.empty())
     {
@@ -59,6 +64,10 @@ static void do_network_hosts(ofstream& file,
         {
             file << "' port='" << parts[1];
         }
+        else if ( default_port != -1 )
+        {
+            file << "' port='" << default_port;
+        }
 
         if (!transport.empty())
         {
@@ -110,11 +119,13 @@ int LibVirtDriver::deployment_description_kvm(
     string  driver          = "";
     string  cache           = "";
     string  disk_io         = "";
+    string  discard         = "";
     string  source          = "";
     string  clone           = "";
     string  ceph_host       = "";
     string  ceph_secret     = "";
     string  ceph_user       = "";
+    string  sheepdog_host   = "";
     string  gluster_host    = "";
     string  gluster_volume  = "";
 
@@ -136,6 +147,7 @@ int LibVirtDriver::deployment_description_kvm(
     string  default_driver          = "";
     string  default_driver_cache    = "";
     string  default_driver_disk_io  = "";
+    string  default_driver_discard  = "";
     bool    readonly;
 
     const VectorAttribute * nic;
@@ -161,6 +173,13 @@ int LibVirtDriver::deployment_description_kvm(
 
     const VectorAttribute * input;
 
+    const VectorAttribute * pci;
+
+    string  domain          = "";
+    /* bus is already defined for disks */
+    string  slot            = "";
+    string  func            = "";
+
     const VectorAttribute * features;
 
     bool pae        = false;
@@ -397,7 +416,7 @@ int LibVirtDriver::deployment_description_kvm(
     }
 
     get_default("DISK", "IO", default_driver_disk_io);
-
+    get_default("DISK", "DISCARD", default_driver_discard);
     get_default("DISK", "TOTAL_BYTES_SEC", default_total_bytes_sec);
     get_default("DISK", "READ_BYTES_SEC", default_read_bytes_sec);
     get_default("DISK", "WRITE_BYTES_SEC", default_write_bytes_sec);
@@ -424,6 +443,7 @@ int LibVirtDriver::deployment_description_kvm(
         driver          = disk->vector_value("DRIVER");
         cache           = disk->vector_value("CACHE");
         disk_io         = disk->vector_value("IO");
+        discard         = disk->vector_value("DISCARD");
         source          = disk->vector_value("SOURCE");
         clone           = disk->vector_value("CLONE");
 
@@ -434,6 +454,7 @@ int LibVirtDriver::deployment_description_kvm(
         gluster_host    = disk->vector_value("GLUSTER_HOST");
         gluster_volume  = disk->vector_value("GLUSTER_VOLUME");
 
+        sheepdog_host   = disk->vector_value("SHEEPDOG_HOST");
         total_bytes_sec = disk->vector_value("TOTAL_BYTES_SEC");
         read_bytes_sec  = disk->vector_value("READ_BYTES_SEC");
         write_bytes_sec = disk->vector_value("WRITE_BYTES_SEC");
@@ -518,7 +539,7 @@ int LibVirtDriver::deployment_description_kvm(
                 file << "-" << vm->get_oid() << "-" << disk_id;
             }
 
-            do_network_hosts(file, ceph_host, "");
+            do_network_hosts(file, ceph_host, "", CEPH_DEFAULT_PORT);
 
             if ( !ceph_secret.empty() && !ceph_user.empty())
             {
@@ -528,6 +549,26 @@ int LibVirtDriver::deployment_description_kvm(
                      << "\t\t\t</auth>" << endl;
             }
         }
+        else if ( type == "SHEEPDOG" || type == "SHEEPDOG_CDROM" )
+        {
+            if (type == "SHEEPDOG")
+            {
+                file << "\t\t<disk type='network' device='disk'>" << endl;
+            }
+            else
+            {
+                file << "\t\t<disk type='network' device='cdrom'>" << endl;
+            }
+
+            file << "\t\t\t<source protocol='sheepdog' name='" << source;
+
+            if ( clone == "YES" )
+            {
+                file << "-" << vm->get_oid() << "-" << disk_id;
+            }
+
+            do_network_hosts(file, sheepdog_host, "tcp", -1);
+        }
         else if ( type == "GLUSTER" || type == "GLUSTER_CDROM" )
         {
             if ( type == "GLUSTER" )
@@ -551,7 +592,7 @@ int LibVirtDriver::deployment_description_kvm(
                 file << one_util::split(source, '/').back();
             }
 
-            do_network_hosts(file, gluster_host, "tcp");
+            do_network_hosts(file, gluster_host, "tcp", GLUSTER_DEFAULT_PORT);
         }
         else if ( type == "CDROM" )
         {
@@ -614,6 +655,15 @@ int LibVirtDriver::deployment_description_kvm(
             file << " io='" << default_driver_disk_io << "'";
         }
 
+        if ( !discard.empty() )
+        {
+            file << " discard='" << discard << "'";
+        }
+        else if ( !default_driver_discard.empty() )
+        {
+            file << " discard='" << default_driver_discard << "'";
+        }
+
         file << "/>" << endl;
 
         // ---- I/O Options  ----
@@ -906,6 +956,50 @@ int LibVirtDriver::deployment_description_kvm(
 
     attrs.clear();
 
+    // ------------------------------------------------------------------------
+    // PCI Passthrough
+    // ------------------------------------------------------------------------
+
+    num = vm->get_template_attribute("PCI",attrs);
+
+    for (int i=0; i < num ;i++)
+    {
+        pci = dynamic_cast<const VectorAttribute *>(attrs[i]);
+
+        if ( pci == 0 )
+        {
+            continue;
+        }
+
+        domain  = pci->vector_value("DOMAIN");
+        bus     = pci->vector_value("BUS");
+        slot    = pci->vector_value("SLOT");
+        func    = pci->vector_value("FUNCTION");
+
+        if ( domain.empty() || bus.empty() || slot.empty() || func.empty() )
+        {
+            vm->log("VMM", Log::WARNING,
+                    "DOMAIN, BUS, SLOT and FUNC must be defined for PCI "
+                    "passthrough. Ignored.");
+            continue;
+        }
+
+        file << "\t\t<hostdev mode='subsystem' type='pci' managed='yes'>";
+        file << endl;
+        file << "\t\t\t<source>" << endl;
+
+        file << "\t\t\t\t<address ";
+        file << "domain='0x" << domain << "' ";
+        file << "bus='0x" << bus << "' ";
+        file << "slot='0x" << slot << "' ";
+        file << "function='0x" << func << "'/>" << endl;
+
+        file << "\t\t\t</source>" << endl;
+        file << "\t\t</hostdev>" << endl;
+    }
+
+    attrs.clear();
+
     file << "\t</devices>" << endl;
 
     // ------------------------------------------------------------------------
@@ -1024,6 +1118,15 @@ int LibVirtDriver::deployment_description_kvm(
         file << "\t" << default_raw << endl;
     }
 
+    // ------------------------------------------------------------------------
+    // Metadata used by drivers
+    // ------------------------------------------------------------------------
+
+    file << "\t<metadata>" << endl;
+    file << "\t\t<system_datastore>" << vm->get_remote_system_dir() <<
+        "</system_datastore>" << endl;
+    file << "\t</metadata>" << endl;
+
     file << "</domain>" << endl;
 
     file.close();
