@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -197,6 +197,14 @@ void VirtualMachineManager::trigger(Actions action, int _vid)
         aname = "SNAPSHOT_DELETE";
         break;
 
+    case DISK_SNAPSHOT_CREATE:
+        aname = "DISK_SNAPSHOT_CREATE";
+        break;
+
+    case DISK_SNAPSHOT_REVERT:
+        aname = "DISK_SNAPSHOT_REVERT";
+        break;
+
     default:
         delete vid;
         return;
@@ -312,6 +320,14 @@ void VirtualMachineManager::do_action(const string &action, void * arg)
     {
         snapshot_delete_action(vid);
     }
+    else if (action == "DISK_SNAPSHOT_CREATE")
+    {
+        disk_snapshot_create_action(vid);
+    }
+    else if (action == "DISK_SNAPSHOT_REVERT")
+    {
+        disk_snapshot_revert_action(vid);
+    }
     else if (action == ACTION_TIMER)
     {
         timer_action();
@@ -532,7 +548,7 @@ void VirtualMachineManager::save_action(
     VirtualMachine *                    vm;
     const VirtualMachineManagerDriver * vmd;
 
-    string        hostname, vnm_mad;
+    string        hostname, vnm_mad, checkpoint_file;
     string        vm_tmpl;
     string *      drv_msg;
     ostringstream os;
@@ -566,13 +582,15 @@ void VirtualMachineManager::save_action(
             goto error_previous_history;
         }
 
-        hostname = vm->get_previous_hostname();
-        vnm_mad  = vm->get_previous_vnm_mad();
+        hostname        = vm->get_previous_hostname();
+        vnm_mad         = vm->get_previous_vnm_mad();
+        checkpoint_file = vm->get_previous_checkpoint_file();
     }
     else
     {
-        hostname = vm->get_hostname();
-        vnm_mad  = vm->get_vnm_mad();
+        hostname        = vm->get_hostname();
+        vnm_mad         = vm->get_vnm_mad();
+        checkpoint_file = vm->get_checkpoint_file();
     }
 
     // Invoke driver method
@@ -584,7 +602,7 @@ void VirtualMachineManager::save_action(
         vm->get_deploy_id(),
         "",
         "",
-        vm->get_checkpoint_file(),
+        checkpoint_file,
         "",
         "",
         "",
@@ -1634,6 +1652,7 @@ void VirtualMachineManager::attach_action(
     }
 
     // Get the driver for this VM
+
     vmd = get(vm->get_vmm_mad());
 
     if ( vmd == 0 )
@@ -2088,6 +2107,234 @@ error_common:
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+void VirtualMachineManager::disk_snapshot_create_action(int vid)
+{
+    VirtualMachine *                    vm;
+    const VirtualMachineManagerDriver * vmd;
+
+    ostringstream os;
+
+    string  vm_tmpl;
+    string* drv_msg;
+
+    string  vm_tm_mad;
+    string  snap_cmd;
+    string  disk_path;
+
+    string  tm_mad;
+    int     ds_id, disk_id, snap_id;
+
+    int rc;
+
+    Nebula& nd           = Nebula::instance();
+    TransferManager * tm = nd.get_tm();
+
+    vm = vmpool->get(vid,true);
+
+    if (vm == 0)
+    {
+        return;
+    }
+
+    if (!vm->hasHistory())
+    {
+        goto error_history;
+    }
+
+    vmd = get(vm->get_vmm_mad());
+
+    if ( vmd == 0 )
+    {
+        goto error_driver;
+    }
+
+    if ( vm->get_snapshot_disk(ds_id, tm_mad, disk_id, snap_id) != 0 )
+    {
+        goto error_disk;
+    }
+
+    rc = tm->snapshot_transfer_command( vm, "SNAP_CREATE", os);
+
+    snap_cmd = os.str();
+
+    os.str("");
+
+    if ( snap_cmd.empty() || rc != 0 )
+    {
+        goto error_no_tm_command;
+    }
+
+    os << vm->get_remote_system_dir() << "/disk." << disk_id;
+
+    disk_path = os.str();
+
+    // Invoke driver method
+    drv_msg = format_message(
+        vm->get_hostname(),
+        vm->get_vnm_mad(),
+        "",
+        "",
+        vm->get_deploy_id(),
+        "",
+        "",
+        vm->get_checkpoint_file(),
+        snap_cmd,
+        "",
+        disk_path,
+        vm->to_xml(vm_tmpl));
+
+    vmd->disk_snapshot_create(vid, *drv_msg);
+
+    vm->unlock();
+
+    delete drv_msg;
+
+    return;
+
+error_disk:
+    os << "disk_snapshot_create, could not find disk to take snapshot";
+    goto error_common;
+
+error_history:
+    os << "disk_snapshot_create, VM has no history";
+    goto error_common;
+
+error_driver:
+    os << "disk_snapshot_create, error getting driver " << vm->get_vmm_mad();
+    goto error_common;
+
+error_no_tm_command:
+    os << "Cannot set disk for snapshot.";
+    goto error_common;
+
+error_common:
+    Nebula              &ne = Nebula::instance();
+    LifeCycleManager *  lcm = ne.get_lcm();
+
+    lcm->trigger(LifeCycleManager::DISK_SNAPSHOT_FAILURE, vid);
+
+    vm->log("VMM", Log::ERROR, os);
+    vm->unlock();
+    return;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+void VirtualMachineManager::disk_snapshot_revert_action(int vid)
+{
+    VirtualMachine *                    vm;
+    const VirtualMachineManagerDriver * vmd;
+
+    ostringstream os;
+
+    string  vm_tmpl;
+    string* drv_msg;
+
+    string  vm_tm_mad;
+    string  snap_cmd;
+    string  disk_path;
+
+    string  tm_mad;
+    int     ds_id, disk_id, snap_id;
+
+    int rc;
+
+    Nebula& nd           = Nebula::instance();
+    TransferManager * tm = nd.get_tm();
+
+    vm = vmpool->get(vid,true);
+
+    if (vm == 0)
+    {
+        return;
+    }
+
+    if (!vm->hasHistory())
+    {
+        goto error_history;
+    }
+
+    vmd = get(vm->get_vmm_mad());
+
+    if ( vmd == 0 )
+    {
+        goto error_driver;
+    }
+
+    if ( vm->get_snapshot_disk(ds_id, tm_mad, disk_id, snap_id) != 0 )
+    {
+        goto error_disk;
+    }
+
+    rc = tm->snapshot_transfer_command(vm, "SNAP_REVERT", os);
+
+    snap_cmd = os.str();
+
+    os.str("");
+
+    if ( snap_cmd.empty() || rc != 0 )
+    {
+        goto error_no_tm_command;
+    }
+
+    os << vm->get_remote_system_dir() << "/disk." << disk_id;
+
+    disk_path = os.str();
+
+    // Invoke driver method
+    drv_msg = format_message(
+        vm->get_hostname(),
+        vm->get_vnm_mad(),
+        "",
+        "",
+        vm->get_deploy_id(),
+        "",
+        "",
+        vm->get_checkpoint_file(),
+        snap_cmd,
+        "",
+        disk_path,
+        vm->to_xml(vm_tmpl));
+
+    vmd->disk_snapshot_revert(vid, *drv_msg);
+
+    vm->unlock();
+
+    delete drv_msg;
+
+    return;
+
+error_disk:
+    os << "disk_snapshot_revert, could not find disk to revert snapshot";
+    goto error_common;
+
+error_history:
+    os << "disk_snapshot_revert, VM has no history";
+    goto error_common;
+
+error_driver:
+    os << "disk_snapshot_revert, error getting driver " << vm->get_vmm_mad();
+    goto error_common;
+
+error_no_tm_command:
+    os << "Cannot set disk for snapshot.";
+    goto error_common;
+
+error_common:
+    Nebula              &ne = Nebula::instance();
+    LifeCycleManager *  lcm = ne.get_lcm();
+
+    lcm->trigger(LifeCycleManager::DISK_SNAPSHOT_FAILURE, vid);
+
+    vm->log("VMM", Log::ERROR, os);
+    vm->unlock();
+    return;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 void VirtualMachineManager::attach_nic_action(
     int vid)
 {
