@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -234,8 +234,7 @@ void VirtualMachineManagerDriver::protocol(const string& message) const
         return;
     }
 
-    if ( vm->get_lcm_state() == VirtualMachine::FAILURE ||
-         vm->get_lcm_state() == VirtualMachine::LCM_INIT )
+    if ( vm->get_lcm_state() == VirtualMachine::LCM_INIT )
     {
         os.str("");
         os << "Ignored: " << message;
@@ -257,7 +256,7 @@ void VirtualMachineManagerDriver::protocol(const string& message) const
 
             is >> deploy_id;
 
-            vm->update_info(deploy_id);
+            vm->set_deploy_id(deploy_id);
 
             vmpool->update(vm);
 
@@ -521,6 +520,44 @@ void VirtualMachineManagerDriver::protocol(const string& message) const
             lcm->trigger(LifeCycleManager::SNAPSHOT_DELETE_FAILURE, id);
         }
     }
+    else if ( action == "DISKSNAPSHOTCREATE" )
+    {
+        Nebula           &ne  = Nebula::instance();
+        LifeCycleManager *lcm = ne.get_lcm();
+
+        if ( result == "SUCCESS" )
+        {
+            vm->log("VMM", Log::INFO, "VM disk snapshot successfully created.");
+
+            lcm->trigger(LifeCycleManager::DISK_SNAPSHOT_SUCCESS, id);
+        }
+        else
+        {
+            log_error(vm, os, is, "Error creating new disk snapshot");
+            vmpool->update(vm);
+
+            lcm->trigger(LifeCycleManager::DISK_SNAPSHOT_FAILURE, id);
+        }
+    }
+    else if ( action == "DISKSNAPSHOTREVERT" )
+    {
+        Nebula           &ne  = Nebula::instance();
+        LifeCycleManager *lcm = ne.get_lcm();
+
+        if ( result == "SUCCESS" )
+        {
+            vm->log("VMM", Log::INFO, "VM disk state reverted.");
+
+            lcm->trigger(LifeCycleManager::DISK_SNAPSHOT_SUCCESS, id);
+        }
+        else
+        {
+            log_error(vm, os, is, "Error reverting disk snapshot");
+            vmpool->update(vm);
+
+            lcm->trigger(LifeCycleManager::DISK_SNAPSHOT_FAILURE, id);
+        }
+    }
     else if ( action == "CLEANUP" )
     {
         Nebula           &ne  = Nebula::instance();
@@ -597,111 +634,110 @@ void VirtualMachineManagerDriver::process_poll(
         VirtualMachine* vm,
         const string&   monitor_str)
 {
-    int rc;
-
-    int        cpu;
-    int        memory;
-    long long  net_tx;
-    long long  net_rx;
-    char       state;
-
-    map<string, string> custom;
-    ostringstream oss;
+    char state;
 
     Nebula &ne = Nebula::instance();
 
     LifeCycleManager* lcm      = ne.get_lcm();
     VirtualMachinePool* vmpool = ne.get_vmpool();
 
-    /* ---------------------------------------------------------------------- */
-    /* Parse VM info                                                          */
-    /* ---------------------------------------------------------------------- */
-
-    rc = parse_vm_info(monitor_str, cpu, memory, net_tx, net_rx, state, custom);
-
-    if (rc == -1) //Parse error, ignore this monitor data
-    {
-        oss << "Ignoring monitoring information, parse error."
-            << " Monitor information was: "
-            << monitor_str;
-
-        NebulaLog::log("VMM", Log::ERROR, oss);
-
-        vm->set_template_error_message(oss.str());
-
-        vm->log("VMM", Log::ERROR, oss);
-
-        vmpool->update(vm);
-
-        return;
-    }
-
-    oss << "VM " << vm->get_oid() << " successfully monitored: " << monitor_str;
-    NebulaLog::log("VMM", Log::DEBUG, oss);
-
     /* ---------------------------------------------------------------------- */
     /* Update VM info only for VMs in ACTIVE                                  */
     /* ---------------------------------------------------------------------- */
 
     if (vm->get_state() == VirtualMachine::ACTIVE)
     {
-        vm->update_info(memory, cpu, net_tx, net_rx, custom);
+        if (vm->update_info(monitor_str) == 0)
+        {
+            vmpool->update_history(vm);
+
+            vmpool->update_monitoring(vm);
+        }
 
         vmpool->update(vm);
 
-        vmpool->update_history(vm);
+        VirtualMachineMonitorInfo &minfo = vm->get_info();
 
-        vmpool->update_monitoring(vm);
+        state = minfo.remove_state();
+    }
+    else
+    {
+        VirtualMachineMonitorInfo minfo;
+        string error;
+
+        if (minfo.update(monitor_str, error) != 0)
+        {
+            ostringstream oss;
+
+            oss << "Ignoring monitoring information, error:" << error
+                << ". Monitor information was: " << monitor_str;
+
+            NebulaLog::log("VMM", Log::ERROR, oss);
+
+            return;
+        };
+
+        state = minfo.remove_state();
     }
 
     /* ---------------------------------------------------------------------- */
     /* Process the VM state from the monitoring info                          */
     /* ---------------------------------------------------------------------- */
 
-    bool process_state = vm->get_state() == VirtualMachine::POWEROFF || (
-        vm->get_state() == VirtualMachine::ACTIVE && (
-            vm->get_lcm_state() == VirtualMachine::RUNNING ||
-            vm->get_lcm_state() == VirtualMachine::UNKNOWN));
-
-    if (!process_state)
-    {
-        return;
-    }
-
     switch (state)
     {
-        case 'a': // Still active, good!
-            if ( vm->get_lcm_state() == VirtualMachine::UNKNOWN)
-            {
-                vm->log("VMM", Log::INFO, "VM found again, state is RUNNING");
+        case 'a': // Active
 
-                vm->set_state(VirtualMachine::RUNNING);
-                vmpool->update(vm);
-            }
-            else if ( vm->get_state() == VirtualMachine::POWEROFF )
+            if ( vm->get_state() == VirtualMachine::POWEROFF ||
+                 (vm->get_state() == VirtualMachine::ACTIVE &&
+                 (  vm->get_lcm_state() == VirtualMachine::UNKNOWN ||
+                    vm->get_lcm_state() == VirtualMachine::BOOT ||
+                    vm->get_lcm_state() == VirtualMachine::BOOT_POWEROFF ||
+                    vm->get_lcm_state() == VirtualMachine::BOOT_UNKNOWN  ||
+                    vm->get_lcm_state() == VirtualMachine::BOOT_SUSPENDED ||
+                    vm->get_lcm_state() == VirtualMachine::BOOT_STOPPED ||
+                    vm->get_lcm_state() == VirtualMachine::BOOT_UNDEPLOY ||
+                    vm->get_lcm_state() == VirtualMachine::BOOT_MIGRATE ||
+                    vm->get_lcm_state() == VirtualMachine::BOOT_MIGRATE_FAILURE ||
+                    vm->get_lcm_state() == VirtualMachine::BOOT_STOPPED_FAILURE ||
+                    vm->get_lcm_state() == VirtualMachine::BOOT_UNDEPLOY_FAILURE ||
+                    vm->get_lcm_state() == VirtualMachine::BOOT_FAILURE )))
             {
-                vm->log("VMM", Log::INFO, "VM found again, state is RUNNING");
-
                 lcm->trigger(LifeCycleManager::MONITOR_POWERON, vm->get_oid());
             }
+
             break;
 
         case 'p': // It's paused
-            vm->log("VMM",Log::INFO, "VM running but monitor state is PAUSED.");
+            if ( vm->get_state() == VirtualMachine::ACTIVE &&
+                ( vm->get_lcm_state() == VirtualMachine::RUNNING ||
+                  vm->get_lcm_state() == VirtualMachine::UNKNOWN ))
+            {
+                vm->log("VMM",Log::INFO, "VM running but monitor state is PAUSED.");
 
-            lcm->trigger(LifeCycleManager::MONITOR_SUSPEND, vm->get_oid());
+                lcm->trigger(LifeCycleManager::MONITOR_SUSPEND, vm->get_oid());
+            }
             break;
 
         case 'e': //Failed
-            vm->log("VMM",Log::INFO,"VM running but monitor state is ERROR.");
+            if ( vm->get_state() == VirtualMachine::ACTIVE &&
+                 vm->get_lcm_state() == VirtualMachine::RUNNING )
+            {
+                vm->log("VMM",Log::INFO,"VM running but monitor state is ERROR.");
 
-            lcm->trigger(LifeCycleManager::MONITOR_FAILURE, vm->get_oid());
+                lcm->trigger(LifeCycleManager::MONITOR_DONE, vm->get_oid());
+            }
             break;
 
         case 'd': //The VM was powered-off
-            vm->log("VMM",Log::INFO,"VM running but monitor state is POWEROFF");
-
-            lcm->trigger(LifeCycleManager::MONITOR_POWEROFF, vm->get_oid());
+            if ( vm->get_state() == VirtualMachine::ACTIVE &&
+                ( vm->get_lcm_state() == VirtualMachine::RUNNING ||
+                  vm->get_lcm_state() == VirtualMachine::SHUTDOWN ||
+                  vm->get_lcm_state() == VirtualMachine::SHUTDOWN_POWEROFF ||
+                  vm->get_lcm_state() == VirtualMachine::SHUTDOWN_UNDEPLOY ))
+            {
+                lcm->trigger(LifeCycleManager::MONITOR_POWEROFF, vm->get_oid());
+            }
             break;
     }
 }
@@ -709,104 +745,6 @@ void VirtualMachineManagerDriver::process_poll(
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int VirtualMachineManagerDriver::parse_vm_info(
-    const string       &monitor_str,
-    int                &cpu,
-    int                &memory,
-    long long          &net_tx,
-    long long          &net_rx,
-    char               &state,
-    map<string,string> &custom)
-{
-    istringstream is;
-
-    int    parse_result = 0;
-    size_t pos;
-
-    string tmp;
-    string var;
-
-    ostringstream   os;
-    istringstream   tiss;
-
-    cpu    = -1;
-    memory = -1;
-    net_tx = -1;
-    net_rx = -1;
-    state  = '-';
-
-    custom.clear();
-
-    is.str(monitor_str);
-
-    while(parse_result == 0 && is.good())
-    {
-        is >> tmp >> ws;
-
-        pos = tmp.find('=');
-
-        if ( pos == string::npos )
-        {
-            parse_result = -1;
-            continue;
-        }
-
-        tmp.replace(pos,1," ");
-
-        tiss.clear();
-
-        tiss.str(tmp);
-
-        tiss >> var >> ws;
-
-        if (!tiss.good())
-        {
-            parse_result = -1;
-            continue;
-        }
-
-        if (var == "USEDMEMORY")
-        {
-            tiss >> memory;
-        }
-        else if (var == "USEDCPU")
-        {
-            tiss >> fixed >> cpu;
-        }
-        else if (var == "NETRX")
-        {
-            tiss >> net_rx;
-        }
-        else if (var == "NETTX")
-        {
-            tiss >> net_tx;
-        }
-        else if (var == "STATE")
-        {
-            tiss >> state;
-        }
-        else if (!var.empty())
-        {
-            string val;
-
-            tiss >> val;
-
-            custom.insert(make_pair(var, val));
-        }
-
-        if (tiss.fail())
-        {
-            parse_result = -1;
-            continue;
-        }
-    }
-
-    return parse_result;
-}
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
 void VirtualMachineManagerDriver::recover()
 {
     NebulaLog::log("VMM",Log::INFO,"Recovering VMM drivers");
