@@ -1,7 +1,7 @@
 #!/usr/bin/env ruby
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -158,6 +158,18 @@ class DummyDriver < VirtualMachineDriver
         send_message(ACTION[:snapshot_delete], result, id)
     end
 
+    def disk_snapshot_create(id, drv_message)
+        result = retrieve_result("disk_snapshot_create")
+
+        send_message(ACTION[:disk_snapshot_create], result, id, "dummy-snap")
+    end
+
+    def disk_snapshot_revert(id, drv_message)
+        result = retrieve_result("disk_snapshot_revert")
+
+        send_message(ACTION[:disk_snapshot_revert], result, id)
+    end
+
     def cleanup(id, drv_message)
         result = retrieve_result("cleanup")
 
@@ -180,13 +192,13 @@ class DummyDriver < VirtualMachineDriver
         end
 
         prev_nettx = 0
-        if msg.elements["VM/NET_TX"]
-            prev_nettx = msg.elements["VM/NET_TX"].text.to_i
+        if msg.elements["VM/MONITORING/NETTX"]
+            prev_nettx = msg.elements["VM/MONITORING/NETTX"].text.to_i
         end
 
         prev_netrx = 0
-        if msg.elements["VM/NET_RX"]
-            prev_netrx = msg.elements["VM/NET_RX"].text.to_i
+        if msg.elements["VM/MONITORING/NETRX"]
+            prev_netrx = msg.elements["VM/MONITORING/NETRX"].text.to_i
         end
 
         # monitor_info: string in the form "VAR=VAL VAR=VAL ... VAR=VAL"
@@ -194,8 +206,11 @@ class DummyDriver < VirtualMachineDriver
         monitor_info = "#{POLL_ATTRIBUTE[:state]}=#{VM_STATE[:active]} " \
                        "#{POLL_ATTRIBUTE[:nettx]}=#{prev_nettx+(50*rand(3))} " \
                        "#{POLL_ATTRIBUTE[:netrx]}=#{prev_netrx+(100*rand(4))} " \
-                       "#{POLL_ATTRIBUTE[:usedmemory]}=#{max_memory * (rand(80)+20)/100} " \
-                       "#{POLL_ATTRIBUTE[:usedcpu]}=#{max_cpu * (rand(95)+5)/100}"
+                       "#{POLL_ATTRIBUTE[:memory]}=#{max_memory * (rand(80)+20)/100} " \
+                       "#{POLL_ATTRIBUTE[:cpu]}=#{max_cpu * (rand(95)+5)/100} " \
+                       "#{POLL_ATTRIBUTE[:disk_size]}=[ID=0, SIZE=#{rand(1024)}] "\
+                       "#{POLL_ATTRIBUTE[:disk_size]}=[ID=1, SIZE=#{rand(1024)}] "\
+                       "#{POLL_ATTRIBUTE[:snapshot_size]}=[ID=0, DISK_ID=0, SIZE=#{rand(1024)}] "
 
         send_message(ACTION[:poll],result,id,monitor_info)
     end
