@@ -1,7 +1,7 @@
 #!/usr/bin/env ruby
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -44,7 +44,12 @@ require 'pp'
 
 class VmmAction
     # List of xpaths required by the VNM driver actions
-    XPATH_LIST = %w(ID DEPLOY_ID TEMPLATE/NIC HISTORY_RECORDS/HISTORY/HOSTNAME)
+    XPATH_LIST = %w(
+        ID DEPLOY_ID
+        TEMPLATE/NIC
+        TEMPLATE/SECURITY_GROUP_RULE
+        HISTORY_RECORDS/HISTORY/HOSTNAME
+    )
 
     attr_reader :data
 
@@ -473,9 +478,11 @@ class ExecDriver < VirtualMachineDriver
         action = VmmAction.new(self, id, :migrate, drv_message)
         pre    = "PRE"
         post   = "POST"
+        failed = "FAIL"
 
         pre  << action.data[:tm_command] << " " << action.data[:vm]
         post << action.data[:tm_command] << " " << action.data[:vm]
+        failed << action.data[:tm_command] << " " << action.data[:vm]
 
         steps=[
             # Execute a pre-migrate TM setup
@@ -494,7 +501,15 @@ class ExecDriver < VirtualMachineDriver
             {
                 :driver     => :vmm,
                 :action     => :migrate,
-                :parameters => [:deploy_id, :dest_host, :host]
+                :parameters => [:deploy_id, :dest_host, :host],
+                :fail_actions => [
+                    {
+                        :driver     => :tm,
+                        :action     => :tm_failmigrate,
+                        :parameters => failed.split,
+                        :no_fail    => true
+                    }
+                ]
             },
             # Execute networking clean up operations
             # NOTE: VM is now in the new host. If we fail from now on, oned will
@@ -572,8 +587,6 @@ class ExecDriver < VirtualMachineDriver
 
         target_index = target.downcase[-1..-1].unpack('c').first - 97
 
-
-
         action = VmmAction.new(self, id, :attach_disk, drv_message)
 
         # Bug #1355, argument character limitation in ESX
@@ -656,7 +669,6 @@ class ExecDriver < VirtualMachineDriver
     # SNAPSHOTCREATE action, creates a new system snapshot
     #
     def snapshot_create(id, drv_message)
-        action   = ACTION[:snapshot_create]
         xml_data = decode(drv_message)
 
         host      = xml_data.elements['HOST'].text
@@ -676,7 +688,6 @@ class ExecDriver < VirtualMachineDriver
     # SNAPSHOTREVERT action, reverts to a system snapshot
     #
     def snapshot_revert(id, drv_message)
-        action   = ACTION[:snapshot_revert]
         xml_data = decode(drv_message)
 
         host      = xml_data.elements['HOST'].text
@@ -696,7 +707,6 @@ class ExecDriver < VirtualMachineDriver
     # SNAPSHOTDELETE action, deletes a system snapshot
     #
     def snapshot_delete(id, drv_message)
-        action   = ACTION[:snapshot_delete]
         xml_data = decode(drv_message)
 
         host      = xml_data.elements['HOST'].text
@@ -716,7 +726,6 @@ class ExecDriver < VirtualMachineDriver
     # CLEANUP action, frees resources allocated in a host: VM and disk images
     #
     def cleanup(id, drv_message)
-        aname    = ACTION[:cleanup]
         xml_data = decode(drv_message)
 
         tm_command = xml_data.elements['TM_COMMAND'].text
@@ -786,9 +795,12 @@ class ExecDriver < VirtualMachineDriver
 
         begin
             source = xml_data.elements["VM/TEMPLATE/NIC[ATTACH='YES']/BRIDGE"]
+            source_ovs =
+                xml_data.elements["VM/TEMPLATE/NIC[ATTACH='YES']/BRIDGE_OVS"]
             mac    = xml_data.elements["VM/TEMPLATE/NIC[ATTACH='YES']/MAC"]
 
             source = source.text.strip
+            source_ovs = source_ovs.text.strip if source_ovs
             mac    = mac.text.strip
         rescue
             send_message(action, RESULT[:failure], id,
@@ -809,6 +821,8 @@ class ExecDriver < VirtualMachineDriver
         net_drv = net_drv.strip if !net_drv.nil?
         net_drv = "-" if net_drv.nil?
 
+        source = source_ovs if net_drv == 'ovswitch' && source_ovs
+
         action = VmmAction.new(self, id, :attach_nic, drv_message)
 
         steps=[
@@ -875,6 +889,204 @@ class ExecDriver < VirtualMachineDriver
         action.run(steps)
     end
 
+    #
+    # DISKSNAPSHOTCREATE action, takes a snapshot of a disk
+    #
+    def disk_snapshot_create(id, drv_message)
+        snap_action  = prepare_snap_action(id, drv_message,
+                                           :disk_snapshot_create)
+        action       = snap_action[:action]
+        strategy     = snap_action[:strategy]
+        drv_message  = snap_action[:drv_message]
+        target       = snap_action[:target]
+        target_index = snap_action[:target_index]
+        xml_data     = snap_action[:xml_data]
+
+        # Get TM command
+        tm_command = ensure_xpath(xml_data, id, action, 'TM_COMMAND') || return
+
+        # Build the process
+        case strategy
+        when :live
+            tm_command_split = tm_command.split
+            tm_command_split[0] += "_LIVE"
+
+            steps = [
+                {
+                    :driver     => :tm,
+                    :action     => :tm_snap_create_live,
+                    :parameters => tm_command_split
+                }
+            ]
+
+        when :detach
+            steps = [
+                # detach_disk or save
+                {
+                    :driver     => :vmm,
+                    :action     => :detach_disk,
+                    :parameters => [:deploy_id, :disk_target_path, target,
+                                    target_index]
+                },
+                # run TM
+                {
+                    :driver     => :tm,
+                    :action     => :tm_snap_create,
+                    :parameters => tm_command.split,
+                    :no_fail    => true
+                },
+                # attach_disk or restore
+                {
+                    :driver     => :vmm,
+                    :action     => :attach_disk,
+                    :parameters => [:deploy_id, :disk_target_path, target,
+                                    target_index, drv_message]
+                }
+            ]
+        when :suspend
+               steps = [
+                # detach_disk or save
+                {
+                    :driver     => :vmm,
+                    :action     => :save,
+                    :parameters => [:deploy_id, :checkpoint_file, :host]
+                },
+                # network drivers (clean)
+                {
+                    :driver   => :vnm,
+                    :action   => :clean
+                },
+                # run TM
+                {
+                    :driver     => :tm,
+                    :action     => :tm_snap_create,
+                    :parameters => tm_command.split,
+                    :no_fail    => true
+                },
+                # network drivers (pre)
+                {
+                    :driver   => :vnm,
+                    :action   => :pre
+                },
+                # attach_disk or restore
+                {
+                    :driver     => :vmm,
+                    :action     => :restore,
+                    :parameters => [:checkpoint_file, :host, :deploy_id]
+                },
+                # network drivers (post)
+                {
+                    :driver       => :vnm,
+                    :action       => :post,
+                    :parameters   => [:deploy_id],
+                    :fail_actions => [
+                        {
+                            :driver     => :vmm,
+                            :action     => :cancel,
+                            :parameters => [:deploy_id, :host]
+                        }
+                    ]
+                }
+            ]
+        else
+            return
+        end
+
+        action.run(steps)
+    end
+
+    #
+    # DISKSNAPSHOTREVERT action, takes a snapshot of a disk
+    #
+    def disk_snapshot_revert(id, drv_message)
+        snap_action  = prepare_snap_action(id, drv_message,
+                                           :disk_snapshot_revert)
+        action       = snap_action[:action]
+        strategy     = @options[:snapshots_strategy]
+        drv_message  = snap_action[:drv_message]
+        target       = snap_action[:target]
+        target_index = snap_action[:target_index]
+        xml_data     = snap_action[:xml_data]
+
+        # Get TM command
+        tm_command = ensure_xpath(xml_data, id, action, 'TM_COMMAND') || return
+
+        case strategy
+        when :detach
+            steps = [
+                # Save VM state / detach the disk
+                {
+                    :driver     => :vmm,
+                    :action     => :detach_disk,
+                    :parameters => [:deploy_id, :disk_target_path, target, target_index]
+                },
+                # Do the snapshot
+                {
+                    :driver     => :tm,
+                    :action     => :tm_snap_revert,
+                    :parameters => tm_command.split,
+                    :no_fail    => true,
+                },
+                # Restore VM / attach the disk
+                {
+                    :driver     => :vmm,
+                    :action     => :attach_disk,
+                    :parameters => [:deploy_id, :disk_target_path, target, target_index,
+                           drv_message]
+                }
+            ]
+        when :suspend
+            steps = [
+                # Save VM state / detach the disk
+                {
+                    :driver     => :vmm,
+                    :action     => :save,
+                    :parameters => [:deploy_id, :checkpoint_file, :host]
+                },
+                # network drivers (clean)
+                {
+                    :driver   => :vnm,
+                    :action   => :clean
+                },
+                # Do the snapshot
+                {
+                    :driver     => :tm,
+                    :action     => :tm_snap_revert,
+                    :parameters => tm_command.split,
+                    :no_fail    => true,
+                },
+                # network drivers (pre)
+                {
+                    :driver   => :vnm,
+                    :action   => :pre
+                },
+                # Restore VM / attach the disk
+                {
+                    :driver     => :vmm,
+                    :action     => :restore,
+                    :parameters => [:checkpoint_file, :host, :deploy_id]
+                },
+                # network drivers (post)
+                {
+                    :driver       => :vnm,
+                    :action       => :post,
+                    :parameters   => [:deploy_id],
+                    :fail_actions => [
+                        {
+                            :driver     => :vmm,
+                            :action     => :cancel,
+                            :parameters => [:deploy_id, :host]
+                        }
+                    ]
+                }
+            ]
+        else
+            return
+        end
+
+        action.run(steps)
+    end
+
 private
 
     def ensure_xpath(xml_data, id, action, xpath)
@@ -889,6 +1101,47 @@ private
         end
     end
 
+    def prepare_snap_action(id, drv_message, action)
+        xml_data = decode(drv_message)
+
+        # Make sure disk target has been defined
+        target_xpath = "VM/TEMPLATE/DISK[DISK_SNAPSHOT_ACTIVE='YES']/TARGET"
+        target       = ensure_xpath(xml_data, id, ACTION[action],
+                                    target_xpath) || return
+        target_index = target.downcase[-1..-1].unpack('c').first - 97
+
+        # Always send ATTACH='YES' for the selected target in case it will end
+        # up being a 'detach' strategy
+        disk   = xml_data.elements[target_xpath].parent
+        attach = REXML::Element.new('ATTACH')
+
+        attach.add_text('YES')
+        disk.add(attach)
+
+        drv_message = Base64.encode64(xml_data.to_s)
+        action = VmmAction.new(self, id, action, drv_message)
+
+        # Determine the strategy
+        vmm_driver_path = 'VM/HISTORY_RECORDS/HISTORY/VMMMAD'
+        tm_driver_path  = "VM/TEMPLATE/DISK[DISK_SNAPSHOT_ACTIVE='YES']/TM_MAD"
+
+        vmm_driver = ensure_xpath(xml_data, id, action, vmm_driver_path) || return
+        tm_driver  = ensure_xpath(xml_data, id, action, tm_driver_path)  || return
+
+        strategy = @options[:snapshots_strategy]
+        if @options[:live_snapshots] && LIVE_DISK_SNAPSHOTS.include?("#{vmm_driver}-#{tm_driver}")
+            strategy = :live
+        end
+
+        {
+            :action       => action,
+            :strategy     => strategy,
+            :drv_message  => drv_message,
+            :target       => target,
+            :target_index => target_index,
+            :xml_data     => xml_data
+        }
+    end
 end
 
 ################################################################################
@@ -897,34 +1150,48 @@ end
 #
 ################################################################################
 
+if ENV['LIVE_DISK_SNAPSHOTS']
+    LIVE_DISK_SNAPSHOTS = ENV['LIVE_DISK_SNAPSHOTS'].split
+else
+    LIVE_DISK_SNAPSHOTS = []
+end
+
 opts = GetoptLong.new(
-    [ '--retries',    '-r', GetoptLong::OPTIONAL_ARGUMENT ],
-    [ '--threads',    '-t', GetoptLong::OPTIONAL_ARGUMENT ],
-    [ '--local',      '-l', GetoptLong::REQUIRED_ARGUMENT ],
-    [ '--shell',      '-s', GetoptLong::REQUIRED_ARGUMENT ],
-    [ '--parallel',   '-p', GetoptLong::NO_ARGUMENT ]
+    [ '--retries',           '-r', GetoptLong::OPTIONAL_ARGUMENT ],
+    [ '--threads',           '-t', GetoptLong::OPTIONAL_ARGUMENT ],
+    [ '--local',             '-l', GetoptLong::REQUIRED_ARGUMENT ],
+    [ '--shell',             '-s', GetoptLong::REQUIRED_ARGUMENT ],
+    [ '--parallel',          '-p', GetoptLong::NO_ARGUMENT ],
+    [ '--live-snapshots',    '-i', GetoptLong::NO_ARGUMENT ],
+    [ '--default-snapshots', '-d', GetoptLong::REQUIRED_ARGUMENT ]
 )
 
-hypervisor      = ''
-retries         = 0
-threads         = 15
-shell           = 'bash'
-local_actions   = {}
-single_host     = true
+hypervisor         = ''
+retries            = 0
+threads            = 15
+shell              = 'bash'
+local_actions      = {}
+single_host        = true
+live_snapshots     = false
+snapshots_strategy = :suspend # Either :detach or :suspend
 
 begin
     opts.each do |opt, arg|
         case opt
             when '--retries'
-                retries   = arg.to_i
+                retries = arg.to_i
             when '--threads'
-                threads   = arg.to_i
+                threads = arg.to_i
             when '--local'
-                local_actions=OpenNebulaDriver.parse_actions_list(arg)
+                local_actions = OpenNebulaDriver.parse_actions_list(arg)
             when '--shell'
-                shell   = arg
+                shell = arg
             when '--parallel'
                 single_host = false
+            when '--default-snapshots'
+                snapshots_strategy = arg.to_sym
+            when '--live-snapshots'
+                live_snapshots = true
         end
     end
 rescue Exception => e
@@ -938,10 +1205,12 @@ else
 end
 
 exec_driver = ExecDriver.new(hypervisor,
-                :concurrency => threads,
-                :retries => retries,
-                :local_actions => local_actions,
-                :shell => shell,
-                :single_host => single_host)
+                :concurrency        => threads,
+                :retries            => retries,
+                :local_actions      => local_actions,
+                :shell              => shell,
+                :single_host        => single_host,
+                :snapshots_strategy => snapshots_strategy,
+                :live_snapshots     => live_snapshots)
 
 exec_driver.start_driver
