@@ -1,6 +1,6 @@
 #!/usr/bin/env ruby
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -138,6 +138,10 @@ class AzureDriver
 
         @public_cloud_az_conf  = YAML::load(File.read(AZ_DRIVER_CONF))
 
+        if @public_cloud_az_conf['proxy_uri']
+            ENV['HTTP_PROXY'] = @public_cloud_az_conf['proxy_uri']
+        end
+
         @instance_types = @public_cloud_az_conf['instance_types']
 
         regions = @public_cloud_az_conf['regions']
@@ -145,11 +149,11 @@ class AzureDriver
 
         # Sanitize region data
         if @region['pem_management_cert'].nil?
-            raise "pem_management_cert not defined for #{host}" 
+            raise "pem_management_cert not defined for #{host}"
         end
 
         if @region['subscription_id'].nil?
-            raise "subscription_id not defined for #{host}" 
+            raise "subscription_id not defined for #{host}"
         end
 
         # Set default endpoint if not declared
@@ -158,7 +162,6 @@ class AzureDriver
         end
 
         Azure.configure do |config|
-          # Configure these 3 properties to use Storage
           config.management_certificate = @region['pem_management_cert']
           config.subscription_id        = @region['subscription_id']
           config.management_endpoint    = @region['management_endpoint']
@@ -168,19 +171,19 @@ class AzureDriver
     end
 
     # DEPLOY action
-    def deploy(id, host, xml_text)
+    def deploy(id, host, xml_text, lcm_state, deploy_id)
+      if lcm_state == "BOOT" || lcm_state == "BOOT_FAILURE"
         load_default_template_values
 
         az_info = get_deployment_info(host, xml_text)
 
         if !az_value(az_info, 'IMAGE')
             STDERR.puts("Cannot find IMAGE in deployment file")
-            exit(-1)
         end
 
         csn = az_value(az_info, 'CLOUD_SERVICE')
 
-        csn = "OpenNebulaDefaultCloudServiceName-#{id}" if !csn
+        csn = "csn#{id}" if !csn
 
         create_params  = create_params(id,csn,az_info)
         create_options = create_options(id,csn,az_info)
@@ -193,21 +196,28 @@ class AzureDriver
           end
         rescue => e
             STDERR.puts(e.message)
-            exit(-1)
         end
 
         if instance.class == Azure::VirtualMachineManagement::VirtualMachine
             puts(instance.vm_name)
         else
             STDERR.puts(instance)
-            exit (-1)
         end
+      else
+        restore(deploy_id)
+        deploy_id
+      end
     end
 
     # Shutdown an Azure instance
-    def shutdown(deploy_id)
-        az_action(deploy_id, :shutdown)
-        az_action(deploy_id, :delete)
+    def shutdown(deploy_id, lcm_state)
+        case lcm_state
+          when "SHUTDOWN"
+            az_action(deploy_id, :shutdown)
+            az_action(deploy_id, :delete)
+          when "SHUTDOWN_POWEROFF", "SHUTDOWN_UNDEPLOY"
+            az_action(deploy_id, :shutdown)
+        end
     end
 
     # Reboot an Azure instance
@@ -225,7 +235,7 @@ class AzureDriver
         az_action(deploy_id, :shutdown)
     end
 
-    # Cancel an Azure instance
+    # Resume an Azure instance
     def restore(deploy_id)
         az_action(deploy_id, :start)
     end
@@ -237,7 +247,7 @@ class AzureDriver
     end
 
     # Get the info of all Aure instances. An Azure instance must have
-    # a name compliant with the "one-#####_csn" format, where ##### are intengers
+    # a name compliant with the "one-####_csn" format, where #### are intengers
     def monitor_all_vms
         totalmemory = 0
         totalcpu    = 0
@@ -264,16 +274,22 @@ class AzureDriver
             @azure_vms.list_virtual_machines.each do |vm|
                 poll_data=parse_poll(vm)
 
-                if vm.vm_name.start_with?('one-') and 
-                   vm.vm_name.match(/([^_]+)_(.+)/) and 
-                   vm.vm_name.match(/([^_]+)_(.+)/).size > 1
-                  
-                    one_id = vm.vm_name.match(/([^_]+)_(.+)/)[1].split("-")[1]
+                vm_template_to_one = vm_to_one(vm)
+                vm_template_to_one = Base64.encode64(vm_template_to_one)
+                vm_template_to_one = vm_template_to_one.gsub("\n","")
+
+                if vm.vm_name.start_with?('one-') and
+                   vm.vm_name.match(/([^_]+)-(.+)/) and
+                   vm.vm_name.match(/([^_]+)-(.+)/).size > 1
+
+                    one_id = vm.vm_name.match(/([^_]+)-(.+)/)[1].split("-")[1]
                 end
 
                 vms_info << "VM=[\n"
                 vms_info << "  ID=#{one_id || -1},\n"
-                vms_info << "  DEPLOY_ID=#{vm.vm_name},\n"
+                vms_info << "  DEPLOY_ID=#{vm.vm_name}-#{vm.cloud_service_name},\n"
+                vms_info << "  VM_NAME=#{vm.vm_name},\n"
+                vms_info << "  IMPORT_TEMPLATE=\"#{vm_template_to_one}\",\n"
                 vms_info << "  POLL=\"#{poll_data}\" ]\n"
 
                 if one_id
@@ -303,8 +319,8 @@ private
     # e.g. 800 for 8 cores) and memory (in KB)
     def instance_type_capacity(name)
         return 0, 0 if @instance_types[name].nil?
-        return @instance_types[name]['cpu'].to_i * 100 ,
-               @instance_types[name]['memory'].to_i * 1024 * 1024
+        return (@instance_types[name]['cpu'].to_f * 100).to_i ,
+               (@instance_types[name]['memory'].to_f * 1024 * 1024).to_i
     end
 
     # Get the Azure section of the template. If more than one Azure section
@@ -332,7 +348,7 @@ private
         if !az
               # If we don't find an Azure location, and ONE just
               # knows about one Azure location, let's use that
-              if all_az_elements.size == 1 and 
+              if all_az_elements.size == 1 and
                  all_az_elements[0].elements["TYPE"].text.downcase.eql? "azure"
                   az = all_az_elements[0]
               else
@@ -366,8 +382,9 @@ private
 
     # Retrive the vm information from the Azure instance
     def parse_poll(instance)
-        info =  "#{POLL_ATTRIBUTE[:usedmemory]}=0 " \
-                "#{POLL_ATTRIBUTE[:usedcpu]}=0 " \
+      begin
+        info =  "#{POLL_ATTRIBUTE[:memory]}=0 " \
+                "#{POLL_ATTRIBUTE[:cpu]}=0 " \
                 "#{POLL_ATTRIBUTE[:nettx]}=0 " \
                 "#{POLL_ATTRIBUTE[:netrx]}=0 "
 
@@ -378,10 +395,10 @@ private
             state = case instance.deployment_status
             when "Running", "Starting"
                 VM_STATE[:active]
-            when "Suspended", "Stopping", 
+            when "Suspended", "Stopping",
                 VM_STATE[:paused]
             else
-                VM_STATE[:deleted]
+                VM_STATE[:unknown]
             end
         end
         info << "#{POLL_ATTRIBUTE[:state]}=#{state} "
@@ -398,12 +415,18 @@ private
                     value_str = value
                 end
 
-                info << "AZ_#{key.to_s.upcase}=#{value_str.gsub("\"","")} "
+                info << "AZ_#{key.to_s.upcase}="
+                info << "\\\"#{value_str.gsub("\"","")}\\\" "
 
             end
         }
 
         info
+      rescue
+        # Unkown state if exception occurs retrieving information from
+        # an instance
+        "#{POLL_ATTRIBUTE[:state]}=#{VM_STATE[:unknown]} "
+      end
     end
 
     def format_endpoints(endpoints)
@@ -421,13 +444,14 @@ private
 
     def create_params(id,csn,az_info)
         params = {
-            # Name will always be 'one-<id>_<cloud_service_name>'
-            :vm_name => "one-#{id}_#{csn}",
+            # Name will always be 'one-<id>-<cloud_service_name>'
+            :vm_name => "one-#{id}-#{csn}",
             :vm_user => az_value(az_info, 'VM_USER'),
             :image => az_value(az_info, 'IMAGE'),
             :password => az_value(az_info, 'VM_PASSWORD'),
             :location => az_value(az_info, 'LOCATION')
         }.delete_if { |k, v| v.nil? }
+
     end
 
     def create_options(id,csn,az_info)
@@ -436,10 +460,10 @@ private
           :winrm_transport => az_value(az_info, 'WIN_RM'),
           :cloud_service_name => csn,
           :tcp_endpoints => az_value(az_info, 'TCP_ENDPOINTS'),
-       # TODO possibly taking the values from user template 
+       # TODO possibly taking the values from user template
        # and create temp files
        #   :private_key_file => 'c:/private_key.key',
-       #   :certificate_file => 'c:/certificate.pem', 
+       #   :certificate_file => 'c:/certificate.pem',
           :ssh_port => az_value(az_info, 'SSHPORT'),
           :vm_size => az_value(az_info, 'INSTANCE_TYPE'),
           :affinity_group_name => az_value(az_info, 'AFFINITY_GROUP'),
@@ -453,7 +477,10 @@ private
     # +deploy_id+: String, VM id in Azure
     # +az_action+: Symbol, one of the keys of the Azure hash constant (i.e :run)
     def az_action(deploy_id, az_action)
-        name, csn = deploy_id.match(/([^_]+)_(.+)/)[1..-1]
+        name, csn = deploy_id.match(/([^_]+)-(.+)/)[1..-1]
+
+        # Imported VMs do not start with one-
+        deploy_id = name if !name.start_with? "one-"
 
         begin
             in_silence do
@@ -529,24 +556,48 @@ private
           $stderr.reopen orig_stderr
         end
        retval
-    end    
+    end
 
-    # Retrive the instance from Azure. If OpenNebula asks for it, then the 
+    # Retrieve the instance from Azure. If OpenNebula asks for it, then the
     # vm_name must comply with the notation name_csn
-    def get_instance(vm_name)
+    def get_instance(deploy_id)
         begin
-            csn = vm_name.match(/([^_]+)_(.+)/)[2]
+            vm_name = deploy_id.match(/([^_]+)-(.+)/)[1]
+            csn     = deploy_id.match(/([^_]+)-(.+)/)[-1]
 
             instance = @azure_vms.get_virtual_machine(vm_name,csn)
             if instance
                 return instance
             else
-                raise "Instance #{vm_name} does not exist"
+                raise "Instance #{deploy_id} does not exist"
             end
         rescue => e
             STDERR.puts e.message
             exit(-1)
         end
     end
+
+    # Build template for importation
+    def vm_to_one(vm)
+        cpu, mem = instance_type_capacity(vm.role_size)
+
+        mem = mem.to_i / 1024 # Memory for templates expressed in MB
+        cpu = cpu.to_f / 100  # CPU expressed in units
+
+        str = "NAME   = \"Instance from #{vm.vm_name}\"\n"\
+              "CPU    = \"#{cpu}\"\n"\
+              "vCPU   = \"#{cpu.ceil}\"\n"\
+              "MEMORY = \"#{mem}\"\n"\
+              "HYPERVISOR = \"AZURE\"\n"\
+              "PUBLIC_CLOUD = [\n"\
+              "  TYPE  =\"azure\"\n"\
+              "]\n"\
+              "IMPORT_VM_ID    = \"#{vm.vm_name}-#{vm.cloud_service_name}\"\n"\
+              "SCHED_REQUIREMENTS=\"NAME=\\\"#{@host}\\\"\"\n"\
+              "DESCRIPTION = \"Instance imported from Azure, from instance"\
+              " #{vm.vm_name}\"\n"
+
+        str
+    end
 end
 
