@@ -1,6 +1,6 @@
 #!/usr/bin/env ruby
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -40,6 +40,7 @@ require 'CommandManager'
 require 'scripts_common'
 require 'rexml/document'
 require 'VirtualMachineDriver'
+require 'opennebula'
 
 # The main class for the EC2 driver
 class EC2Driver
@@ -124,6 +125,10 @@ class EC2Driver
                     :opt => 'security_groups',
                     :proc => lambda {|str| str.split(',')}
                 },
+                "SECURITYGROUPIDS" => {
+                    :opt => 'security_group_ids',
+                    :proc => lambda {|str| str.split(',')}
+                },
                 "AVAILABILITYZONE" => {
                     :opt => 'placement/availability_zone'
                 },
@@ -172,7 +177,7 @@ class EC2Driver
             :cmd => :create_tags,
             :args => {
                 "TAGS" => {
-                    :opt  => '-t',
+                    :opt  => 'tags',
                     :proc => lambda {|str|
                         hash = {}
                         str.split(',').each {|s|
@@ -198,12 +203,14 @@ class EC2Driver
         :ip_address,
         :subnet_id,
         :security_groups,
-        :instance_type
+        :instance_type,
+        :image_id
     ]
 
     # EC2 constructor, loads credentials and endpoint
-    def initialize(host)
-        @host = host
+    def initialize(host, host_id=nil)
+        @host    = host
+        @host_id = host_id
 
         public_cloud_ec2_conf  = YAML::load(File.read(EC2_DRIVER_CONF))
 
@@ -230,7 +237,8 @@ class EC2Driver
     end
 
     # DEPLOY action, also sets ports and ip if needed
-    def deploy(id, host, xml_text)
+    def deploy(id, host, xml_text, lcm_state, deploy_id)
+      if lcm_state == "BOOT" || lcm_state == "BOOT_FAILURE"
         ec2_info = get_deployment_info(host, xml_text)
 
         load_default_template_values
@@ -244,6 +252,29 @@ class EC2Driver
                 :min_count => 1,
                 :max_count => 1})
 
+        # The OpenNebula context will be only included if not USERDATA
+        #   is provided by the user
+        if !ec2_value(ec2_info, 'USERDATA')
+            xml = OpenNebula::XMLElement.new
+            xml.initialize_xml(xml_text, 'VM')
+
+            if xml.has_elements?('TEMPLATE/CONTEXT')
+                # Since there is only 1 level ',' will not be added
+                context_str = xml.template_like_str('TEMPLATE/CONTEXT')
+
+                if xml['TEMPLATE/CONTEXT/TOKEN'] == 'YES'
+                    # TODO use OneGate library
+                    token_str = generate_onegate_token(xml)
+                    if token_str
+                        context_str << "\nONEGATE_TOKEN=\"#{token_str}\""
+                    end
+                end
+
+                userdata_key = EC2[:run][:args]["USERDATA"][:opt]
+                opts[userdata_key] = Base64.encode64(context_str)
+            end
+        end
+
         begin
             instance = AWS.ec2.instances.create(opts)
         rescue => e
@@ -251,7 +282,8 @@ class EC2Driver
             exit(-1)
         end
 
-        tags = generate_options(:tags, ec2_info) || {}
+        tags = generate_options(:tags, ec2_info)['tags'] || {}
+
         tags['ONE_ID'] = id
         tags.each{ |key,value|
             begin
@@ -272,11 +304,20 @@ class EC2Driver
         end
 
         puts(instance.id)
+      else
+        restore(deploy_id)
+        deploy_id
+      end
     end
 
     # Shutdown a EC2 instance
-    def shutdown(deploy_id)
-        ec2_action(deploy_id, :terminate)
+    def shutdown(deploy_id, lcm_state)
+        case lcm_state
+            when "SHUTDOWN"
+                ec2_action(deploy_id, :terminate)
+            when "SHUTDOWN_POWEROFF", "SHUTDOWN_UNDEPLOY"
+                ec2_action(deploy_id, :stop)
+        end
     end
 
     # Reboot a EC2 instance
@@ -289,12 +330,12 @@ class EC2Driver
         ec2_action(deploy_id, :terminate)
     end
 
-    # Stop a EC2 instance
+    # Save a EC2 instance
     def save(deploy_id)
         ec2_action(deploy_id, :stop)
     end
 
-    # Cancel a EC2 instance
+    # Resumes a EC2 instance
     def restore(deploy_id)
         ec2_action(deploy_id, :start)
     end
@@ -302,7 +343,10 @@ class EC2Driver
     # Get info (IP, and state) for a EC2 instance
     def poll(id, deploy_id)
         i = get_instance(deploy_id)
-        puts parse_poll(i)
+        vm = OpenNebula::VirtualMachine.new_with_id(id, OpenNebula::Client.new)
+        cw_mon_time = vm["LAST_POLL"] ? vm["LAST_POLL"].to_i : Time.now.to_i
+        do_cw = (Time.now.to_i - cw_mon_time) >= 360
+        puts parse_poll(i, vm, do_cw, cw_mon_time)
     end
 
     # Get the info of all the EC2 instances. An EC2 instance must include
@@ -333,17 +377,43 @@ class EC2Driver
         usedcpu    = 0
         usedmemory = 0
 
+        # Build an array of VMs and last_polls for monitoring
+        vpool      = OpenNebula::VirtualMachinePool.new OpenNebula::Client.new
+        vpool.info
+        onevm_info = {}
+
+        # Get last cloudwatch monitoring time
+        host_obj    = OpenNebula::Host.new_with_id(@host_id,
+                                                  OpenNebula::Client.new)
+        host_obj.info
+        cw_mon_time = host_obj["/HOST/TEMPLATE/CWMONTIME"]
+
+        if !cw_mon_time
+            cw_mon_time = Time.now.to_i
+        else
+            cw_mon_time = cw_mon_time.to_i
+        end
+
+        do_cw = (Time.now.to_i - cw_mon_time) >= 360
+
+        vpool.each{|vm| onevm_info[vm.deploy_id] = vm }
+
         begin
             AWS.ec2.instances.each do |i|
                 next if i.status != :pending && i.status != :running
 
-                poll_data=parse_poll(i)
-
                 one_id = i.tags['ONE_ID']
 
+                poll_data=parse_poll(i, onevm_info[i.instance_id],do_cw, cw_mon_time)
+
+                vm_template_to_one = vm_to_one(i)
+                vm_template_to_one = Base64.encode64(vm_template_to_one).gsub("\n","")
+
                 vms_info << "VM=[\n"
                 vms_info << "  ID=#{one_id || -1},\n"
                 vms_info << "  DEPLOY_ID=#{i.instance_id},\n"
+                vms_info << "  VM_NAME=#{i.instance_id},\n"
+                vms_info << "  IMPORT_TEMPLATE=\"#{vm_template_to_one}\",\n"
                 vms_info << "  POLL=\"#{poll_data}\" ]\n"
 
                 if one_id
@@ -364,6 +434,12 @@ class EC2Driver
         host_info << "FREEMEMORY=#{(totalmemory - usedmemory).round}\n"
         host_info << "FREECPU=#{(totalcpu - usedcpu).round}\n"
 
+        if do_cw
+            host_info << "CWMONTIME=#{Time.now.to_i}"
+        else
+            host_info << "CWMONTIME=#{cw_mon_time}"
+        end
+
         puts host_info
         puts vms_info
     end
@@ -392,11 +468,11 @@ private
         # our desired host name
         all_ec2_elements.each { |element|
             cloud=element.elements["HOST"]
-            if cloud and cloud.text.upcase == host.upcase
+            if cloud && cloud.text.upcase == host.upcase
                 ec2 = element
             else
                 cloud=element.elements["CLOUD"]
-                if cloud and cloud.text.upcase == host.upcase
+                if cloud && cloud.text.upcase == host.upcase
                     ec2_deprecated = element
                 end
             end
@@ -419,13 +495,25 @@ private
         ec2
     end
 
-    # Retrive the vm information from the EC2 instance
-    def parse_poll(instance)
+    # Retrieve the vm information from the EC2 instance
+    def parse_poll(instance, onevm, do_cw, cw_mon_time)
         begin
-            info =  "#{POLL_ATTRIBUTE[:usedmemory]}=0 " \
-                    "#{POLL_ATTRIBUTE[:usedcpu]}=0 " \
-                    "#{POLL_ATTRIBUTE[:nettx]}=0 " \
-                    "#{POLL_ATTRIBUTE[:netrx]}=0 "
+            onevm.info
+            if onevm && do_cw
+              cloudwatch_str = cloudwatch_monitor_info(instance.instance_id,
+                                                       onevm,
+                                                       cw_mon_time)
+            else
+              previous_cpu   = onevm["/VM/MONITORING/CPU"] ? onevm["/VM/MONITORING/CPU"] : 0
+              previous_netrx = onevm["/VM/MONITORING/NETRX"] ? onevm["/VM/MONITORING/NETRX"] : 0
+              previous_nettx = onevm["/VM/MONITORING/NETTX"] ? onevm["/VM/MONITORING/NETTX"] : 0
+
+              cloudwatch_str = "CPU=#{previous_cpu} "\
+                               "NETTX=#{previous_nettx} "\
+                               "NETRX=#{previous_netrx} "
+            end
+
+            info =  "#{POLL_ATTRIBUTE[:memory]}=0 #{cloudwatch_str}"
 
             state = ""
             if !instance.exists?
@@ -566,5 +654,150 @@ private
             exit(-1)
         end
     end
+
+    # Build template for importation
+    def vm_to_one(instance)
+        cpu, mem = instance_type_capacity(instance.instance_type)
+
+        cpu  = cpu.to_f / 100
+        vcpu = cpu.ceil
+        mem  = mem.to_i / 1024 # Memory for templates expressed in MB
+
+        str = "NAME   = \"Instance from #{instance.id}\"\n"\
+              "CPU    = \"#{cpu}\"\n"\
+              "VCPU   = \"#{vcpu}\"\n"\
+              "MEMORY = \"#{mem}\"\n"\
+              "HYPERVISOR = \"ec2\"\n"\
+              "PUBLIC_CLOUD = [\n"\
+              "  TYPE  =\"ec2\",\n"\
+              "  AMI   =\"#{instance.image_id}\"\n"\
+              "]\n"\
+              "IMPORT_VM_ID    = \"#{instance.id}\"\n"\
+              "SCHED_REQUIREMENTS=\"NAME=\\\"#{@host}\\\"\"\n"\
+              "DESCRIPTION = \"Instance imported from EC2, from instance"\
+              " #{instance.id}, AMI #{instance.image_id}\"\n"
+
+        str
+    end
+
+    # Extract monitoring information from Cloud Watch
+    # CPU, NETTX and NETRX
+    def cloudwatch_monitor_info(id, onevm, cw_mon_time)
+        cw=AWS::CloudWatch::Client.new
+
+        # CPU
+        begin
+            cpu = get_cloudwatch_metric(cw,
+                                        "CPUUtilization",
+                                        cw_mon_time,
+                                        ["Average"],
+                                         "Percent",
+                                         id)
+            if cpu[:datapoints].size != 0
+                cpu = cpu[:datapoints][-1][:average]
+            else
+                cpu = 0
+            end
+            cpu = cpu.to_f.round(2).to_s
+        rescue => e
+            STDERR.puts(e.message)
+        end
+
+        # NETTX
+        nettx = 0
+        begin
+            nettx_dp = get_cloudwatch_metric(cw,
+                                             "NetworkOut",
+                                             cw_mon_time,
+                                             ["Sum"],
+                                             "Bytes",
+                                             id)[:datapoints]
+            previous_nettx = onevm["/VM/MONITORING/NETTX"]
+            nettx = previous_nettx ? previous_nettx.to_i : 0
+
+            nettx_dp.each{|dp|
+                nettx += dp[:sum].to_i
+            }
+        rescue => e
+            STDERR.puts(e.message)
+        end
+
+        # NETRX
+        netrx = 0
+        begin
+            netrx_dp = get_cloudwatch_metric(cw,
+                                             "NetworkIn",
+                                             cw_mon_time,
+                                             ["Sum"],
+                                             "Bytes",
+                                             id)[:datapoints]
+            previous_netrx = onevm["/VM/MONITORING/NETRX"]
+            netrx = previous_netrx ? previous_netrx.to_i : 0
+
+            netrx_dp.each{|dp|
+                netrx += dp[:sum].to_i
+            }
+        rescue => e
+            STDERR.puts(e.message)
+        end
+
+        "CPU=#{cpu.to_s} NETTX=#{nettx.to_s} NETRX=#{netrx.to_s} "
+    end
+
+    # Get metric from AWS/EC2 namespace from the last poll
+    def get_cloudwatch_metric(cw, metric_name, last_poll, statistics, units, id)
+       options={:namespace=>"AWS/EC2",
+                :metric_name=>metric_name,
+                :start_time=> (Time.at(last_poll.to_i)-65).iso8601,
+                :end_time=> (Time.now-60).iso8601,
+                :period=>60,
+                :statistics=>statistics,
+                :unit=>units,
+                :dimensions=>[{:name=>"InstanceId", :value=>id}]}
+
+        cw.get_metric_statistics(options)
+    end
+
+    # TODO move this method to a OneGate library
+    def generate_onegate_token(xml)
+        # Create the OneGate token string
+        vmid_str  = xml["ID"]
+        stime_str = xml["STIME"]
+        str_to_encrypt = "#{vmid_str}:#{stime_str}"
+
+        user_id = xml['TEMPLATE/CREATED_BY']
+
+        if user_id.nil?
+            STDERR.puts {"VMID:#{vmid} CREATED_BY not present" \
+                " in the VM TEMPLATE"}
+            return nil
+        end
+
+        user = OpenNebula::User.new_with_id(user_id,
+                                            OpenNebula::Client.new)
+        rc   = user.info
+
+        if OpenNebula.is_error?(rc)
+            STDERR.puts {"VMID:#{vmid} user.info" \
+                " error: #{rc.message}"}
+            return nil
+        end
+
+        token_password = user['TEMPLATE/TOKEN_PASSWORD']
+
+        if token_password.nil?
+            STDERR.puts {"VMID:#{vmid} TOKEN_PASSWORD not present"\
+                " in the USER:#{user_id} TEMPLATE"}
+            return nil
+        end
+
+        cipher = OpenSSL::Cipher::Cipher.new("aes-256-cbc")
+        cipher.encrypt
+        cipher.key = token_password
+        onegate_token = cipher.update(str_to_encrypt)
+        onegate_token << cipher.final
+
+        onegate_token_64 = Base64.encode64(onegate_token).chop
+    end
 end
 
