@@ -1,7 +1,6 @@
-#!/bin/bash
-
+#!/usr/bin/env ruby
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -87,7 +86,7 @@ class SLDriver
               },
               "BLOCKDEVICE" => {
                   :opt => 'blockDevices',
-                  :proc => lambda {|str| 
+                  :proc => lambda {|str|
                            [{:device=>0, :diskImage=>{:capacity=>str}}]}
 
               },
@@ -109,14 +108,14 @@ class SLDriver
               },
               "SSHKEYS" => {
                   :opt => 'sshKeys', # Array of SSH Keys ids
-                  :proc => lambda {|str| 
+                  :proc => lambda {|str|
                              sshArray = Array.new
                              str.split(",").each { |id|
-                                sshArray << {:id=> id} 
+                                sshArray << {:id=> id}
                              }
                              sshArray
                            }
-                           
+
               },
               "POSTSCRIPT" => {
                   :opt => 'postInstallScriptUri'
@@ -159,7 +158,7 @@ class SLDriver
     # SoftLayer constructor, loads credentials and endpoint
     def initialize(host)
         @host = host
-        
+
         public_cloud_sl_conf  = YAML::load(File.read(SL_DRIVER_CONF))
 
         @instance_types = public_cloud_sl_conf['instance_types']
@@ -172,12 +171,15 @@ class SLDriver
 
         client_cred[:endpoint_url]=@region['endpoint'] if @region['endpoint']
 
+        client_cred[:timeout] = public_cloud_sl_conf['timeout'] || 120
+
 
         @sl_client = SoftLayer::Client.new(client_cred)
     end
 
     # DEPLOY action
-    def deploy(id, host, xml_text)
+    def deploy(id, host, xml_text, lcm_state, deploy_id)
+      if lcm_state == "BOOT" || lcm_state == "BOOT_FAILURE"
         load_default_template_values
 
         sl_info = get_deployment_info(host, xml_text)
@@ -211,11 +213,20 @@ class SLDriver
         end
 
         puts(vgid)
+      else
+        restore(deploy_id)
+        deploy_id
+      end
     end
 
     # Shutdown a SoftLayer instance
-    def shutdown(deploy_id)
-        sl_action(deploy_id, :terminate)
+    def shutdown(deploy_id, lcm_state)
+        case lcm_state
+        when "SHUTDOWN"
+          sl_action(deploy_id, :terminate)
+        when "SHUTDOWN_POWEROFF", "SHUTDOWN_UNDEPLOY"
+          sl_action(deploy_id, :stop)
+        end
     end
 
     # Reboot a SoftLayer instance
@@ -234,7 +245,7 @@ class SLDriver
     end
 
     # Start a SoftLayer instance
-    def restore(deploy_id)0
+    def restore(deploy_id)
         sl_action(deploy_id, :start)
     end
 
@@ -268,7 +279,7 @@ class SLDriver
         vms = @sl_client['Account'].getVirtualGuests
 
         vms_info = "VM_POLL=YES\n"
-        
+
         usedcpu    = 0
         usedmemory = 0
 
@@ -277,7 +288,7 @@ class SLDriver
                 vm   = @sl_client['Virtual_Guest'].object_with_id(i['id'].to_i)
                 poll_data = parse_poll(vm)
                 # Retrieve Virtual Guest to consult tags
-                
+
                 tags = vm.getTagReferences
                 # We know that the first one is the ONE identifier
                 one_id=nil
@@ -286,8 +297,15 @@ class SLDriver
                 rescue
                 end
                 next if one_id=="one_id"
+
+                vm_template_to_one = vm_to_one(i, @host)
+                vm_template_to_one = Base64.encode64(vm_template_to_one)
+                vm_template_to_one = vm_template_to_one.gsub("\n","")
+
                 vms_info << "VM=[\n"
                 vms_info << "  ID=#{one_id || -1},\n"
+                vms_info << "  VM_NAME=#{i['fullyQualifiedDomainName']},\n"
+                vms_info << "  IMPORT_TEMPLATE=\"#{vm_template_to_one}\",\n"
                 vms_info << "  DEPLOY_ID=#{i['id']},\n"
                 vms_info << "  POLL=\"#{poll_data[0...-1]}\" ]\n"
 
@@ -324,7 +342,7 @@ private
                @instance_types[name]['memory'].to_i * 1024 * 1024
     end
 
-    # Get the SoftLayer section of the template. If more than one SoftLayer 
+    # Get the SoftLayer section of the template. If more than one SoftLayer
     # section, the DATACENTER element is used and matched with the host
     def get_deployment_info(host, xml_text)
         xml = REXML::Document.new xml_text
@@ -332,7 +350,7 @@ private
         sl              = nil
         all_sl_elements = xml.root.get_elements("//USER_TEMPLATE/PUBLIC_CLOUD")
 
-        all_sl_elements = all_sl_elements.select { |element| 
+        all_sl_elements = all_sl_elements.select { |element|
              element.elements["TYPE"].text.downcase.eql? "softlayer"
         }
 
@@ -379,8 +397,9 @@ private
 
     # Retrieve the VM information from the SoftLayer instance
     def parse_poll(vm)
-        info =  "#{POLL_ATTRIBUTE[:usedmemory]}=0 " \
-                "#{POLL_ATTRIBUTE[:usedcpu]}=0 " \
+      begin
+        info =  "#{POLL_ATTRIBUTE[:memory]}=0 " \
+                "#{POLL_ATTRIBUTE[:cpu]}=0 " \
                 "#{POLL_ATTRIBUTE[:nettx]}=0 " \
                 "#{POLL_ATTRIBUTE[:netrx]}=0 "
 
@@ -407,7 +426,7 @@ private
                             when "PAUSED"
                                 VM_STATE[:paused]
                             else
-                                VM_STATE[:deleted]
+                                VM_STATE[:unknown]
                     end
         end
         info << "#{POLL_ATTRIBUTE[:state]}=#{state} "
@@ -426,6 +445,11 @@ private
         info << "SL_CRED_USER=#{user} SL_CRED_PASSWORD=#{pwd}" if user and pwd
 
         info
+      rescue
+        # Unkown state if exception occurs retrieving information from
+        # an instance
+        "#{POLL_ATTRIBUTE[:state]}=#{VM_STATE[:unknown]} "
+      end
     end
 
     # Execute a SoftLayer command
@@ -510,7 +534,7 @@ private
     end
 
     # Load the default values that will be used to create a new instance, if
-    # not provided in the template. These values are defined in the 
+    # not provided in the template. These values are defined in the
     # SL_DRIVER_DEFAULT file
     def load_default_template_values
         @defaults = Hash.new
@@ -551,4 +575,27 @@ private
             exit(-1)
         end
     end
-end
\ No newline at end of file
+
+    # Build template for importation
+    def vm_to_one(i, host)
+        cpu    = i['maxCpu'].to_i * 100
+        mem    = i['maxMemory'].to_i * 1024
+
+        mem = mem.to_i / 1024 # Memory for templates expressed in MB
+        cpu = cpu.to_f / 100  # CPU expressed in units
+
+        str = "NAME   = \"#{i['fullyQualifiedDomainName']}\"\n"\
+              "CPU    = \"#{cpu}\"\n"\
+              "vCPU   = \"#{cpu.ceil}\"\n"\
+              "MEMORY = \"#{mem}\"\n"\
+              "HYPERVISOR = \"SOFTLAYER\"\n"\
+              "PUBLIC_CLOUD = [\n"\
+              "  TYPE  =\"SOFTLAYER\"\n"\
+              "]\n"\
+              "IMPORT_VM_ID    = \"#{i['id']}\"\n"\
+              "SCHED_REQUIREMENTS=\"NAME=\\\"#{@host}\\\"\"\n"\
+              "DESCRIPTION = \"Instance imported from SoftLayer, from instance"\
+              " #{i['id']}\"\n"
+        str
+    end
+end
