@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -175,6 +175,19 @@ int AddressRange::from_vattr(VectorAttribute *vattr, string& error_msg)
         vattr->replace("VLAN", "YES");
     }
 
+    /* ------------------------- Security Groups ---------------------------- */
+
+    value = vattr->vector_value("SECURITY_GROUPS");
+
+    if (value.empty())
+    {
+        security_groups.clear();
+    }
+    else
+    {
+        one_util::split_unique(value, ',', security_groups);
+    }
+
     /* ------------------------ AR Internal Data ---------------------------- */
 
     vattr->replace("AR_ID", id);
@@ -246,13 +259,28 @@ int AddressRange::update_attributes(
                 attr->vector_value("PARENT_NETWORK_AR_ID"));
     }
 
+    /* ------ Remove non-persistent attributes, generated in to_xml() ------ */
+
+    vup->remove("MAC_END");
+
+    vup->remove("IP_END");
+
+    vup->remove("IP6_ULA");
+
+    vup->remove("IP6_ULA_END");
+
+    vup->remove("IP6_GLOBAL");
+
+    vup->remove("IP6_GLOBAL_END");
+
     /* ----------------- restricted attributes ----------------- */
 
     if (keep_restricted && restricted_set)
     {
-        remove_all_except_restricted(attr);
+        VectorAttribute va_aux(*attr);
+        remove_all_except_restricted(&va_aux);
 
-        vup->merge(attr, true);
+        vup->merge(&va_aux, true);
     }
 
     /* ----------------- update known attributes ----------------- */
@@ -261,8 +289,6 @@ int AddressRange::update_attributes(
 
     if (vup->vector_value("SIZE", new_size) == 0)
     {
-        map<unsigned int, long long> itup;
-
         if (is_reservation && new_size != size)
         {
             error_msg = "The SIZE of a reservation cannot be changed.";
@@ -276,6 +302,8 @@ int AddressRange::update_attributes(
 
             return -1;
         }
+
+        next = 0;
     }
     else
     {
@@ -305,6 +333,15 @@ int AddressRange::update_attributes(
     vup->replace("GLOBAL_PREFIX", new_global);
     vup->replace("ULA_PREFIX", new_ula);
 
+    string value = vup->vector_value("SECURITY_GROUPS");
+
+    security_groups.clear();
+
+    if (!value.empty())
+    {
+        one_util::split_unique(value, ',', security_groups);
+    }
+
     /* Replace with the new attributes */
 
     attr->replace(vup->value());
@@ -342,6 +379,15 @@ int AddressRange::from_vattr_db(VectorAttribute *vattr)
 
     rc += attr_to_allocated(vattr->vector_value("ALLOCATED"));
 
+    value = vattr->vector_value("SECURITY_GROUPS");
+
+    security_groups.clear();
+
+    if (!value.empty())
+    {
+        one_util::split_unique(value, ',', security_groups);
+    }
+
     if (type == NONE)
     {
         rc = -1;
@@ -359,6 +405,10 @@ void AddressRange::to_xml(ostringstream &oss, const vector<int>& vms,
     const map<string,string>&          ar_attrs = attr->value();
     map<string,string>::const_iterator it;
 
+    int          rc;
+    unsigned int mac_end[2];
+    string       aux_st;
+
     bool all_vms = (vms.size() == 1 && vms[0] == -1);
     bool all_vns = (vns.size() == 1 && vns[0] == -1);
 
@@ -375,6 +425,48 @@ void AddressRange::to_xml(ostringstream &oss, const vector<int>& vms,
                 << "]]></"<< it->first << ">";
     }
 
+    mac_end[1] = mac[1];
+    mac_end[0] = (mac[0] + size - 1);
+
+    oss << "<MAC_END><![CDATA[" << mac_to_s(mac_end) << "]]></MAC_END>";
+
+    aux_st = attr->vector_value("IP");
+
+    if (aux_st != "")
+    {
+        unsigned int ip_i;
+
+        rc = ip_to_i(aux_st, ip_i);
+
+        if (rc == 0)
+        {
+            oss << "<IP_END><![CDATA[" << ip_to_s(ip_i + size - 1) << "]]></IP_END>";
+        }
+    }
+
+    if (type & 0x00000004)
+    {
+        string ip6_s;
+
+        if (ula6[1] != 0 || ula6[0] != 0 ) /* Unique Local Address */
+        {
+            ip6_to_s(ula6, mac, ip6_s);
+            oss << "<IP6_ULA><![CDATA[" << ip6_s << "]]></IP6_ULA>";
+
+            ip6_to_s(ula6, mac_end, ip6_s);
+            oss << "<IP6_ULA_END><![CDATA[" << ip6_s << "]]></IP6_ULA_END>";
+        }
+
+        if (global6[1] != 0 || global6[0] != 0 ) /* Glocal Unicast */
+        {
+            ip6_to_s(global6, mac, ip6_s);
+            oss << "<IP6_GLOBAL><![CDATA[" << ip6_s << "]]></IP6_GLOBAL>";
+
+            ip6_to_s(global6, mac_end, ip6_s);
+            oss << "<IP6_GLOBAL_END><![CDATA[" << ip6_s << "]]></IP6_GLOBAL_END>";
+        }
+    }
+
     oss << "<USED_LEASES>" << used_addr << "</USED_LEASES>";
 
     if (allocated.empty())
@@ -489,6 +581,42 @@ int AddressRange::mac_to_i(string mac, unsigned int i_mac[]) const
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+string AddressRange::mac_to_s(const unsigned int i_mac[]) const
+{
+    ostringstream oss;
+    unsigned int  temp_byte;
+
+    for (int i=5;i>=0;i--)
+    {
+        if ( i < 4 )
+        {
+            temp_byte = i_mac[0];
+            temp_byte >>= i*8;
+        }
+        else
+        {
+            temp_byte = i_mac[1];
+            temp_byte >>= (i%4)*8;
+        }
+
+        temp_byte &= 255;
+
+        oss.width(2);
+        oss.fill('0');
+        oss << hex << temp_byte;
+
+        if(i!=0)
+        {
+            oss << ":";
+        }
+    }
+
+    return oss.str();
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 int AddressRange::ip_to_i(const string& _ip, unsigned int& i_ip) const
 {
     istringstream iss;
@@ -532,6 +660,31 @@ int AddressRange::ip_to_i(const string& _ip, unsigned int& i_ip) const
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+string AddressRange::ip_to_s(unsigned int i_ip) const
+{
+    ostringstream oss;
+    unsigned int  temp_byte;
+
+    for (int index=0;index<4;index++)
+    {
+        temp_byte =   i_ip;
+        temp_byte >>= (24-index*8);
+        temp_byte &=  255;
+
+        oss << temp_byte;
+
+        if(index!=3)
+        {
+            oss << ".";
+        }
+    }
+
+    return oss.str();
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 int AddressRange::prefix6_to_i(const string& prefix, unsigned int ip[]) const
 {
     struct in6_addr s6;
@@ -555,74 +708,13 @@ int AddressRange::prefix6_to_i(const string& prefix, unsigned int ip[]) const
     return 0;
 }
 
-/* ************************************************************************** */
-/* ************************************************************************** */
-
-void AddressRange::set_mac(unsigned int addr_index, VectorAttribute * nic) const
-{
-    ostringstream oss;
-    unsigned int  temp_byte;
-
-    for (int i=5;i>=0;i--)
-    {
-        if ( i < 4 )
-        {
-            temp_byte = mac[0] + addr_index;
-            temp_byte >>= i*8;
-        }
-        else
-        {
-            temp_byte = mac[1];
-            temp_byte >>= (i%4)*8;
-        }
-
-        temp_byte &= 255;
-
-        oss.width(2);
-        oss.fill('0');
-        oss << hex << temp_byte;
-
-        if(i!=0)
-        {
-            oss << ":";
-        }
-    }
-
-     nic->replace("MAC", oss.str());
-}
-
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-void AddressRange::set_ip(unsigned int addr_index, VectorAttribute * nic) const
-{
-    ostringstream oss;
-    unsigned int  temp_byte;
-
-    for (int index=0;index<4;index++)
-    {
-        temp_byte =   ip + addr_index;
-        temp_byte >>= (24-index*8);
-        temp_byte &=  255;
-
-        oss << temp_byte;
-
-        if(index!=3)
-        {
-            oss << ".";
-        }
-    }
-
-    nic->replace("IP", oss.str());
-}
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-void AddressRange::set_ip6(unsigned int addr_index, VectorAttribute * nic) const
+int AddressRange::ip6_to_s(const unsigned int prefix[], const unsigned int mac[], string& ip6_s) const
 {
     unsigned int eui64[2];
-    unsigned int mlow = mac[0] + addr_index;
+    unsigned int mlow = mac[0];
 
     struct in6_addr ip6;
     char dst[INET6_ADDRSTRLEN];
@@ -633,34 +725,71 @@ void AddressRange::set_ip6(unsigned int addr_index, VectorAttribute * nic) const
     ip6.s6_addr32[2] = htonl(eui64[1]);
     ip6.s6_addr32[3] = htonl(eui64[0]);
 
-    /* Link Local */
-    ip6.s6_addr32[0] = htonl(0xfe800000);
-    ip6.s6_addr32[1] = 0;
+    ip6.s6_addr32[0] = htonl(prefix[1]);
+    ip6.s6_addr32[1] = htonl(prefix[0]);
 
     if ( inet_ntop(AF_INET6, &ip6, dst, INET6_ADDRSTRLEN) != 0 )
     {
-        nic->replace("IP6_LINK", dst);
+        ip6_s = dst;
+        return 0;
+    }
+
+    return -1;
+}
+
+/* ************************************************************************** */
+/* ************************************************************************** */
+
+void AddressRange::set_mac(unsigned int addr_index, VectorAttribute * nic) const
+{
+    unsigned int new_mac[2];
+
+    new_mac[0] = mac[0] + addr_index;
+    new_mac[1] = mac[1];
+
+    nic->replace("MAC", mac_to_s(new_mac));
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+void AddressRange::set_ip(unsigned int addr_index, VectorAttribute * nic) const
+{
+    nic->replace("IP", ip_to_s(ip + addr_index));
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+void AddressRange::set_ip6(unsigned int addr_index, VectorAttribute * nic) const
+{
+    unsigned int new_mac[2];
+    string       ip6_s;
+
+    new_mac[0] = mac[0] + addr_index;
+    new_mac[1] = mac[1];
+
+    /* Link Local */
+    unsigned int local_prefix[2] = {0, 0xfe800000};
+
+    if (ip6_to_s(local_prefix, new_mac, ip6_s) == 0)
+    {
+        nic->replace("IP6_LINK", ip6_s);
     }
 
     if (ula6[1] != 0 || ula6[0] != 0 ) /* Unique Local Address */
     {
-        ip6.s6_addr32[0] = htonl(ula6[1]);
-        ip6.s6_addr32[1] = htonl(ula6[0]);
-
-        if ( inet_ntop(AF_INET6, &ip6, dst, INET6_ADDRSTRLEN) != 0 )
+        if (ip6_to_s(ula6, new_mac, ip6_s) == 0)
         {
-            nic->replace("IP6_ULA", dst);
+            nic->replace("IP6_ULA", ip6_s);
         }
     }
 
-    if (global6[1] != 0 || global6[0] != 0 ) /* Glocal Unicast */
+    if (global6[1] != 0 || global6[0] != 0 ) /* Global Unicast */
     {
-        ip6.s6_addr32[0] = htonl(global6[1]);
-        ip6.s6_addr32[1] = htonl(global6[0]);
-
-        if ( inet_ntop(AF_INET6, &ip6, dst, INET6_ADDRSTRLEN) != 0 )
+        if (ip6_to_s(global6, new_mac, ip6_s) == 0)
         {
-            nic->replace("IP6_GLOBAL", dst);
+            nic->replace("IP6_GLOBAL", ip6_s);
         }
     }
 }
@@ -951,12 +1080,12 @@ int AddressRange::free_addr(PoolObjectSQL::ObjectType ot, int obid,
 
     unsigned int index = mac_i[0] - mac[0];
 
-    if ((0 <= index) && (index < size))
+    if ( index < 0)
     {
-        return free_addr(ot, obid, index);
+        return -1;
     }
 
-    return -1;
+    return free_addr(ot, obid, index);
 }
 
 /* -------------------------------------------------------------------------- */
@@ -1070,6 +1199,30 @@ int AddressRange::free_addr_by_range(PoolObjectSQL::ObjectType ot, int obid,
     return freed;
 }
 
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+const char * AddressRange::SG_RULE_ATTRIBUTES[] = {
+    "AR_ID",
+    "TYPE",
+    "SIZE",
+    "MAC",
+    "IP"};
+
+const int  AddressRange::NUM_SG_RULE_ATTRIBUTES = 5;
+
+void AddressRange::process_security_rule(VectorAttribute * rule)
+{
+    for ( int i = 0; i < NUM_SG_RULE_ATTRIBUTES; i++ )
+    {
+        string st = attr->vector_value(SG_RULE_ATTRIBUTES[i]);
+
+        if ( st != "" )
+        {
+            rule->replace(SG_RULE_ATTRIBUTES[i], st);
+        }
+    }
+}
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
