@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -105,8 +105,12 @@ int VirtualNetwork::insert(SqlDB * db, string& error_str)
     vector<Attribute *> ars;
     ostringstream       ose;
 
+    string sg_str;
+
     bool b_vlan;
 
+    int rc, num_ars;
+
     //--------------------------------------------------------------------------
     // VirtualNetwork Attributes from the template
     //--------------------------------------------------------------------------
@@ -182,13 +186,42 @@ int VirtualNetwork::insert(SqlDB * db, string& error_str)
     // Get the Address Ranges
     //--------------------------------------------------------------------------
 
-    remove_template_attribute("AR", ars);
+    num_ars = remove_template_attribute("AR", ars);
+    rc      = add_var(ars, error_str);
 
-    if (add_var(ars, error_str) != 0)
+    for (int i=0; i < num_ars; i++)
+    {
+        delete ars[i];
+    }
+
+    if ( rc != 0)
     {
         goto error_ar;
     }
 
+    //--------------------------------------------------------------------------
+    // Add default Security Group
+    //--------------------------------------------------------------------------
+
+    erase_template_attribute("SECURITY_GROUPS", sg_str);
+
+    if (sg_str.empty())
+    {
+        sg_str = "0";
+    }
+    else
+    {
+        set<int> sgs;
+
+        sg_str.append(",0");
+
+        one_util::split_unique(sg_str, ',', sgs);
+
+        sg_str = one_util::join(sgs.begin(), sgs.end(), ',');
+    }
+
+    add_template_attribute("SECURITY_GROUPS", sg_str);
+
     //--------------------------------------------------------------------------
     // Insert the Virtual Network
     //--------------------------------------------------------------------------
@@ -225,6 +258,7 @@ int VirtualNetwork::post_update_template(string& error)
 {
     string new_bridge;
     bool   b_vlan;
+    string sg_str;
 
     /* ---------------------------------------------------------------------- */
     /* Update Configuration Attributes (class & template)                     */
@@ -232,6 +266,7 @@ int VirtualNetwork::post_update_template(string& error)
     /*  - VLAN_ID                                                             */
     /*  - VLAN                                                                */
     /*  - BRIDGE                                                              */
+    /*  - SECURITY_GROUPS                                                     */
     /* ---------------------------------------------------------------------- */
     erase_template_attribute("PHYDEV", phydev);
 
@@ -263,6 +298,12 @@ int VirtualNetwork::post_update_template(string& error)
 
     add_template_attribute("BRIDGE", bridge);
 
+    security_groups.clear();
+
+    obj_template->get("SECURITY_GROUPS", sg_str);
+
+    one_util::split_unique(sg_str, ',', security_groups);
+
     return 0;
 }
 
@@ -478,6 +519,13 @@ int VirtualNetwork::from_xml(const string &xml_str)
     ObjectXML::free_nodes(content);
     content.clear();
 
+    //Security groups internal attribute (from /VNET/TEMPLATE/SECURITY_GROUPS)
+    string sg_str;
+
+    obj_template->get("SECURITY_GROUPS", sg_str);
+
+    one_util::split_unique(sg_str, ',', security_groups);
+
     // Address Range Pool
     ObjectXML::get_nodes("/VNET/AR_POOL", content);
 
@@ -486,7 +534,7 @@ int VirtualNetwork::from_xml(const string &xml_str)
         return -1;
     }
 
-    // Virtual Network template
+    // Address Ranges of the Virtual Network
     rc += ar_pool.from_xml_node(content[0]);
 
     ObjectXML::free_nodes(content);
@@ -510,6 +558,9 @@ int VirtualNetwork::nic_attribute(
     string inherit_val;
     vector<string>::const_iterator it;
 
+    set<int> nic_sgs;
+    int      ar_id;
+
     //--------------------------------------------------------------------------
     //  Set default values from the Virtual Network
     //--------------------------------------------------------------------------
@@ -573,12 +624,40 @@ int VirtualNetwork::nic_attribute(
         rc = allocate_addr(vid, nic, inherit_attrs);
     }
 
+    //--------------------------------------------------------------------------
+    //  Copy the security group IDs
+    //--------------------------------------------------------------------------
+
+    one_util::split_unique(nic->vector_value("SECURITY_GROUPS"), ',', nic_sgs);
+
+    nic_sgs.insert(security_groups.begin(), security_groups.end());
+
+    if (nic->vector_value("AR_ID", ar_id) == 0)
+    {
+        const set<int> ar_sgs = ar_pool.get_security_groups(ar_id);
+
+        nic_sgs.insert(ar_sgs.begin(), ar_sgs.end());
+    }
+
+    nic->replace("SECURITY_GROUPS",
+        one_util::join(nic_sgs.begin(), nic_sgs.end(), ','));
+
     return rc;
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+void VirtualNetwork::process_security_rule(
+        VectorAttribute *        rule,
+        vector<VectorAttribute*> &new_rules)
+{
+    ar_pool.process_security_rule(rule, new_rules);
+};
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 int VirtualNetwork::add_var(vector<Attribute *> &var, string& error_msg)
 {
     for (vector<Attribute *>::iterator it=var.begin(); it!=var.end(); it++)
