@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -15,7 +15,7 @@
 #--------------------------------------------------------------------------- #
 
 ################################################################################
-# The VNMMAD module provides the basic abstraction to implement custom 
+# The VNMMAD module provides the basic abstraction to implement custom
 # virtual network drivers. The VNMAD module includes:
 #   - VNMNetwork with base classes and main functionality to manage Virtual Nets
 #   - SGIPTables a module with a SG implementation based in iptables/ipset
@@ -43,7 +43,7 @@ module VNMMAD
                 @hypervisor = hypervisor
             end
 
-            @vm = VNMNetwork::VM.new(REXML::Document.new(vm_tpl).root, 
+            @vm = VNMNetwork::VM.new(REXML::Document.new(vm_tpl).root,
                 xpath_filter, deploy_id, @hypervisor)
         end
 
@@ -83,16 +83,17 @@ module VNMMAD
         def detect_hypervisor
             lsmod       = `#{VNMNetwork::COMMANDS[:lsmod]}`
             xen_file    = "/proc/xen/capabilities"
+            kvm_dir     = "/sys/class/misc/kvm"
 
             if File.exists?(xen_file)
                 "xen"
-            elsif lsmod.match(/kvm/)
+            elsif lsmod.match(/kvm/) || File.exists?(kvm_dir)
                 "kvm"
             else
                 nil
             end
         end
-        
+
         # Returns true if the template contains the deprecated firewall attributes:
         # - ICMP
         # - WHITE_PORTS_TCP
@@ -127,7 +128,7 @@ module VNMMAD
             else
                 cmd_str = "#{cmd}"
             end
-            
+
             return cmd_str
         end
     end
