@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -14,9 +14,9 @@
 # limitations under the License.                                             #
 #--------------------------------------------------------------------------- #
 
-require 'OpenNebulaNetwork'
+require 'vnmmad'
 
-class OpenvSwitchVLAN < OpenNebulaNetwork
+class OpenvSwitchVLAN < VNMMAD::VNMDriver
     DRIVER = "ovswitch"
 
     FIREWALL_PARAMS =  [:black_ports_tcp,
@@ -72,7 +72,13 @@ class OpenvSwitchVLAN < OpenNebulaNetwork
     def deactivate
         lock
 
+        attach_nic_id = @vm['TEMPLATE/NIC[ATTACH="YES"]/NIC_ID']
+
         process do |nic|
+            if attach_nic_id && attach_nic_id != nic[:nic_id]
+                next
+            end
+
             @nic = nic
 
             # Remove flows
@@ -91,7 +97,7 @@ class OpenvSwitchVLAN < OpenNebulaNetwork
     end
 
     def tag_vlan
-        cmd =  "#{COMMANDS[:ovs_vsctl]} set Port #{@nic[:tap]} "
+        cmd =  "#{command(:ovs_vsctl)} set Port #{@nic[:tap]} "
         cmd << "tag=#{vlan}"
 
         run cmd
@@ -100,7 +106,7 @@ class OpenvSwitchVLAN < OpenNebulaNetwork
     def tag_trunk_vlans
         range = @nic[:vlan_tagged_id]
         if range? range
-            ovs_vsctl_cmd = "#{COMMANDS[:ovs_vsctl]} set Port #{@nic[:tap]}"
+            ovs_vsctl_cmd = "#{command(:ovs_vsctl)} set Port #{@nic[:tap]}"
 
             cmd = "#{ovs_vsctl_cmd} trunks=#{range}"
             run cmd
@@ -159,7 +165,7 @@ class OpenvSwitchVLAN < OpenNebulaNetwork
     def del_flows
         in_port = ""
 
-        dump_flows = "#{COMMANDS[:ovs_ofctl]} dump-flows #{@nic[:bridge]}"
+        dump_flows = "#{command(:ovs_ofctl)} dump-flows #{@nic[:bridge]}"
         `#{dump_flows}`.lines do |flow|
             next unless flow.match("#{@nic[:mac]}")
             flow = flow.split.select{|e| e.match(@nic[:mac])}.first
@@ -175,13 +181,13 @@ class OpenvSwitchVLAN < OpenNebulaNetwork
     def add_flow(filter,action,priority=nil)
         priority = (priority.to_s.empty? ? "" : "priority=#{priority},")
 
-        run "#{COMMANDS[:ovs_ofctl]} add-flow " <<
+        run "#{command(:ovs_ofctl)} add-flow " <<
             "#{@nic[:bridge]} #{filter},#{priority}actions=#{action}"
     end
 
     def del_flow(filter)
         filter.gsub!(/priority=(\d+)/,"")
-        run "#{COMMANDS[:ovs_ofctl]} del-flows " <<
+        run "#{command(:ovs_ofctl)} del-flows " <<
             "#{@nic[:bridge]} #{filter}"
     end
 
@@ -192,7 +198,7 @@ class OpenvSwitchVLAN < OpenNebulaNetwork
     def port
         return @nic[:port] if @nic[:port]
 
-        dump_ports = `#{COMMANDS[:ovs_ofctl]} \
+        dump_ports = `#{command(:ovs_ofctl)} \
                       dump-ports #{@nic[:bridge]} #{@nic[:tap]}`
 
         @nic[:port] = dump_ports.scan(/^\s*port\s*(\d+):/).flatten.first
