@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -34,9 +34,9 @@ $: << RUBY_LIB_LOCATION
 
 require 'yaml'
 require 'CommandManager'
-require 'OpenNebulaNetwork'
+require 'vnmmad'
 
-class OpenNebulaVMware < OpenNebulaNetwork
+class OpenNebulaVMware < VNMMAD::VNMDriver
     DRIVER = "vmware"
 
     XPATH_FILTER = "TEMPLATE/NIC"
