@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -90,38 +90,59 @@ vector<string> ObjectXML::operator[] (const char * xpath_expr)
     xmlXPathObjectPtr obj;
     vector<string>    content;
 
+    ostringstream oss;
+    xmlNodePtr    cur;
+    xmlChar *     str_ptr;
+
     obj = xmlXPathEvalExpression(
         reinterpret_cast<const xmlChar *>(xpath_expr), ctx);
 
-    if (obj == 0 || obj->nodesetval == 0)
+    if (obj == 0)
     {
         return content;
     }
 
-    xmlNodeSetPtr ns = obj->nodesetval;
-    int           size = ns->nodeNr;
-    xmlNodePtr    cur;
-    xmlChar *     str_ptr;
-
-    for(int i = 0; i < size; ++i)
+    switch (obj->type)
     {
-        cur = ns->nodeTab[i];
+        case XPATH_NUMBER:
+            oss << obj->floatval;
 
-        if ( cur == 0 || cur->type != XML_ELEMENT_NODE )
-        {
-            continue;
-        }
+            content.push_back(oss.str());
+            break;
 
-        str_ptr = xmlNodeGetContent(cur);
+        case XPATH_NODESET:
+            for(int i = 0; i < obj->nodesetval->nodeNr ; ++i)
+            {
+                cur = obj->nodesetval->nodeTab[i];
 
-        if (str_ptr != 0)
-        {
-            string element_content = reinterpret_cast<char *>(str_ptr);
+                if ( cur == 0 || cur->type != XML_ELEMENT_NODE )
+                {
+                    continue;
+                }
 
-            content.push_back(element_content);
+                str_ptr = xmlNodeGetContent(cur);
+
+                if (str_ptr != 0)
+                {
+                    string element_content = reinterpret_cast<char *>(str_ptr);
+
+                    content.push_back(element_content);
+
+                    xmlFree(str_ptr);
+                }
+            }
+            break;
+
+        case XPATH_UNDEFINED:
+        case XPATH_BOOLEAN:
+        case XPATH_STRING:
+        case XPATH_POINT:
+        case XPATH_RANGE:
+        case XPATH_LOCATIONSET:
+        case XPATH_USERS:
+        case XPATH_XSLT_TREE:
+            break;
 
-            xmlFree(str_ptr);
-        }
     }
 
     xmlXPathFreeObject(obj);
@@ -564,7 +585,7 @@ int ObjectXML::validate_xml(const string &xml_doc)
 
 void ObjectXML::xml_parse(const string &xml_doc)
 {
-    xml = xmlParseMemory (xml_doc.c_str(),xml_doc.length());
+    xml = xmlReadMemory (xml_doc.c_str(),xml_doc.length(),0,0,XML_PARSE_HUGE);
 
     if (xml == 0)
     {
